 			//FMM.C file
#include "Lowest.h" 
#include "int3.h"
#include "mpole.h"
#include "FMM.h"
#include "System.h"
#include "cell.h"
#include <iomanip> // needed for setw
#include <fstream> // needed for ostream?


extern System *My;
class cell;

const int MAXLENFMM = 100;

FMM::FMM()//constructor for FMM, called at the file scope when MyFMM is created
{
  Parray = 0;  //to control output of the arrays
  if (My->Psetup)
     My->outlog << "in constructor for FMM, Parray = "<<Parray<<endl;

//use default values:
  aspectRatio[0]=aspectRatio[1]=aspectRatio[2]=-1;  //to catch errors
  MaxAngle2 = 0.5;
  calculateKernels = 0;
  calculateExterior = 0;
  draftFMM = 0;
  MaxOrder = 1;  //we should not want to use order higher than 16
  MaxOrder1 = MaxOrder + 1;

  if (My->Psetup)            
    My->outlog<<"finishing constructor...\n"<<endl;
}

FMM::~FMM()  //deallocates memory
{
  My->outlog<<"Destructor for FMM"<<endl;
  My->outlog<<"deallocating memory for arrays... ";
  delete[] NumBelow;
  delete[] NumOddBelow;
  delete[] OddOnes;
  delete[] RecurseAxis;
  delete[] RecurseFactor;
  delete[] RecurseMpole;

//specific for a system of uniformly magnetized cells

  for (int i = 0; i < NumComps; i++)
    delete[] ConvSum[i];
  delete[] ConvSum;
  delete[] IndArr;

  for (int i = 0; i < MaxOrder1; i++)
  {
    for (int j = 0; j < MaxOrder1; j++)
      delete[] index[i][j];
    delete[] index[i];
  }
  delete[] index;
  My->outlog<<"deallocated"<<endl;
}

//This function calculates some FMM arrays.
//This can't be done before, in Read_SetSizes, because AvgCoeff depends on
//the smallest cell size, determined in ParseBuild
void FMM::setup() 
{
  if (My->Psetup)
    My->outlog<<"starting FMM::setup, MaxOrder = "<<MaxOrder
      <<", MaxOrder1 = "<<MaxOrder1<<endl;

  NumOddBelow = new int[MaxOrder1];  //number of multipoles with odd order

  NumComps = NumBelow[MaxOrder]; //total number of components, will be used for
                                //allocating arrays in setup()

  scratchMpole1 = new mpole(-1);  //three levels of hierarchy in function calls
  scratchMpole2 = new mpole(-1); //example: cell::updateStat calls AddTaylor, which
                    //in its turn calls SetToDeriv. All of them use scratchMpoles

  Vshell = new mpole(-1);  //added by DMApf
  Vext = new mpole(-1);
  VextNew = new mpole(-1);

//initialize reducedSize:  //DMApf
  //find geometrical average:
  averageSize = 1.;
  int numOfPerDir = 0; //number of periodic directions
  for (int i=0; i<3; i++)
    if (My->periodic[i])
    {
      numOfPerDir++;
      averageSize*=My->PeriodVec[i];  //temporary storage for "volume"
    }
  
  averageSize = powf(averageSize, 1./numOfPerDir);
      
  for (int i=0; i<3; i++)
    if (My->periodic[i])
      reducedSize[i] = My->PeriodVec[i]/averageSize; 

  My->outlog<<"reducedSize is "<<reducedSize[0]<<" "<<reducedSize[1]
    <<" "<<reducedSize[2]<<endl;

  RecurseMpole = new int[NumComps];  //used for recursive computations
  RecurseAxis = new int[NumComps];  //0 for x, 1 for y, 2 for z
  RecurseFactor = new real[NumComps];  //1/n[RecurseAxis]
  for (int i = 0; i < NumComps; i++)   //to catch possible errors
    RecurseFactor[i] = RecurseAxis[i] = RecurseMpole[i] = -1;

  IndArr = new int3[NumComps];     //inverts index[nx][ny][nz]

  ConvSum = new int*[NumComps];
  for (int i = 0; i < NumComps; i++)
    ConvSum[i] = new int [NumComps];

  //allocating memory for index ... - sequential index i_n in paper
  index = new int**[MaxOrder1];
  for (int i = 0; i < MaxOrder1; i++)
  {
    index[i] = new int*[MaxOrder1];
    for (int j = 0; j < MaxOrder1; j++)
      index[i][j] = new int [MaxOrder1];
  }

  int *tempOddOnes = new int [NumComps];  //temporary storage for odd-order multipoles
      //initializing tempOddOnes to -1 (for debug)
  for ( int i = 0; i < NumComps; i++)
    tempOddOnes[i] = -1;  //to catch errors

//initializing index, IndArr and OddOnes
  int nxi, iOdd = 0, in;//dummies
  int i = 0; // indexes 3-tuple n 
  for (int order = 0; order < MaxOrder1; order++)// loop order 
    {
    for (int nzi = 0; nzi <= order; nzi++) 
    // loop index multiplicities, to add to order
      for (int nyi = 0; nyi <= order - nzi; nyi++)
      {
        nxi = order - nzi - nyi;
        IndArr[i][0] = nxi;    // store this index set 
        IndArr[i][1] = nyi;
        IndArr[i][2] = nzi;
        index[nxi][nyi][nzi] = i;
        if (order % 2)    //if this order is odd
            tempOddOnes[iOdd++] = i;  
            //store the i_n's that corresponds to odd order
        // Now figure out how to recursively calculate things.
        // from a lower index, if possible from (nxi-1,nyi,nzi):
        if (nxi) { RecurseMpole[i] = index[nxi-1][nyi][nzi];
           RecurseAxis[i] = 0; RecurseFactor[i] = 1.0/nxi;}
             //for products c^p, see paper
        else if(nyi) { RecurseMpole[i] = index[nxi][nyi-1][nzi];
           RecurseAxis[i] = 1; RecurseFactor[i] = 1.0/nyi;}
        else if(nzi) { RecurseMpole[i] = index[nxi][nyi][nzi-1];
           RecurseAxis[i] = 2; RecurseFactor[i] = 1.0/nzi;}
        if (Parray)
          My->outlog<<"i = "<<i<<" n_i = ( "<<IndArr[i][0]<<" "<<IndArr[i][1]
            <<" "<<IndArr[i][2]<<" )"<<endl;
        i++;
      }
     assert(NumBelow[order]==i);
     NumOddBelow[order] = iOdd;
  }


    //allocating memory for OddOnes, size is iOdd
  OddOnes = new int[iOdd];  //we know the number of components with odd order
  for (int i = 0; i<iOdd; i++)
    OddOnes[i] = tempOddOnes[i];  
    //copy the temporary array onto the permanent storage
  assert(tempOddOnes[iOdd] == -1);  //all the "odd components" have been coppied
  assert(tempOddOnes[iOdd-1]);    //the last "odd components" must be non-zero

  //delete temporary storage
  delete [] tempOddOnes; tempOddOnes = NULL;

   //initialize ConvSum
  for (int i = 0; i <NumComps; i++)
  {
    int3 ni(IndArr[i]);  //copy-constructor
    int n = ni.sum();  //order of ni
    int NumP = NumBelow[MaxOrder - n];
    for (int ip = 0; ip < NumP; ip++)
    {
      int3 tot(ni + IndArr[ip]);
      assert(tot.sum()<MaxOrder1);
      ConvSum [i][ip] = index[tot[0]][tot[1]][tot[2]];
    }//end for ip
  }  //end for i

//now compute Deriv. index arrays
  DerivTerms = new int[NumComps];
  DerivPower = new int*[NumComps];
  DerivPower [0] = new int[1];
  DerivCoef = new real*[NumComps];
  DerivCoef [0] = new real[1];
  DerivTerms[0] = 1;  //only one p
  DerivPower[0][0] = 0;
  DerivCoef[0][0] = 1/(4.*M_PI);
  for (int i = 1; i<NumComps; i++)
  {
    in = RecurseMpole[i];
    int n = (IndArr[in]).sum();
    scratchMpole1->SetToZero();//this will be F_n', where n' = n+x or n+y or n+z

    for (int j = 0; j < DerivTerms[in]; j++) //loop terms in equation for D
    {
      real term = DerivCoef[in][j];  //F_n[p] in paper
      int3 pVec = IndArr[DerivPower[in][j]];
      real term1 = term * pVec[RecurseAxis[i]];//like F_m*p_x in paper
      pVec[RecurseAxis[i]]+=1;  //now pVec is p+x in paper
      scratchMpole1->values[index[pVec[0]][pVec[1]][pVec[2]]]+=(term*(2*n+1)-term1);
      pVec[RecurseAxis[i]]-=1;  //restores the original pVec
      if (pVec[RecurseAxis[i]])
      {   //no contribution if pVec[RecurseAxis] == 0
        pVec[RecurseAxis[i]]-=1; 
        for (int axis = 0; axis < 3; axis++)
        {
          if(axis!=RecurseAxis[i])
          //need to loop only the other components, see paper
          {
            pVec[axis]+=2;
            scratchMpole1->values[index[pVec[0]][pVec[1]][pVec[2]]]-=term1;
            pVec[axis]-=2; //restore the old value
          }
        }
      }//end for if(pVec)
    }//end for j

//Now extract nonzero terms from scratchMpole1 
    int nterms = 0;  //count nonzero terms to allocate array
    for (int j = 0; j < NumComps; j++)
      if (fabs(scratchMpole1->values[j])) nterms++;
    DerivCoef[i] = new real [nterms];
    DerivPower[i] = new int [nterms];
    DerivTerms[i] = nterms;
//loop again to fill arrays
    nterms = 0;
    for (int j = 0; j < NumComps; j++)
    {
      if (fabs(scratchMpole1->values[j])>0.001) //if nonzero
      {
        DerivCoef[i][nterms] = scratchMpole1->values[j];
        DerivPower[i][nterms] = j;
        if(Parray)
        {
          print("n", IndArr[i]);
          print(", p ", IndArr[DerivPower[i][nterms]]);
          My->outlog<<", DerivCoef_n[p] = "<<DerivCoef[i][nterms]<<endl;
        }
        nterms++;
      }
    } //end for j
  } //end for i
  if (My->Psetup) {
     My->outlog<<"recursive initialization is finished"<<endl;
     My->outlog<<"****************************************************"<<endl;
  }

  if (My->Psetup)
    My->outlog<<"finishing setup"<<endl;

} //end of setup


//output the array of integers to outlog file; NO ENDLINE AT THE END
void FMM::print(char *title, const int array[], const int size)
{
  My->outlog <<title<<" = ";

  for (int i = 0; i < size; i++)
    My->outlog << array[i] <<' ';
  My->outlog<<flush;
}

//output the valarray to outlog file: "title = (1, 2, 3)" NO ENDLINE AT THE END
void FMM::print(char *title, const int3 &array)
{
  My->outlog <<title<<" = ( ";

  for (int i = 0; i < 3; i++)
    My->outlog << array[i] <<' ';
  My->outlog<<")"<<flush;
}


//added by DMAk
//calculate newKernels, put it to Kernels and return the maximum
//difference between them
real FMM::calcKernels()
{
 //KernelType gives the normal to the face that has non-zero charge
  for (int KernelType = 2; KernelType >= 0; KernelType--)
  {
    cout<<"in calcKernels, KernelType = "<<KernelType<<endl;
    if (!My->Psetup)
      My->outlog<<"in calcKernels, KernelType = "<<KernelType<<endl;
    My->root->setChargeAtCenterFace(KernelType);  //to set the charge at the particular face
    My->root->updateMoment();   //to calculate multipole moments of all cells
    My->root->updateStat();   // to calculate Hstat in all cells
    My->root->getKernelsRecursively(KernelType);   //to calculate newKernels from H_stat
  }
  
  if (My->Psetup)  {
    printMap ("before compareAndReplaceKernels,    Kernels:", Kernels);
    printMap ("before compareAndReplaceKernels, newKernels:", newKernels);
  }
  real delta = compareAndReplaceKernels();
  cout<<"delta = "<<delta<<endl;
  return delta;
}


//new version:
//added by DMAk:
//returns difference between Kernels and newKernels
//and put newKernels to Kernels:
real FMM::compareAndReplaceKernels ()
{
  assert (!newKernels.empty());
  std::map<int3, Kernel>::iterator iterMap1, iterMap2;
  int3 key,
    keyDelta;//key, corresponding to the largest deviation
  real delta = 0.;  //maximum deviation
  real error = 0.;
  for (iterMap1 = newKernels.begin(); iterMap1 != newKernels.end(); iterMap1++)
  {
    key = iterMap1->first;
    iterMap2 = Kernels.find (key);
    if (iterMap2 == Kernels.end())  //the key has not been found, need to put it
    {
      Kernels[key] = Kernel();
      iterMap2 = Kernels.find (key);
    }
    error =  fabs(iterMap1->second.Kx - iterMap2->second.Kx) +
             fabs(iterMap1->second.Ky - iterMap2->second.Ky) +
             fabs(iterMap1->second.Kz - iterMap2->second.Kz);
    if (error > delta)
    {
      delta = error;
      keyDelta = key;
    }
    iterMap2->second = iterMap1->second; //copy newKernel to Kernel
  }
  cout<<"keyDelta = "<<keyDelta<<", delta = "<<delta<<endl;
  return delta;
}


//set up the array of pointers to faces, that are used in setChargeAtCenterFace
void FMM::setupCenterFaces (cell *curCell)
{
  int cellIndex;
  static int faceNumber=0;  //number of faces, to catch errors 
  if (curCell->child0)
  {
    setupCenterFaces(curCell->child0);
    setupCenterFaces(curCell->child1);
  }
  else //smallest cell
  {
    MVector centerTemp; //center of the 2x2x2 cell near the origin in positive octant
    centerTemp = My->root->size*0.5+My->root->rect->MinCS;

    if (fabs(curCell->center[0]-centerTemp[0]) < My->root->rect->MinCS[0] &&
        fabs(curCell->center[1]-centerTemp[1]) < My->root->rect->MinCS[1] &&
        fabs(curCell->center[2]-centerTemp[2]) < My->root->rect->MinCS[2])
    {   //this is a cell at the center
      for (int faceN=0; faceN<6; faceN++) //loop faces
      {
        for (int i=0; i<3; i++)  //loop indices
          if (My->almost_same(curCell->cellfacelist[faceN].myFace->faceCenter[i],
            My->root->size[i]*0.5))
          {
            cellIndex=0;  //to distinguish the faces in centerFaces
            while (centerFaces[i][cellIndex])
              cellIndex++;
            centerFaces[i][cellIndex]=curCell->cellfacelist[faceN].myFace;
            faceNumber++;
            My->outlog<<"centerFace["<<i<<"]["<<cellIndex<<"] is found, center = "
              <<centerFaces[i][cellIndex]->faceCenter-My->root->size*0.5<<endl;
            if (i==2 && cellIndex==3) assert (faceNumber==12);
          }
      }
    }
  }//end of else
}


//added by DMAd (for debug)
void FMM::printMap (char *title, const std::map<int3, Kernel> & mapToPrint)
{
  My->outlog<<"From FMM::printMap: "<<title<<endl;
  if (mapToPrint.empty() )
    My->outlog<<"Map is empty";
  else {
    std::map<int3, Kernel>::const_iterator j;
    for (j = mapToPrint.begin(); j!=mapToPrint.end(); j++)
      My->outlog<<j->first<<"   Kernel   "<<j->second<<endl;
  }
  My->outlog<<endl;
}


//This function reads the FMM parameters and calculates the array size
//(NumBelow) that is needed by ParseBuild to build the cells
void FMM::Read_SetSizes (ifstream &inputFile)
{
  char junk[MAXLENFMM+1], fileName1[MAXLENFMM+1], fileName2[MAXLENFMM+1];
  inputFile>>junk>>MaxAngle2>>MaxOrder>>calculateKernels
    >>calculateExterior>>draftFMM>>fileName1>>fileName2;
//only one parameter can be non-zero:
  assert (calculateKernels + calculateExterior+draftFMM <2);
  if (draftFMM)  //we use draft version of the FMM
  MaxOrder = 1;  //only the first order multipoles are used
  MaxOrder1 = MaxOrder + 1;

//we need to set up NumBelow array, which will be used to allocate memory for
//multipole moment when cells are created.

  NumBelow = new int[MaxOrder1];  //the size of these arrays is already known

  for (int i = 0; i < MaxOrder1; i++)   //initialization
    NumBelow [i] = (i+1)*(i+2)*(i+3)/6;  // no. components w/order <= argument;
                                             // used to predict ConvTerms

  if (calculateKernels)  //we need to write them down after calculation:
    KernelsFile.open(fileName1, ios::out);   //reopen it for output
  else  //we need to read the Kernels in
    KernelsFile.open(fileName1, ios::in);
 if (!KernelsFile)
  {
    cerr<<"cann't open file "<<fileName1<<endl;
	cout<<"can't open kernel file "<<fileName1<<endl; // dunno where cerr goes!
	My->outlog<<"can't open kernel file "<<fileName1<<endl;
    exit(6);
  }
  if (My->periodic[0] || My->periodic[1] || My->periodic[2])
  {
    if (calculateExterior) //we calculate exterior mpole and images (DMApf)
      ImageFile.open(fileName2, ios::out);
    else //we need to read images and Vext
      ImageFile.open(fileName2, ios::in);
    if (!ImageFile)
    {
      cerr<<"cann't open file "<<fileName2<<endl;
      exit(7);
    }
  }

  My->outlog<<"Reading parameters for the FMM\nMaxAngle2 = "
    <<MaxAngle2<<", KernelsFile file is "<<fileName1
    <<", images file is "<<fileName2
    <<"\ncalculateKernels = "<<calculateKernels
    <<", calculateExterior = "<<calculateExterior
    <<", draftFMM = "<<draftFMM<<", MaxOrder = "<<MaxOrder<<endl;
  inputFile.getline(junk, MAXLENFMM, '\n');
}


//read Kernels from file KernelsFile and store them in Kernels
void FMM::ReadKernels()
{
  if (My->Psetup) My->outlog<<"in ReadKernels ..."<<endl;
  cout<<"reading the Kernels from the file ...";

  char ch;
  std::pair<int3, Kernel> Ktemp;
  char junk[MAXLENFMM+1];

  while((ch=KernelsFile.peek())!='!')
  {
    switch(ch)
    {
      case '#':
        KernelsFile.getline(junk, MAXLENFMM, '\n');
        break;
      case 'a': // now don't use this?? May in future let quantum be MVector.
        KernelsFile>>junk>>aspectRatio[0]>>aspectRatio[1]>>aspectRatio[2];
        KernelsFile.getline(junk, MAXLENFMM, '\n');
        break;
      case 'k':
        KernelsFile>>junk>>Ktemp.first.values[0]>>Ktemp.first.values[1]
          >>Ktemp.first.values[2]>>junk>>Ktemp.second.Kx>>Ktemp.second.Ky
          >>Ktemp.second.Kz;
        KernelsFile.getline(junk, MAXLENFMM, '\n');
        if (!((Kernels.insert(Ktemp)).second))
          cout<<"\n\t\tERROR WHILE READING KERNELS, key = "<<Ktemp.first<<endl;
        break;
      default:
        cout<<"ERROR WITH KERNELSIN, first character is "<<ch<<endl;
        exit(0);
    } //end of switch
  } //end of while

  KernelsFile.getline(junk, MAXLENFMM, '\n');
  if (My->Psetup) My->outlog<<"exiting ReadKernels\n"<<endl;
  cout<<"    the Kernels have been read"<<endl;
}//end of ReadKernels


//to write down calculated Kernels
void FMM::WriteKernels()  //added by DMAf
{
 //write down the Kernels only if they have been calculated
  assert (calculateKernels); 
  if (My->Psetup) My->outlog<<"in WriteKernels..."<<endl;
  map<int3, Kernel>::iterator iterMap;
  real eqCube = powf(My->root->rect->MinCS[0]*My->root->rect->MinCS[1]
    *My->root->rect->MinCS[2],1/3.);
  for (int i=0; i<3; i++)
    aspectRatio[i] = My->root->rect->MinCS[i]/eqCube;

  KernelsFile<<"aspectRatio "<<aspectRatio[0]<<"  "<<aspectRatio[1]<<"  "
    <<aspectRatio[2]<<endl;
  KernelsFile<<"# Kernels"<<endl;
  KernelsFile<<"#"<<setw(11)<<"key"<<setw(18)<<"Kx"<<setw(10)<<"Ky"<<setw(10)
            <<"Kz"<<endl;

  for (iterMap=Kernels.begin(); iterMap!=Kernels.end(); iterMap++)
  {
    KernelsFile<<"key"<<setw(3)<<iterMap->first.values[0]<<' '
      <<setw(3)<<iterMap->first.values[1]<<' '<<setw(3)
      <<iterMap->first.values[2]<<" Kernel "
      <<setw(10)<<iterMap->second.Kx<<' '<<setw(10)<<iterMap->second.Ky
      <<' '<<setw(10)<<iterMap->second.Kz<<endl;
  }
  KernelsFile<<"! end of Kernels file"<<flush;
  if (My->Psetup) My->outlog<<"exiting WriteKernels\n"<<endl;
}


//used for periodic FMM (DMApf):
//returns 1 if all three values are even
int FMM::coarseCharacter (const int3 &key) const
{
  return (!(key.values[0] % 2) && !(key.values[1] % 2) && !(key.values[2] % 2));
}


//used for periodic FMM
//recursively calculate Vext, find images of the root cell and write everything
//to ImageFile
void FMM::calcVextAndWriteImages ()
{
  assert (My->periodic[0] || My->periodic[1] ||My->periodic[2]);
  cout<<"Begin recursive calculation of exterior potential"<<endl;
   //need to calculate Acoef's
  calcVshellAndWriteImages ();

  //we need to setup a recursive procedure for calculation of Vext
  int counter = 0;  //to prevent infinite recursion
  real delta = 0.;
  while (((delta = calcExtOneIter()) > 0.000005) && (counter < 30))
    counter++;
  if (counter==30)
  {
    cerr<<"ERROR: iterative calculation of Vext, delta = "<<delta<<endl;
    exit(12);
  }

//write down the external potential mpole (calculated assuming root cell size =1)
//it needs to be rescaled before use
  ImageFile<<"Size: "<<reducedSize[0]<<"  "<<reducedSize[1]<<"  "<<reducedSize[2]<<endl;
  ImageFile<<"Vext: "<<*Vext<<endl;
  ImageFile<<"! end of the file"<<endl;
  cout<<"Recursive calculation has been finished, error = "<<delta<<endl;
}


//used for periodic FMM (DMApf):
//Vshell is calculated and images of the root cell are written to the ImageFile
void FMM::calcVshellAndWriteImages ()
{
  real Acoef, sum;
// We will loop over p, an integer vector describing 
// the position of an image of the root cell.
// First determine keyMax, the largest possible value for p we need to worry about.
// This will occur in the direction of the smallest dimension of the system,
// so we find this direction:
  int iSmallest=0;
  for (int i=0; i<3; i++)
    if ((My->PeriodVec[i] < My->PeriodVec[iSmallest]) && My->periodic[i])
      iSmallest = i;

  int3 keyMax; // only one component is actually used, for now
  // Increase keyMax until it's in the exterior. 
  keyMax[iSmallest]=2;   //start from 2
  
  do {keyMax[iSmallest]++;}
  while (My->root->Near(DispCell(My->root, keyMax)));

  My->outlog<<"keyMax = "<<keyMax<<endl;

  keyMax[iSmallest]*=2;  //the radius of the sphere is 2R (see notes)
  mpole *Dshell = new mpole(-1); //utility mpole, it is used to set the pot'l of the shell

  int3 p, t; // see writeup; t is the small displacement (+-1 or 0) of the
  // coarse lattice to construct the fine lattice.  It is nonzero only in
  // the periodic directions, so the loop is trivial (0 to 0) if not periodic.
  // If periodic, loop is from -keyMax to +keyMax.
  // note (L ? a:b) is a if L is true, b if L is false.
  // This nested loop over p and t calculates A_p (Acoef) from Eq. ... of the writeup.

  for (p.values[0] = (My->periodic[0] ? -keyMax[iSmallest] : 0);
      p.values[0] <= (My->periodic[0] ? keyMax[iSmallest] : 0); p.values[0]++)
    for (p.values[1] = (My->periodic[1] ? -keyMax[iSmallest] : 0);
        p.values[1] <= (My->periodic[1] ? keyMax[iSmallest] : 0); p.values[1]++)
      for (p.values[2] = (My->periodic[2] ? -keyMax[iSmallest] : 0);
          p.values[2] <= (My->periodic[2] ? keyMax[iSmallest] : 0); p.values[2]++)
      {
        sum = .0;
		// now loop displacements t (nonzero only in periodic directions)
        for (t.values [0]=(My->periodic[0] ? -1 : 0);
            t.values [0]<= (My->periodic[0] ? 1 :0); t.values[0]++)
          for (t.values [1]=(My->periodic[1] ? -1 : 0);
              t.values [1]<= (My->periodic[1] ? 1 :0); t.values[1]++)
            for (t.values [2]=(My->periodic[2] ? -1 : 0);
                t.values [2]<= (My->periodic[2] ? 1 :0); t.values[2]++)
              if (coarseCharacter (p-t)) //so (p-t)/2 is an integer
                if (!My->root->Near(DispCell(My->root, (p-t)/2)))
                  sum+=calculateSymFactor(t);
                

        Acoef = !My->root->Near(DispCell(My->root, p)) - sum;

    //now we know Acoef for the given value of p, we need to find Vshell:
        if (!My->almost_same(Acoef, 0.))  //needed to eliminate p=(0,0,0) case
        {
          Dshell->SetToDeriv (MVector(p[0]*reducedSize[0], p[1]*reducedSize[1],
                              p[2]*reducedSize[2]));
          *Vshell += (*Dshell)*Acoef;
        }

    //writing down the images of the root cell
        if (My->root->Near(DispCell(My->root, p)) && !p.isZero())
        {
          ImageFile<<"RootImage "<<p.values[0]<<' '<<p.values[1]<<' '<<p.values[2]<<endl;
          assert (My->almost_same(Acoef, 0.0));
        }
      }
  delete Dshell;
}  //end of the function



//used for periodic FMM (DMApf):
real FMM::calcExtOneIter()
{ // Assumes an estimate of Vext, the potential of a cored array of point charges,
	// is known and calculates a better one.
	// Uses Vext 
  mpole *tempMpole = new mpole(-1),
        *VextAdd = new mpole(-1);
  tempMpole->SetToPowerSym(My->periodic); // prepare to convolute potential with w(t)
  VextAdd->AddDownConv (Vext, tempMpole); // set VextAdd to the convolution
//divide VextAdd by 2^(n+1)
  VextAdd->rescale (2.);

  *VextNew = *Vshell + *VextAdd;  // add shell to get new estimate for Vext
// we need to put the divergent values to zero.
// 
  int periodicity = My->periodic[0]+My->periodic[1]+My->periodic[2];
  // for periodicity = 1    2    3
  // divergent orders  0    0    2
  // vanishing orders  1    1    3
  // so the following works if you check each case:
  for (int i=0; i<NumBelow[periodicity-1]; i++)
    VextNew->values[i]=0.0;

  real delta = Vext->compare (VextNew);  //maximum difference between mpoles
  My->outlog<<"delta = "<<delta<<endl;

  *Vext = *VextNew;   //put VextNew to Vext

  delete tempMpole;
  delete VextAdd;
  return delta;
}

//used for periodic FMM (DMApf):
real FMM::calculateSymFactor (const int3 &key) // the function w(t) in the writeup
{
  real W = 1.;
  for (int i=0; i<3; i++)
    if (key.values[i]!=0)
      W/=2.;
  return W;
}


//reads all images of the root cell and puts them to root->Partners list
void FMM::readAndSetupRootImg ()
{
  My->outlog<<"reading images and Vext from file "<<endl;
  char ch, junk[MAXLENFMM+1];
  int imgDisp[3];
  real reducedSizeIn[3];  //to check reducedSize

  while ((ch = ImageFile.peek()) != '!')
  {
    switch (ch)
    {
    case '#':
      ImageFile.getline(junk, MAXLENFMM, '\n');
      break;
    case 'S':   //reduced Size of the system 
		// (i.e., scaled so volume = 1; sensitive to aspect ratio but not size
		// cored-lattice potential Vext should depend only on .
      ImageFile>>junk>>reducedSizeIn[0]>>reducedSizeIn[1]>>reducedSizeIn[2];
      ImageFile.getline(junk, MAXLENFMM, '\n');
      My->outlog<<"reducedSizeIn = "<<MVector (reducedSizeIn)<<endl;
      My->outlog<<"reducedSize = "<<MVector (reducedSize)<<endl;

      for (int i=0; i<3; i++)  //compare the sizes
        assert (My->almost_same(reducedSizeIn[i], reducedSize[i]));
      break;
    case 'R':
      ImageFile>>junk>>imgDisp[0]>>imgDisp[1]>>imgDisp[2];
      ImageFile.getline(junk, MAXLENFMM, '\n');
      My->outlog<<"RootImage = "<<imgDisp[0]<<"  "<<imgDisp[1]<<"  "<<imgDisp[2]<<endl;
      My->root->Partners.push_back(DispCell(My->root, imgDisp));
      break;
    case 'V':
      ImageFile>>junk>>*Vext;
      My->outlog<<"Vext is "<<*Vext<<endl;
      Vext->rescale (averageSize);
      ImageFile.getline(junk, MAXLENFMM, '\n');
      break;
    default:
        cout<<"ERROR WITH Image, first character is "<<ch<<endl;
        exit(0);
    }
  }
}



void Kernel::SetToZero()
{
  Kx=Ky=Kz=0.0;
}

void Kernel::SetToMonopoleField(const int3 &key)
{
  mpole *D = new mpole(1);
  MVector dist (key);
  dist*=-0.5;  //0.5 is due to double space key
  D->SetToDeriv (dist);
  
  Kx = D->GetValue(1,0,0);
  Ky = D->GetValue(0,1,0);
  Kz = D->GetValue(0,0,1);
}

Kernel Kernel::operator+(const Kernel& Kright) const
{
  Kernel Kresult;
  Kresult.Kx = Kx + Kright.Kx;
  Kresult.Ky = Ky + Kright.Ky;
  Kresult.Kz = Kz + Kright.Kz;
  return Kresult;
}

void Kernel::operator+=(const Kernel& Kright)
{
  *this = *this + Kright;
}

Kernel Kernel::operator- (const Kernel & Kright) const
{
  Kernel Kresult;
  Kresult.Kx = Kx - Kright.Kx;
  Kresult.Ky = Ky - Kright.Ky;
  Kresult.Kz = Kz - Kright.Kz;
  return Kresult;
}

void Kernel::operator-=(const Kernel& Kright)
{
  *this = *this - Kright;
}

Kernel Kernel::operator/ (const real &right) const
{
  Kernel Kresult;
  Kresult.Kx = Kx/right;
  Kresult.Ky = Ky/right;
  Kresult.Kz = Kz/right;
  return Kresult;
}

void Kernel::operator/= (const real &right)
{
  *this = *this/right;
}


//added by DMAk
void Kernel::operator*= (const real number)
{
  Kx*=number;
  Ky*=number;
  Kz*=number;
}

NearPartner::NearPartner(face *F, Kernel *KerPtr)
{
  f = F;
  K = KerPtr;
}

// the following should overload << for derived class ofstream as well?
ostream& operator<<(ostream &out, const Kernel &k) //overload << for Kernels
{
  out<<" ("<<k.Kx<<' '<<k.Ky<<' '<<k.Kz<<") ";
  return out;
}

