//FMM.h classes needed for FMM

#ifndef FMM_H
#define FMM_H

#include <map> // leave this here for now, someday replace by class map
   // (tricky because it's a template)
#include <fstream> // for ofstream
// #include <ostream> // 
#include "types.h"

class mpole;
class int3;
class face;
class MVector; //added by DMAk
class cell;
 
using namespace std;

//specific to a system of uniformly magnetized cells:
class Kernel
{
  friend std::ostream &operator<< (std::ostream&, const Kernel&);
public:
//  data member:
  real Kx, Ky, Kz;  //charge FMM

// functions:
  Kernel ()
  { Kx=Ky=Kz=0.;}
  void SetToZero ();  //sets the kernel to zero
  void SetToMonopoleField(const int3&); //sets to monopole field
//operations:
  Kernel operator+ (const Kernel &) const; //Kernel addition
  void operator+= (const Kernel &);
  Kernel operator- (const Kernel &) const; //subtraction
  void operator-= (const Kernel &);
  Kernel operator/ (const real &) const; //division by a real number
  void operator/= (const real &);
  void operator*= (const real number);  //multiplication by number
};

class FMM
{
public:
  FMM();// initilize some parameters to default values
  ~FMM();//deallocate memory occupied by the arrays
  void setup();//allocate and initialize all arrays
//added by DMAk
  real calcKernels();  //calculate Kernels and return delta - maximum difference
                            //between the old Kernels and the new ones
  real compareAndReplaceKernels();  //compare newKernels with Kernels, returns difference
  void print(char *, const int [], const int);//to output the array of integers
  void print(char *, const int3&);//to output the valarray
  void printMap (char *title, const std::map<int3, Kernel> & mapToPrint);
  void Read_SetSizes (ifstream &);  //to read parameters from the script file
           //and calculate the array size NumBelow that is needed by ParseBuild
  void ReadKernels();
  void WriteKernels();
//the following functions are used in periodic FMM (DMApf)
  void calcVextAndWriteImages (); //calculate Vext, find images of the root cell
  real calculateSymFactor (const int3& key);  //calculate W(key)=1/(2^nkey)
                               //where nkey is the number of nonzero key.values 
  void readAndSetupRootImg (); //read in the images and put them to root->partners

//********************** data members: *************************
  fstream KernelsFile; //file to input (calculateKernels=0) or output (calculateKernels=1) Kernels 
  fstream ImageFile;  //file to input (calculateExterior=0) or output (=1) images and Vext
  mpole *scratchMpole1, *scratchMpole2;  //scratch mpoles (to increase performance)
  mpole *Vext;               //mpole needed for periodic FMM (DMApf)
  real aspectRatio[3];  //aspect ratio of the smallest cell (for Kernels)
  real reducedSize[3];   //reduced size of the system (DMApf),can it be MVector? 
  real averageSize; //geometrical average of the size along periodic directions
  int Parray;                //to control array otput; Parray = 0 means "no output"
  int NumComps;              //total number of components, see paper
   //the arrays needed for mpole:
  int *RecurseAxis;          //these arrays are needed for recursive calculations of
  real *RecurseFactor;      //Deriv. arrays
  int *RecurseMpole;         //the "previous multipole"
  int ***index;              //opposite to IndArr, gives i_n from vector n
  int3 *IndArr;//opposite to index, gives vector index n from i_n
  int *NumBelow;             //total number of components with order <=
  int **ConvSum;             //give i_{n+m} from i_n and i_m
  int *OddOnes;              //store the components with odd order
  int *NumOddBelow;          //number of such components with order <=
  int *DerivTerms;           //give the number of p's for a certain order
  int **DerivPower;          //give all p's from i_n
  real **DerivCoef;         //F_n[p] in paper
  std::map<int3, Kernel> Kernels; //Kernels for calculation of the potential from
                             //the NearPartners - close smallest cells
  real MaxAngle2;           // square of max opening angle
//added by DMAk:
  std::map<int3, Kernel> newKernels;  //temporary storage for calculated Kernels
  void setupCenterFaces (cell *); //find the center faces recursively
  face *centerFaces[3][4];  //center faces, 3 - # of different kernel types
                    //the 2nd index is to distinguish 4 faces with the same normal
  int calculateKernels;  //=1 if we calculate Kernels only,
                         //=0 if we read Kernels in and make simulation
  int calculateExterior; //=1 if we calculate exterior mpole potential mpole
                         //=0 if we read it in and make simulation (DMApf)
  int draftFMM;      //DMAfs: =0 if we use complete FMM (full magnetostatic field)
                     //       =1 if we use only self-demag field
  int MaxOrder, MaxOrder1;  //DMAfs: MaxOrder is maximum order of multipole moments

private:                 //(utility functions)
  void calcVshellAndWriteImages ();       //calculate Vshell and write down the images
  real calcExtOneIter (); //make one iteration for Exterior mpole calculation
  int coarseCharacter (const int3 &) const;  //coarse lattice character. function

//data members
  mpole *VextNew, *Vshell;
};


//specific to a system of uniformly magnetized cells:
class NearPartner //used to store NearPartner for smallest cells
{
public:
  face *f;  //face that produces the potential
  Kernel *K; //Kernel for brute force calculations for smallest cells

  NearPartner(face *, Kernel*);  //constructor for charge FMM
};

#endif //FMM_H

