// This is System.C file version 1.63
#include "cell.h"
#include "System.h"

#include "Lowest.h"
#include "units.h"
#include "FMM.h"
#include "mpole.h"
#include "Average.h"
#include "pulse.h"
#include "Hyst.h"
#ifdef GRAPHICS
  #include "display.h"
  static void mySensorCallback (void *data, SoSensor *)
  {   //cout<<"Entering mySensorCallback"<<endl;
    if (((System*)data)->HandleCallback())
    return; //if returns 1 display scene & repeat
    else
      exit(0);
  }
  // extern MySystemCallback(void *data, SoSensor *);
#endif

#include <string>
unsigned long getSeed(); // defined in Random.cpp 
void setSeed(unsigned long); // defined in Random.cpp

#define FileLen 160 
// maximum length of init. cond. file name, runID, and movie file name

const int MAXLEN = 100;
System::System(){
  Ext_Field1=NULL; Ext_Field2=NULL; Hyst=NULL; 
  #ifdef GRAPHICS
  UpVector=NULL; // for test
  mySensor=NULL; 
  #endif
  MyFMM=NULL; // may be necessary to create successfully????

for (int i=0; i<Max_Type; i++) Avgs[i]=WatchedAvgs[i]=NULL;
  nt = 0;
  t=0;
  //added by DMAm & DMAs
  #ifdef GRAPHICS
  movieVersion = 1.1;   //version that this program can read
  this_frame=0; //current frame
  numberSkipFrames = 1; //number of frames to skip
  Repeat = 1; // pause after each frame
  ShowWatchedMStatic=ShowWatchedHk=ShowWatchedExch=0; // default to false
  ShowWatchedTot=0;
  #endif
}

System::~System() { }

System *My = new System(); // errors if this is in main()
int ZeroDisp[3] = {0,0,0};

void System::ReadScript()
{ 
#ifdef _DEBUG 
	cout<<"Starting System::ReadScript"<<endl;
#endif
	if(!paramFile) 
	{cout<<"cannot access script file"<<endl; MYEXIT;}
    string temp, junk; // declare various temporaries (can't inside case stmt)
	// use iostream string instead of c-style string so we can manipulate
	// it to make other file names.
	// don't need: char c80[80]; // c-style string
	MVector ScratchVector;
	cell *cellptr;
    char ch;
	real xdum,ydum; // multi-use dummies
	int idum,jdum;
    real movieVersionRead; //version of the given movie, added by DMAm

	paramFile>>ws; // eatwhite, get rid of tabs in 1st line
	if (paramFile.peek()=='I') // want to override runID from .script name
	  {cout<<"Override default runID "<<runID<<" from .script filename with ";
	   paramFile>>junk>>runID; getline(paramFile,junk,'\n'); paramFile>>ws; 
	   cout<<runID<<endl; 
	  }

	outlog.open((runID+".log").c_str()); // c_str converts to c-string
	if(outlog) cout<<"opened"; else cout<<"can't open";
	cout<<" file "<<runID+".log"<<" for outlog"<<endl;
    // open Avgout later, when we know we need it
	movielog.open((runID+".traj").c_str());   //changed by DMA to traj
	if(movielog) cout<<"opened"; else cout<<"can't open";
	cout<<" file "<<runID+".traj"<<" for movielog"<<endl;
    // cin.get(); // debug

  #ifdef GRAPHICS
    if (My->Movname[0] != NULL)  //we are reading movie file
       My->direction=My->FORWARD;
     else
       My->direction=My->COMPUTE; 
       // to wait, direction can be overwritten by T|W line in script 
  #endif


  //read in parameters
 
  while ((ch = paramFile.peek()) != '!')
  {

#ifdef _DEBUG // use this form for following progress in more detail
	  cout<<"Reading script file, found 1st char = "<<ch<<endl;
#endif // than end user would want to see
   // cout<<"hitkey"<<endl; cin.get();
  switch(ch)
   {
    case '#':
     // just reads & discards rest of line
      break;
	case '+': // field increment
	  paramFile>>junk>>Increment;
	  outlog<<"Increment set to "<<Increment;
	  break;
    case 'm': // read a movie, don't compute 
      paramFile>>junk>>Movname;
      assert(junk[0] == 'm'); //sanity check
      outlog<<"read name of movie file: "<<Movname<<endl;
      mov.open(Movname.c_str());
      cout<<"Movname="<<Movname<<endl;
      if (runID+".traj" == Movname)
      //    {cout<<"Can't read&write same file"<<endl; EXIT;} 
      {cout<<"Can't read&write same file"<<endl; exit(0);} //Shuxia_pc
      if(!mov) cout<<"cann't open file mov"<<endl;
      if (mov.peek() == 'V')   //need to check the version
      {
        mov>>junk>>movieVersionRead;
        getline(mov,junk,'\n'); // getline() is in header <string>
        #ifdef GRAPHICS
        outlog<<"version of the movie is "<<movieVersionRead
          <<", program can handle version "<<movieVersion<<endl;
        if (movieVersionRead!=movieVersion)
        { cout<<"* * * incorrect version of the movie file! * * *"<<endl;
          MYEXIT;}
        #endif 
      }
      #ifdef GRAPHICS
      direction = STOP; //read the movie in forward direction (DMAm)
      #endif
      break;
      
      case 'c': // to set a cell's magnetization
	  float Mx,My,Mz; // temporaries
      /* paramFile>>junk>>iCell>>Mx>>My>>Mz;
	  assert(iCell<cell::NextcellID);
      cout<<"Defining initial M (case 'c') for cell "<<iCell<<", M="<<Mx<<","<<
		  My<<","<<Mz<<endl;  
      cellptr = CellFromID[iCell];
	  */
	  paramFile>>junk>>ScratchVector>>Mx>>My>>Mz;
	  cellptr = root->SubCellContaining(ScratchVector);
	  cout<<"Defining initial M (case 'c') for cell "<<*cellptr<<" at "<<cellptr
		  <<" with size="<<cellptr->size<<", M="<<Mx<<","<<My<<","<<Mz<<endl;  
	  assert(cellptr); 
	  cellptr->M = MVector(Mx,My,Mz); // M may not be normalized, just get q:
      cellptr->q.rotateTo(MVector(Mx,My,Mz)); 
	  cellptr->M =((cellptr->q * (cellptr->rect->M0)) * (~cellptr->q)).getMVector();
      //cout<<"Cell pointer = "<<CellFromID[iCell]<<", changed q to "<<CellFromID[iCell]->q<<endl; 
      break;
//Shuxia_pc
      /*case 'C': //to choose the algorithm (dM/dt or dq/dt) added by DMA
      paramFile>>junk>>use_q;
      outlog<<"found the value for use_q, use_q = "<<use_q<<endl;*/ 
      case 'A': //to choose the algorithm Heun(10),Euler(00) or dqdt(01)
      paramFile>>junk>>Heun>>use_q;
	  outlog<<"found the value for Heun and use_q: "<<Heun<<use_q<<endl;
      break;

      case 'p': //control print out and projection
      paramFile>>junk>>Psetup>>Pcpt>>CompAvg>>Proj>>movie;
      outlog<<"Psetup="<<Psetup<<",Pcpt="<<Pcpt<<",CompAvg="<<CompAvg
                <<", Proj="<<Proj<<", movie="<<movie<<endl;

      if(CompAvg){
	  Avgout.open((runID+".Avgout").c_str());
      if(Avgout) cout<<"opened"; else cout<<"can't open";
	  cout<<" file "<<runID+".Avgout"<<" for Avgout"<<endl;

      Avgs[101] = new Average(Max_N); // 7 = number of quantities to average
      Avgs[101]->init();
      Avgs[0] = new Average(Max_N);
      Avgs[0]->init();
      }
      //added by DMAm - to write the movie version to a file
      #ifdef GRAPHICS
      if (movie)
        movielog<<"Version "<<movieVersion<<endl;
      #endif
      break;
    case 'D': // "Display" block
#ifdef GRAPHICS
	// Now actually set up graphics (cell system has been set up):
    assert(!UpVector); // for debug -- this SHOULD fail if call Run twice, just remove it then.
	if(!UpVector) setupGraphics(); // call if it hasn't been called yet,
	   // UpVector just happens to be something it creates
	mySensor = NULL; // to see if it gets created
//	if (!mySensor) mySensor = new SoTimerSensor(mySensorCallback, My); // thinks "My is float!
	if (!mySensor) mySensor = new SoTimerSensor(mySensorCallback, this);
	assert(mySensor);
	// mySensor->setInterval(1./25); // defaults to 1/30!
    mySensor->schedule(); // can we do this twice?
#endif // read the rest even if no graphics, so don't need to change script file
      getline(paramFile,junk); paramFile>>ws; // skip white space
      assert(junk[1]=='i'); //input file should say "Display..."
      while((ch = paramFile.peek()) != '!')  //inner nested while loop
      {   cout<<"Read 1st char in DisplayBlock = "<<ch<<endl; // debug
         switch(ch)
          {
            case '#':
              break;
            case 'T': // time display
				paramFile>>junk>>xdum>>ydum;
#ifdef GRAPHICS // this MUST come after setting up graphics, above
				TimeText = new SoText2; //create text string, its existence
				  // is used as flag that we're displaying
				TimeText->string="uninit";
			  SoTransform *TimeTextTransform; //local vbl, translation of the time display
  			  // assert (TimeTextTransform); //debug
			  TimeTextTransform = new SoTransform; // can't share line w/declaration
			  TimeTextTransform->translation.setValue(xdum*root->size[0],ydum*root->size[1],0); 
			    // put it at center bottom?
			  cout<<"about to create TimeNode"<<endl;
			  SoSeparator *TimeNode; //node that contains the time display
			  TimeNode = new SoSeparator;
			  cout<<"about to add child to timenode"<<endl;
			  TimeNode->addChild(TimeTextTransform);
			  SoMaterial *TimeMaterial; //material for the vector
			  TimeMaterial = new SoMaterial;
              assert (TimeMaterial);
              TimeMaterial->diffuseColor.setValue(0,0,0); // black
              TimeNode->addChild(TimeMaterial);
			  TimeNode->addChild(TimeText);
			  rootReal->addChild(TimeNode);
#endif
			  cout<<"DOne with case TimeDisplay"<<endl;
				break;
 case 'H': // field display
		paramFile>>junk>>xdum>>ydum>>idum>>jdum;// i,jdum=WhichComps,WhereLabel
#ifdef GRAPHICS // this MUST come after setting up graphics, above
        externalField = new VectorDisplay("Hext",idum,jdum, // label,whichcomps,wherelabel
        MVector (root->center[0]+(xdum-0.5)*root->size[0],
		   root->center[1]+(ydum-0.5)*root->size[1],root->LowerLeft()[2]),
		fieldScale, 0, 0, 1); // position by x,ydum
		cout<<"about to add ExternalField display"<<endl;
        assert (externalField);
        rootReal->addChild(externalField->getNode()); 
#endif
	    cout<<"DOne with case H_ext display"<<endl;
		break;

 		   default: cout<<"Error in Display block"<<endl; MYEXIT;
		  } // end inner switch of Display block
		  getline(paramFile,junk,'\n'); // read end of line with ch
	      paramFile>>ws; // skip white space at beginning of next line
	  } // end while of DIsplay block - found !, ending list
      // leave end of its line for getline(paramFile,junk) to rea



	  break; // end case D, display block
	case 'E': //Export data block
      getline(paramFile,junk); paramFile>>ws; // skip white space
      assert(junk[1]=='x'); //input file should say "Export data"
	  Xport.open((runID+".data").c_str()); // c_str converts to c-string
          Xport.precision(12);
	  cout<<"Opening file "<<runID+".data"<<" for data export, ";
	  if(Xport) cout<<"opened OK"<<endl; 
	  else {cout<<endl<<"FAILED -- is another program reading it?";	
	    MYEXIT;}
	  
      // not need: Xport_t=TRUE; // if write file at all, write t
	  XportM=0; XportH=0; // default to not print

      while((ch = paramFile.peek()) != '!')  //inner nested while loop
      {   cout<<"Read 1st char for xport = "<<ch<<endl; // debug
         switch(ch)
          {
            case '#':
              break;
            case 'M': //magnetization vector
              XportM=1;
              outlog<<"will export magnetization"<<endl;
              break;
           case 'H': //magnetic field vector
              XportH=1; 
              outlog<<"will export magnetic field"<<endl;
			  break;
		  } // end inner switch of export list
		  getline(paramFile,junk,'\n'); // read end of line with ch
	      paramFile>>ws; // skip white space at beginning of next line
	  } // end while of export list - found !, ending list
      // leave end of its line for getline(paramFile,junk) to read
	  #ifndef LOGSWITCH_H
	  Xport<<"t         "; // write header line
      #endif
      #ifdef LOGSWITCH_H
	  Xport<<"#t   E    Mx      My       Mz   Theta   Phi  Hx      Hy       Hz "; // write header line
      #endif
	  if(XportM) Xport<<"Mx      My       Mz   ";
	  if(XportH) Xport<<"Hx      Hy       Hz   ";
	  Xport<<endl;
	  break; // end case E, export data

   case 'G': //System geometry, build the system according to script file
		if(!paramFile) {cout<<"can't read script file"; MYEXIT;}
      //paramFile>>junk>>quantum; assert(junk[2]='o'); // should read Geometry
	paramFile>>junk>>quantum; assert(junk[2]=='o'); // should read Geometry //Shuxia_pc
          cout<<"Read junk, quantum = "<<junk<<quantum<<endl;
	  cout<<"paramFile is "; if(paramFile) cout<<"OK"; else cout<<"BAD"; cout<<endl;
	  getline(paramFile,junk,'\n'); // go to next line
      outlog<<"About to read in Geometry of the system"<<endl;
      // assert (MyFMM->NumBelow);  //to check that the array was created
      root = new cell(NULL);
      assert (root);
      // root->size = MVector(PeriodVec);
      // root->center = root->size/2; 
      // The center must be set here in case the root cell is
      // "non-rectangular". If it is rectangular, the center is set
      // AGAIN by SetCenter, hopefully to the same value.
      // root->diameter2 = root->size.Square();
      // assert (MyFMM->NumBelow);

      root->parseBuild(); // assumes we are at beginning of line
	  if (Psetup) outlog<<"Returned from parseBuild"<<endl;

     //by now, the tree has been constructed, NextcellID is the total number of
     // cells in the system, initialize the array of cell pointers here

   /* I don't understand where the following came from, screws up for 1 cell.
      Worry later.
   if (root->rect)   //the root is rectangular
      { cout<<"root is rect,ptr="<<root->rect<<", mu0="<<mu_0<<endl<<flush;
	    cout<<"Ms="<<root->rect->Ms<<endl<<flush;
		cout<<", minCS="<<root->rect->MinCS<<endl<<flush; 
		// fieldScale has been replaced by inverse since this written!!!!!

        fieldScale = (real)(3*root->rect->ExchangeA/(mu_0*root->rect->Ms)
         * (1./(root->rect->MinCS[0]*root->rect->MinCS[0])
           + 1./(root->rect->MinCS[1]*root->rect->MinCS[1])
           + 1./(root->rect->MinCS[2]*root->rect->MinCS[2])));

        if (fieldScale < H_bias.Magnitude())  //find the maximum
          fieldScale = H_bias.Magnitude();

        outlog<<" fieldScale is: "<<fieldScale<<endl;
      }
*/
      cout<<"About to allocate,now CellFromID="<<CellFromID<<endl;
      CellFromID = new cell*[cell::NextcellID];
	  cout<<"Allocated CellFromID, now CellFromID="<<CellFromID<<endl;
  // Know A, Ms, so we can initialize Jex:
      root->initJexRecursively();
	  // could initMRecursively here too? instead of earlier when setup rect
      root->initIDRecursively(); //must follow geometry and precede use of CellFromID
      cout<<"Ret. from initIDRecursively, CellFromID[1]="<<CellFromID[1]<<endl;
	  // Could we combine all 3 of these global recursions?
	  // CalculateExterior, if needed
  
  if(MyFMM)
  {
	  CreatePartners(); // set up partner lists before run or calc kernels
	  //recursively calculate Kernels
    if (MyFMM->calculateKernels)
    {
      int counter=0; //to prevent infinite repetition
      cout<<"beginning calculation of Kernels"<<endl;
      MyFMM->setupCenterFaces(root);  //to set the center faces
      while ((MyFMM->calcKernels() > 0.000001) && (counter < 30))
        counter++;
      MyFMM->WriteKernels();
      exit (0);
    }
	else // not calc kernels, may be calculating exterior:
	  if (MyFMM->calculateExterior)
		{
		MyFMM->calcVextAndWriteImages();
		exit (2);
		}
  } // end if(MyFMM)
      break;
      
      case 'i':  // read initial condition; this must be after geometry
      { real tstart,tshift; // time to search for in file, possible shift
        paramFile>>junk>>ICname>>tstart>>tshift;
        cout<<"read init. cond. filename "<<ICname<<", starttime="<<tstart<<", shift="<<tshift<<endl;
        if(Psetup)  outlog<<"read initial condition file, "<<ICname<<endl;
        ifstream infil;
        infil.open (ICname.c_str());
         assert (infil); // infil returns 0 if file didn't exist
         outlog<<"Opened file "<<ICname<<", about to read it"<<endl;
         while (t<=tstart) // stops reading when t>=tstart
         readFrameBinary(infil);
         outlog<<"Found frame at t="<<t<<", should be >tstart="<<tstart<<endl;
         if(WatchedCell) cout<<"Read initial q="<<WatchedCell->q<<endl; // debug
         t-=tshift; // shift time origin (useful if t gets too large)
		 infil.close();
        // writeFrameBinary(movielog); // just to debug
       } 
       break;

    case 'H':
      paramFile>>junk>>H_bias;
      outlog<<"found 1st char in line (H), read value="<<H_bias<<endl;
      break;
    case 'h': //Pulsed Field part (PBV, 7/20/04)
      paramFile>>junk; // reads to white space
      int iPulse; paramFile>>iPulse; // reads number of pulse (1 or 2)
      outlog<<"Case h, found pulse number "<<iPulse<<endl;
      PulsedField  **Ext_FieldPtr; // pointer to Ext_Field1 or Ext_Field2,
         // a pulse object pointer
      switch(iPulse)
         {
         case 1:  Ext_FieldPtr=&Ext_Field1; break; // now *Ext_FieldPtr is Ext_Field1
         case 2:  Ext_FieldPtr=&Ext_Field2; break;
         default: cout<<"Error in iPulse"<<endl; MYEXIT;
          } // end switch(iPulse), now *Ext_FieldPtr is the pointer to this pulse object
       if (! *Ext_FieldPtr) *Ext_FieldPtr = new PulsedField(); // create it if it doesn't exist
       (*Ext_FieldPtr)->Read(&paramFile); // read pulse information
       // outlog<<"found 1st char(h), read field pulse, fields now "<<Ext_Field1->Hmax<<" and "<<Ext_Field2->Hmax<<endl; 
	   break; 

    case 'B': // bigness of field vector, really field scale
      paramFile>>junk>>fieldScale;  // RELATIVE to M scale here!
      outlog<<"Read relative fieldScale="<<fieldScale<<endl;
	  fieldScale*=GlobalMScale; // now absolute, in nm/(kA/m)
	  outlog<<"Absolute field scale in  nm / (kA/m) is "<<fieldScale<<endl;
      break;
    case 'Y':
      paramFile>>junk;
      Hyst = new Hysteresis();
      Hyst->Read(&paramFile);
      break;
    case '&': //other parameters, & means 'rest'
      paramFile>>junk>>gamma>>Temp>>AvgInt>>IncInt>>MovInt;
      outlog<<"found 1st char in line (&), gamma = "<<gamma<<", Temp = "
        <<Temp<<",AvgInt= "<<AvgInt<<",IncInt= "<<IncInt
        <<", MovInt = "<<MovInt<<endl;
      break;
    case 'f': //parameters for recursive FMM added by DMAf
	  // These MUST be early, needed for geometry, need know whether calc kernels
      // long before you actually do, so not to read kernels
      MyFMM = new FMM; //DMAfs: memory allocation for FMM, need after outlog.
	  assert (MyFMM);

      MyFMM->Read_SetSizes (paramFile);  //read parameters and calculate NumBelow
      MyFMM->setup();   //prepare the FMM class
      if (!MyFMM->calculateKernels)
        MyFMM->ReadKernels();
      break;
    
    case 'P':
      paramFile>>junk>>periodic[0]>>periodic[1]>>periodic[2]>>PeriodVec[0]
               >>PeriodVec[1]>>PeriodVec[2];
      outlog<<"found 1st char in line (P),periodic boundary conditions are: "
             <<periodic[0]<<", "<<periodic[1]<<", "<<periodic[2]<<endl;
      outlog<<"found 1st char in line (P),periodic length is: "
             <<PeriodVec[0]<<", "<<PeriodVec[1]<<", "<<PeriodVec[2]<<endl;
      break;
   
     case 'R': // Run block: gather info about when to stop, & run
      getline(paramFile,junk); paramFile>>ws; // skip white space
      assert(junk[1]=='u'); //input file should say "Run block"
      while((ch = paramFile.peek()) != '!')  //inner nested while loop
      {  // cout<<"Read 1st char = "<<ch<<endl; // debug
         switch(ch)
          {
            case '#':
              break;
            case 'T': //input value of tstop manually
              paramFile>>junk>>tstop;
              outlog<<"found 1st char in Time block (V), tstop="<<tstop<<endl;
              break;
            case 'd':
              paramFile>>junk>>dt;
              outlog<<"found 1st char in Run block (d), dt="<<dt<<endl;
//DMAd              outlog<<"*******************, w="<<w<<endl;
//DMAd              outlog<<"******** Expected LAMDA is: "<<0.5*w*w*dt-alpha*w<<endl;
              break;
		    case 'D': //number of update loops per callback call
		       paramFile>>junk>>d;
		       outlog<<"found 1st char in line (D), d="<<d<<endl;
		       break;
            case 'n': // get fixed # of precession cycles?  NOT USED
              paramFile>>junk>>D;
              outlog<<"found 1st char in Run block (n), D="<<D<<endl;
              assert(fieldScale); // ?? has this been set? IS INVERSE?
              dt=D*2*M_PI/(gamma*fieldScale);
              outlog<<" dt (case D) is: "<<dt<<endl;
              break;
            case 'N': //number of dt's, overwrite  old dt, for exact solution
              int n; // this is not debugged
              paramFile>>junk>>n;
              outlog<<"found 1st char in line (N), read value, N="<<n<<endl;  
              assert(n > 0);
              dt = tstop/n;
              outlog<<" dt (case N) is: "<<dt<<endl;
              break;
       case 'K': //Kikuchi's exact solution
              assert((root->rect) && (root->rect->alpha));
              assert(H_bias.Magnitude()); //changed to bias by DMAh
              tstop =log(9.0)/(root->rect->alpha*gamma*(H_bias.Magnitude()));  //DMAh
              outlog<<"found 1st char in Time block (K), tstop="<<tstop<<endl;
              break;
/*          case x: //spin wave case // This SHOULDN"T be here, is in geom?
              cout<<"ERROR, this part of the code is not finished yet"<<endl;
              assert(root->rect);  //the root cell must be rectangular
              real w;
              
              // Use formula for frequency, gamma*(H+anisCoef*Ms*cos(SpinAx)+Ms*cos(SpinAx)JB(k))
              // where Ax = Ay in this case
              // (see notes), where B(k)=sum over nbrs of 1-exp(ikr)
              real NumPeriods;
              paramFile>>junk>>NumPeriods;
              outlog<<"read NumPeriods = "<<NumPeriods;
              outlog<<"cos(kVector[0]*MinCS[0]) = "<<cos(kVector[0]*root->rect->MinCS[0])
                     <<", cos(kVector[1]*MinCS[1]) = "<<cos(kVector[1]*root->rect->MinCS[1])
                     <<",cos(kVector[2]*MinCS[2]) = "<<cos(kVector[2]*root->rect->MinCS[2])<<endl;
              // Now calculate the sum B in a loop over i.
              // the if statements are to handle the nonperiodic case
              // We can handle the nonperiodic case only if Lx/MinCS = 1 or 2
              // cells.

        //these assertions are put by DMA: (H_bias and easyAxis must be along z-axis)
              assert (almost_same(sqrt(H_bias[0]*H_bias[0]+
                H_bias[1]*H_bias[1])/H_bias.Magnitude(), 0.0));
              assert (almost_same(sqrt(root->rect->avEasyAxis[0]*root->rect->avEasyAxis[0]
                +root->rect->avEasyAxis[1]*root->rect->avEasyAxis[1])
                /root->rect->avEasyAxis.Magnitude(), 0.0));

              real fac; // number of neighbor cells in i direction
              w = 0.; // initialize
              int i;
              for (i=0;i<3;i++)
                {
                 if (periodic[i]) fac=2.;
                      // number of neighbors in i direction
                 else
                 //not periodic;Lx/MinCS is # cells, -1 to not count cell itself
                 {fac = root->size[0]/root->rect->MinCS[0]-1.; assert (fac<1.1);}
                   // Spin wave calc fails if width>2;
                 w += fac*(1.-cos(kVector[i]*root->rect->MinCS[i]));
                   // when fac=2, this is exp(ika)+exp(-ika)
                   // note that when fac=1, cos=0 or 1
                }
              if (SpinAx == SpinAy)
                w *= gamma*root->rect->A/(mu_0*root->rect->Ms*root->rect->MinCS
                  *root->rect->MinCS)*cos(SpinAx);
              else
                cout<<"Spin Wave set up error, SpinAx should = SpinAy"<<endl;

              // now it's exchange part of omega
              w += gamma*H_bias.Magnitude(); // add external field term, changed to bias by DMAh

              if (SpinAx == SpinAy)
                w += gamma*root->anisCoef*root->rect->Ms*cos(SpinAx);
                          // add anisotropy field term 
              else
                cout<<"Spin Wave set up error, SpinAx should = SpinAy"<<endl;

//DMA: THE FOLLOWING LINES ARE BEING CHANGED FOR SPIN WAVE TEST WITH DEMAG FIELD
              real w1, w2, cosOfkAngle;
              w1 = w;
              cosOfkAngle = kVector[2]/kVector.Magnitude();
              w2 = w+gamma*root->rect->Ms*(1.-cosOfkAngle*cosOfkAngle);
              w = sqrt(w1*w2);
              outlog<<"w = "<<w<<endl;
              assert(w != 0);
              tstop = NumPeriods*(2*M_PI)/w;
              outlog<<"Done with Time block for spin wave case (S), tstop="
                     <<tstop<<endl;
              break;
*/
            case 'p':
			  paramFile>>junk>>xdum;
              outlog<<"found 1st char in Run block (p), xdum="<<xdum<<endl;
              assert(fieldScale); // is inverse now?
              dt=xdum*2*M_PI/(gamma*fieldScale); // dt = dum periods
              outlog<<" dt (case p) is: "<<dt<<endl;
              break;
			case 'P': // set pause mode initially
			 #ifdef GRAPHICS
                          Repeat = 0;
                         #endif
			  break;
            case 'W': // wait for user before starting
              #ifdef GRAPHICS
                cout<<"Setting direction to STOP"<<endl;
                direction = STOP; 
              #else // note this option creates problems debugging, just freezes
                cout<<"********* Bad option T | W, not using graphics"<<endl;
              #endif
              break;
            default:
              outlog<<"Error in run block, 1st char is:"<<ch<<endl;
              MYEXIT;;
            } //end of inner Run switch
          getline(paramFile,junk,'\n');
	      paramFile>>ws; // skip white space at beginning of next line
        } //end of inner Run while loop, shd be at !
      // leave end of its line for getline(paramFile,junk) to read
      // assert(junk[0] == '!');
      gdt = gamma*dt;
	  // dt determines the format of time display
	  
	  xdum=1.0/d; // dummy is unit; was 1.0; now resolution is d*dt
	  int places; // # places after decimal point
	  for(places=0; xdum>dt; places++)
		  xdum/=10;
	  if (places>7) 
	    {places=7; outlog<<"dt too small to display all digits"<<endl;}
	  TimeFormat[0]='\0'; // empty string, can't just set TimeFormat="t=..."
	  strcat(TimeFormat,"t=%X.Xf ns");  // fix up X's next
      TimeFormat[3]=char(int('2')+places); 
      TimeFormat[5]=char(int('0')+places); 
	  // outlog<<"TimeFormat="<<TimeFormat<<endl;
	  
      outlog<<"gdt (angle/(kA/m)) is: "<<gdt;
      outlog<<", dt = "<<dt<<", t = "<<t<<endl;
      root->setRectangleParamRecurs();  //calc. parameters for all Rectangles
         // this is here because they depend on dt
      // Now actually run!
//cout<<"before setupGraphics, q="<<My->WatchedCell->q<<endl; // debug
#ifdef GRAPHICS
   CalcAllFromM(); // know M at initial time, calculate H, etc. to display
   #ifdef WIN32
        SoWin::mainLoop();
   #else
	    SoXt::mainLoop(); 
   #endif
#else
       //while(My->evolve()) // returns false when dt>tstop
      while(evolve()) //Shuxia_pc 
      {   
      //  if(My->Pcpt) 
        if(Pcpt) //Shuxia_pc
        //My->outlog<<"without graphics, enter while(evolve) loop **********"<<endl;
       outlog<<"without graphics, enter while(evolve) loop **********"<<endl;
         }
       //if(My->movie) {
         if(movie)   {    //Shuxia_pc  
       /*  My->writeFrameBinary(My->movielog); 
         My->movielog<<"~end of the movie"<<endl;
         My->movielog.close(); }
        */
        writeFrameBinary(movielog);
        movielog<<"~end of the movie"<<endl;
        movielog.close(); }
      // My->finalCheck(); //checks things about the final state, for debugging  //Shuxia - How to find the final correct M
         exit(0);
#endif //GRAPHICS
      break; //end of case 'R', Run block

    case 'W': //Block to describe watched cell
	  outlog<<"Reading watched cell block"<<endl;
      getline(paramFile,junk); paramFile>>ws; // skip white space
      assert(junk[1]=='a'); //input file should say "Watched Cell"
      while((ch = paramFile.peek()) != '!')  //inner nested while loop
      {  // cout<<"Read 1st char = "<<ch<<endl; // debug
         switch(ch)
          {
            case '#':
              break;
            case 'P': //position of watched cell
		      paramFile>>junk>>W[0]>>W[1]>>W[2];
              outlog<<"found 1st char in line (W), position to be watched is: "
               <<W[0]<<", "<<W[1]<<", "<<W[2]<<endl;
              WatchedCell = root->SubCellContaining(W);
              cout<<"WatchedCell is "; 
			  if(WatchedCell) cout<<*WatchedCell; 
	          else cout<<"NULL"; cout<<endl;
              break;

            case 'H': // display "total" field (no anisottropy,exch!)
              assert (WatchedCell);
              #ifdef GRAPHICS   //Shuxia_pc
	          WatchedCell->ShowH = 1;
              cout<<"Will display total field at watched cell"<<endl;
              #endif
              break;
            case 'A': // display anisotropy field
              assert (WatchedCell);
			  ShowWatchedHk = 1;
              cout<<"Will display anisotropy field at watched cell"<<endl;
              break;

            case 'T': // display total field (including anis,exch,random)
              assert (WatchedCell);
              #ifdef GRAPHICS //Shuxia_pc
			  ShowWatchedTot = 1;
              cout<<"Will display anisotropy field at watched cell"<<endl;
              #endif
              break;

            case 'X': // display exchange field
             assert (WatchedCell);
              #ifdef GRAPHICS	//Shuxia_pc
             ShowWatchedExch = 1; 
              //cout<<"Will display anisotropy field at watched cell"<<endl;
              cout<<"Will display exchange field at watched cell"<<endl;
              #endif
              break;
        	case 'M': // display magnetostatic field
              assert (WatchedCell);
               #ifdef GRAPHICS   //Shuxia_pc
        	   ShowWatchedMStatic = 1;
                cout<<"Show MS Field"<<endl;
              #endif
              break;

            default:
              outlog<<"Error in WatchedCell block, 1st char is:"<<ch<<endl;
              MYEXIT;;
            } //end of inner WatchedCell switch
          getline(paramFile,junk,'\n');
	      paramFile>>ws; // skip white space at beginning of next line
        } //end of inner WatchedCell while loop, shd be at !

      cout<<"leaving watched-cell block"<<endl;

      break; //end of case 'W', Watched Cell block
 
    case 'z': // zero out time (set initial t=0)
      cout<<"Resetting time to zero"<<endl;
      t = 0;
      break;

    default:
     cout<<"Read script line, 1st char unknown:"<<ch<<endl;
     cout<<"Error in parameter input file, 1st char is:"<<ch<<"=#"<<(int)ch<<endl; MYEXIT;;
    } //end switch, read script file
    getline(paramFile,junk);
	if (!paramFile) {cout<<"Error in script file"<<endl; exit(2);}
	// cout<<"At end of line, read "<<junk<<endl;
	paramFile>>ws; // skip white space at beginning of next line
  } //end while
    // don't really need this??::
    getline(paramFile,junk); // get rid of ! line.
	assert(junk[0]=='!');
	paramFile>>ws; 
} // end of System::ReadScript()

void System::CalcAllFromM() 
  // assumes small-cell M(t) known, calculates all from them:
  // external+pulse field (depends on t)
  // moments, energy, magnetostatic field, exchange, 
{
   if(MyFMM) root->updateMoment(); //update moments, initialize H_av and SmoothField, DMAf
//total field: bias, pulse and demagField from the far surface
   /*H_ext = H_bias;
   if (Ext_Field1)    //pulsed field is present
     H_ext += Ext_Field1->GetValue(t);
   if (Ext_Field2) H_ext += Ext_Field2->GetValue(t); // allow only 2

   if (Hyst) H_ext += Hyst->GetField(dt);
   */
   if (FirstUpdate) // calculate stuff that depends only on t (needn't recalculate later)
   {  H_ext = H_bias;
      if(Ext_Field1) //pulsed field is present
		   H_ext += Ext_Field1->GetValue(t);
      if(Ext_Field2) H_ext += Ext_Field2->GetValue(t); // allow only 2
      if(Hyst) H_ext += Hyst->GetField(dt);
	  cout<<", H_ext="<<H_ext<<endl;
    } // end if FirstUpdate
      
 
 //Shuxia_pc

   if (MyFMM) // need SmoothField; if not, H_ext is used in update_m
   {  root->SmoothField->SetToVector (-H_ext); //set SmoothField to homogeneous field 
      //correction to energy (eliminate external field from Energy_stat)
      root->Energy_stat=-0.5*(*(root->moment)*(*(root->SmoothField)));
      root->updateStat(); //added by DMAf
   }
   root->updateExch();
   //put display code in one place
   #ifdef GRAPHICS
   if (LastUpdate && (nt%d==0)) // for display, needed only if nt is a multiple of d
      {externalField->updateVector(H_ext);
       //cout<<"updated H_ext display="<<H_ext<<",now call updateOIScene(root)"<<endl;
      if(TimeText) // if displaying time, make new string
	  {
		  sprintf(TimeString,TimeFormat,t); //convert t to c-string
		TimeText->string=TimeString;
		// outlog<<"Converted time t="<<t<<" into string "<<strbuf<<endl;
	  }
       updateOIScene(root);}
   #endif
   // cout<<"updated H_ext display="<<H_ext<<endl;
  // cout<<"About to call updateOIScene(root)"<<endl;
  //updateOIScene(root);
  // cout<<"returned from updateOIScene, exiting CalcAllFromM"<<endl;
}  // end CalcAllFromM

int System::evolve() // on entry, M and H all known at t
{  // called from main() and System::HandleCallback()
   if(Pcpt)
   outlog<<"In System::evolve() now, t="<<t<<"!!!!!!!!!!!!!!!!!!!!"<<endl;
 
   // cout<<"In evolve(), t="<<t<<", q="<<WatchedCell->q<<endl;
    //Formerly calculated root cell magnetization here -- for using simple demag tensor
   // originally, but later to display M1+m2 in MRAM.
   // Calculated q from M.  Deleted it. Is in pre-8/05 codes.
   t += dt; nt++; // now M is at time t, but H is at t-dt
   if(Pcpt) outlog<<"new nt = "<<nt<<", dt = "<<dt<<endl;
    FirstUpdate=1; // only false the second time through the Heun algorithm
// things that depend only on t are only calculated if(FirstUpdate)
    LastUpdate=1-Heun; // if not Heun. Normalize and display only if LastUpdate.
    root->update(); CalcAllFromM();  // calcs M(t), then H(t), almost the same for Heun and Euler algorithms
  if (Heun) // make another pass to correct M
    {FirstUpdate=0; LastUpdate=1; root->update(); CalcAllFromM(); }

   //root->update(); // gets M(t+dt)
   //CalcAllFromM(); // calculates t-dependent H_ext, and all H's at time t
   // now M and H both refer to time t (t_old+dt)
   // so we can write lots of stuff at time t
   //cout<<"evolve: t="<<t<<", M_Watch="<<WatchedCell->M<<", H="<<H_ext<<endl;
   if(CompAvg){
   if((nt%AvgInt) == 0)
   {
      My->outlog<<"t = "<<t; //  ?? why do you need "My->" here?
	  if (WatchedCell) My->outlog<<", WC->M = "<<WatchedCell->M
        <<", H_av = "<<WatchedCell->H_av;
	  My->outlog<<endl;
     Avgout.precision(8);
     //for Hysteresis 
     Avgout<<H_ext[0]<<"	"<<root->M.Magnitude()<<"   "<<root->M[0]<<"	"<<t<<endl;
     //My->Avgout<<t<<"	"<<root->M[0]<<"   "<<H_ext[0]<<endl;
     /*
     Avgout<<t;
     Avgs[101]->WriteandInit();
     Avgs[0]->WriteandInit();
     Avgout<<endl;
     */
    }
   }
  //if(Xport) // if file is open for export
  #ifndef LOGSWITCH_H 
  if(Xport)
   {      
	   My->Xport<<t<<"   ";
	  if (XportM) Xport<<WatchedCell->M<<"   ";
	  if (XportH) Xport<<H_ext<<"   ";
	  Xport<<endl;
   }
  #endif

   if(movie && ((nt%MovInt) == 0)) writeFrameBinary(movielog);

   return (t < (tstop-.5*dt)); // "time isn't up yet"
}

//added by DMAk:
void System::CreatePartners ()
{
  root->Partners.push_back(DispCell(root, ZeroDisp));
  if (!(MyFMM->calculateKernels) && !(MyFMM->calculateExterior)
     && ((periodic[0] > 0) || (periodic[1]>0) || (periodic[2]>0))) //DMApf
    MyFMM->readAndSetupRootImg();
  root->CullPartners(); //recursively generates all partner lists
}

void System::writeFrameBinary(ofstream & fileOut)
{  // like writeFrame, but writes exact data for restart
  cell *C1;
  real Magn;
  fileOut<<"T";
 fileOut.write((char*)&t,sizeof(t));
 // Now save binary seed. 
 cout.precision(4);
 //cout<<"Write frame, t="<<t<<" R: "<<getSeed()<<endl;
 //if(WatchedCell) cout<<", q="<<WatchedCell->q<<", M="<<WatchedCell->M<<endl;
 fileOut<<"\nR: "<<getSeed();
  for (int i=0; i<(cell::NextcellID); i++)
    {fileOut<<endl;
    C1=CellFromID[i];
    // write info on cell if smallest OR largest(root):
    if(!(C1->child0) || (C1->CellType == root->CellType))
  {
   fileOut<<"C:  "<<i<<"q";
   fileOut.write((char*)&C1->q,sizeof(C1->q));
   Magn = C1->M.Magnitude();
   fileOut.write((char*)&Magn,sizeof(Magn));
   if(0) // if(0) effectively comments out if(Pcpt)
     {outlog<<"Writing cell "<<i<<", q="<<C1->q<<endl;
     outlog<<"binary form ";
     outlog.write((char*)&C1->q,sizeof(C1->q)); outlog<<"!end"<<endl;
     outlog<<"|M|="<<Magn<<", binary=";
     outlog.write((char*)&Magn,sizeof(Magn)); outlog<<"!endM"<<endl;
     }
    } // end if(!...)
     } //end of for()
  fileOut<<"\n!end of quaternion values at step "<<nt<<", time "<<t<<endl;
} // end writeFrameBinary

//the version of the movie file this readFrameBinary() can read
//is stored in movieVersion. Each time we make a crucial change
//to this function we need to change movieVersion accordingly.
//Possibly we can have a switch statement in readFrameBinary() that
//allows us to read different versions of the movie.:

int System::readFrameBinary(ifstream &inFile)

 {
  outlog<<"Starting System::readFrameBinary()"<<endl;

  if (inFile.peek()=='~')   //EOF is met while reading the movie
  {
    cout<<" * * * * * * END OF THE MOVIE * * * * * * "<<endl;
    #ifdef GRAPHICS
    direction = STOP;  // we may want to read file even if no graphics
    #endif
    return 0;  //movie file is finished
  }

  char ch,ch1,ch2; // temporaries
  char junk[MAXLEN+1];
  cell* C1;
  MVector TempM;
// while(t<=tstart)
// {
  while((ch = inFile.peek()) != '!') 
  {if(Pcpt) outlog<<"reading frame, 1st char="<<ch<<endl;
   switch(ch)
   {
    case '#':
    case 'V':
      break;
    case 'T':
      inFile>>ch1;
      assert(ch1=='T');
      inFile.read((char*)&t, sizeof(t));
      cout<<"read t = "<<t;
      #ifdef GRAPHICS
      // cout<<", this_frame = "<<this_frame;
      #endif
      cout<<endl;
      break;
     case 'R':
     unsigned long seed;
     inFile>>junk>>seed;
     assert(junk[0]=='R');
     setSeed(seed);
     cout<<" seed="<<seed<<" at t="<<t<<endl;
     break;
 case 'C':
      int ID;
      real absM;
      inFile>>junk>>ID>>ch2;
      // cout<<" junk[1]="<<junk[1]<<ID<<ch2<<endl;
      assert(ch2=='q');
      C1=CellFromID[ID];
      inFile.read((char*)&C1->q, sizeof(C1->q));
      inFile.read((char*)&absM, sizeof(absM));
      if(Pcpt) outlog<<"Converted to quaternion"<<C1->q<<endl;
      if (C1->child0) // not smallest
        {
        if (C1->rect) TempM= C1->rect->M0*(absM/C1->rect->Ms);
          // rect, but not smallest; correct the length
        else TempM = MVector(0,0,absM); // non-rectangle
        }
      else // smallest
 //       {TempM=C1->rect->M0;}
    TempM=C1->rect->M0;
if(Pcpt) outlog<<"Defined TempM along z, "<<TempM<<endl;
      C1->M = ((C1->q * TempM) * (~C1->q)).getMVector();
      // In 2/05, was root->rect!! Wrong.
      //this rotates (0,0,absM) by q to get correct M 7-04
if (Pcpt) outlog<<"Read cell::q="<<C1->q<<", M ="<<C1->M<<endl;
      outlog<<"found cellID and quaternion, cellID="<<ID <<" quaternion= " <<CellFromID[ID]->q<<endl;
       break;
      default:  //changed a little by DMA
      cout<<"Error in readFrameBinary() reading "<<ch<<" character."<<endl;
      MYEXIT;
      break;
    } // end switch
   inFile.getline(junk,MAXLEN,'\n');
    } // end while
  inFile.getline(junk, MAXLEN, '\n');
//   } // end of while(t<tsart)

  if (Pcpt) outlog<<"Returning from System::read"<<endl<<flush;
  return 1; //successful
} // end ReadFrameBinary

void System::finalCheck()
{
  outlog<<"Starting System::finalCheck"<<endl;
  assert(paramFile);

  char ch; // just like in ::ReadScript()
  string junk;
  MVector M1; // for some reason you can't declare M1 inside switch!
  getline(paramFile,junk); // should say "Final Check"
  if (junk[0]!='F') {cout<<"No final check"<<endl; return;}
  while ((ch = paramFile.peek()) != '!')
  {
   switch(ch)
   {
    case '#': // comment
      break;
    case 'E':
      real AllowedErr;
      paramFile>>junk>>AllowedErr;
      outlog<<"read in Allowed error = "<<AllowedErr<<endl;
      break;
    case 'M': // expected value of M at watched cell
      real m0, m1, m2;
      paramFile>>junk>>m0>>m1>>m2;
      M1 = MVector(m0,m1,m2);
      outlog<<"read expected M value for watched cell, M1 = ("<<M1[0]<<","
             <<M1[1]<<","<<M1[2]<<")"<<endl;
	  assert(WatchedCell);
      outlog<<"WatchedCell->M="<<WatchedCell->M<<endl; 
      real Error;
      Error = (M1-WatchedCell->M).Magnitude();
      outlog<<"WatchedCell's CellType is: "<<WatchedCell->CellType<<endl;      
      //cout<<"WatchedCell's CellType is: "<<WatchedCell->CellType<<endl;
      (WatchedCell->M).print(outlog,"Watched cell had M=");
      M1.print(outlog,"Expected value =");
      outlog<<"******** actual LAMDA*2M_PI/w is: "<<log(sqrt((WatchedCell->M[0]*WatchedCell->M[0]+WatchedCell->M[1]*WatchedCell->M[1])/(M1[0]*M1[0]+M1[1]*M1[1])))<<endl;
      outlog<<"Error = "<<Error<<endl;
      outlog<<"Error = "<<Error<<endl;
      assert(Error<AllowedErr);
      break;
    default:
      outlog<<"Error in final check"<<ch<<endl; MYEXIT;;
      break;
   } // end switch
      getline(paramFile,junk); // for each case
  } // end while
  
  cout<<"*********** PASSED FINAL CHECK **************"<<endl;
}

MVector System::Displace(MVector V, const int* disp1)  
// displaces vector by "disp1";
{
  //assert(PeriodVec[0]);
  V[0]+=disp1[0]*PeriodVec[0];
  V[1]+=disp1[1]*PeriodVec[1];
  V[2]+=disp1[2]*PeriodVec[2];
  return V;
}
void System::prepareExit()  //output some information, close files, etc
{
  if (Movname[0] != NULL) //we have been reading a movie
    mov.close();
  else 
    if (movie) //if we have been writing a movie
    {
      writeFrameBinary(movielog);
      movielog<<"~end of movie file"<<endl;
      movielog.close();
    }
  }
#ifdef GRAPHICS
//handle the callback action, called from mySensorCallback
//this_frame always corresponds to the position of the get-pointer
//in mov file. This is important for proper operation of FORWARD
//when the position of this_frame is not in the pos_frame array
int System::HandleCallback()
{
  int i; //used in COMPUTE part of the switch
  // cout<<"InHandleCallback"<<endl;
  switch (direction)
  {
    case HOME: //go to beginning of the movie file
      mov.seekg(0);  //find the beginning of the movie file
      readFrameBinary(mov); //show the first frame
      direction = STOP; //pause the movie
      this_frame = 0;
      cout<<"starting from the beginning of the movie"<<endl;
      break;

    case FORWARD:  //movie file is read in forward direction
      cout<<"forward direction, speed is "<<numberSkipFrames;
      if ((int)pos_frame[this_frame+numberSkipFrames]!=0) //position is known
      { // need cast type to (int) to compare to 0!
        this_frame+=numberSkipFrames;
        mov.seekg(pos_frame[this_frame], ios::beg);
      readFrameBinary(mov);
      }
      else  //position is unknown
        for (i=0; i<numberSkipFrames; i++)
        {
         if (readFrameBinary(mov))    //EOF has not been met
          this_frame++;
          else break;

          pos_frame[this_frame]=mov.tellg();  //save the position of the frame
        }
      if (!Repeat)
        direction = STOP; //to pause the movie

      break;

    case BACKWARD: //skip several frames, moving backwards
      cout<<"backward direction, speed is "<<numberSkipFrames;
      if (this_frame>numberSkipFrames)
      {  //beginning is not reached
        this_frame-=numberSkipFrames;
        mov.seekg(pos_frame[this_frame], ios::beg);
     readFrameBinary(mov);
    cout<<" t = "<<t<<endl;
        if (!Repeat)  //pause the movie
          direction = STOP;
      }
      else   //beginning of the movie is reached
        direction = HOME;
      break;

    case STOP:     //pause the movie file
      return 1; // don't updateOIScene
      break;

    case COMPUTE:   //call evolve
    // loop recursively through all cells, changing their magnetization etc.
      // cout<<"In callback, case COMPUTE"<<endl;
      do
	    if(!evolve()) // returns false when dt>tstop
        { direction = DONE; break;}
      while (nt%d > 0); // quit when multiple of display interval  

 //     for (i=0; i<d; i++) // no longer use loop
 //       if(!evolve()) // returns false when dt>tstop
 //       { direction = DONE; break;}
      if (!Repeat) direction = STOP; // pause computing
      break;

    case DONE:      //means t>tstop
      //read new information from the script file
/************************************************************************
 final check should be here, is removed
      direction = COMPUTE; //to compute again with new parameters
      return 1;
  else I need to exit:
************************************************************************/
      direction = EXIT;
      break;

    case EXIT:      //unschedule? and return from the program
      if (Movname[0] != NULL) //we have been reading a movie
      {
        cout<<"!end of "<<Movname<<".log file"<<endl;
        mov.close(); }
      else {
        cout<<"Call writeFrameBinary to write final condition"<<endl<<flush;
        if (movie) //if we have been writing a movie
        {
          writeFrameBinary(movielog); 
          movielog<<"~end of movie file"<<endl;
          movielog.close();
        }
      }
      cout<<"exiting the program, t="<<t<<endl;
      // mySensor->unschedule(); not sure why did this -- let user close window
      // cout<<"Unscheduled"<<flush<<endl;
      direction = WAITFORUSERTOCLOSE;
      return 1; // was 0, closed window
      break;
    case WAITFORUSERTOCLOSE:
      return 1;
      break;
  } //end of switch (direction)
  return 1;
} //end of HandleCallback

void System::updateOIScene(cell* curCell)
{
  if (curCell->child0)  //not smallest cell
  {
    updateOIScene (curCell->child0);
    updateOIScene (curCell->child1);  //recursive calls
    curCell->M = curCell->child0->M + curCell->child1->M; // need calc M
    // Note that this is wierdly normalized!
  }
  
  if (curCell->ShowM)
  {
    if(curCell->child0) // not smallest, need calc q
		curCell->q.rotateTo(curCell->M); 
	  
    real x0,x1,x2,x3;
    curCell->q.CopyTo(x0,x1,x2,x3);
    if (Pcpt)
      outlog<<"in updateOIScene, cell center is "<<curCell->center<<endl;
    curCell->myMTransform->rotation.setValue(x1,x2,x3,x0);
                     //OI uses wrong order!
    real lengthScale = GlobalMScale*curCell->M.Magnitude(); // in nm
    curCell->myMTransform->scaleFactor.setValue(lengthScale, lengthScale, lengthScale);
  }
  
  if (curCell->ShowH)
         curCell->HDisp->updateVector(curCell->H_av);
}
#endif  //GRAPHICS

int System::almost_same(real x, real y)//added by DMApf to compare two real numbers
{
  if ((fabs(x) < 0.000001) && (fabs(y) < 0.000001))
     return 1;  //both are zero
  else
  if (fabs(x/y - 1.) < 0.00001)
    return 1;   //both are non-zero but equal
  else
    return 0;
}
#ifdef GRAPHICS
void System::ListCells(ostream& fstrm)
{   fstrm<<"List of cells:"<<endl;
for (int i=0;i<cell::NextcellID;i++)
		fstrm<<"#"<<i<<":"<<CellFromID[i]<<endl;
}
#endif

