#ifndef SYSTEM_H
#define SYSTEM_H

#define Max_Type 103 // maximum number of cell types: not really so many,
  // 101,102 are special
#define MAX_FRAMES 10000 //for movie, added by DMAm
#include "types.h"

#include "MVector.h" // to declare MVector members of system
#include "Quaternion.h"
#include <fstream> // not enuf to declare class name
#include <string> 

class cell;
class FMM;
class Average;
class PulsedField;
class Hysteresis;

#ifdef GRAPHICS
   class SoSeparator;
   class VectorDisplay; 
   class SoTimerSensor; // this is used near end
   class SoMaterial; // to avoid needing display.h
   class SoText2;
#endif

using namespace std;
 
class System{
public:
  //***************data members*********************
  ifstream paramFile;
  ofstream Avgout; //output file storing the average
  ofstream Xport; // file for routine plotting of data
  // ofstream paramin; //file to store input parameters, for debugging
  ofstream Errorlog; // for startup errors
  ofstream outlog; //log file for bulky output, for debugging
  ofstream movielog; // log file for replay the movie
  string ICname;
  string Movname;
//  char *ICname; // name of init.cond. file
//  char *Movname; // name of movie file
  ifstream mov; // input stream to read in movie file
  int Psetup, Pcpt, CompAvg,Proj,movie, use_q,Heun,FirstUpdate,LastUpdate;
     //boolean variable to control print out, Psetup--split cells, Pcpt--compute
     //CompAvg--compute average
     //Proj-project LocalAxis perpendicular to M
     //movie-control print out of movie file
     //use_q for dmdt algorithm, use_q=0 to use dmdt
  int d; //Display frequency, number of dts per display
  int m; // =1, read in movie file; =0, display and calculate at the same time 
  string runID;
//  char * runID;
  //the above data members are used in ReadScript() function

  FMM *MyFMM;  //pointer to FMM class (DMAfs)
  //***********************************
  MVector H_total;
 //**************************************
  cell *root;
  cell **CellFromID;
     //array of pointers to store the point to a cell
  real t; //time variable
  int nt; //number of time steps at each update
  int AvgInt; //averaging interval
  int IncInt; //increment interval
  int MovInt; //interval to write out movie file
  //static int ZeroDisp[3];
  real quantum; // distance quantum, smallest distance used. Could later be MVector.
  int periodic[3]; //boolean, periodic[0]=1 if periodic in x direction
  MVector PeriodVec; // periodic length when the system is periodic and the root
                     // cell is not a rectangle (include the air gap)
  MVector W; //position to watch, read in from input file
  cell *WatchedCell; // at which we may display the following:
  int XportM, XportH; // Boolean, to write fields to file Xport
  Average *Avgs[Max_Type];
  Average *WatchedAvgs[Max_Type];

  //data members from former Parameters class
  real Increment; // increment for bias field & ...
  MVector H_bias; //bias field (DMAh)
  MVector H_ext; // includes transient pulse
#ifdef  GRAPHICS
// MVector kVector; //now in Rectang
  SoSeparator *rootReal; //roots for scene graphs, in real&tree windows
  SoSeparator *UpVector; // unit vector in z direction, with cone point.
  VectorDisplay *externalField, *MStaticField, *ExchField, *AnisField, *TotField; //added by PBV 
  int ShowWatchedMStatic, ShowWatchedHk, ShowWatchedExch, ShowWatchedTot; // display vector
  // int dummyjunik;
  SoText2 *TimeText; // exists only if we display time 
  char TimeFormat[11]; // one extra; easier to put characters into c-string
  char TimeString[20]; // temporary, <string> class won't support sprintf
#endif

#endif
  Quaternion q_init; //used to set initial quaternion for a cell
  real fieldScale; //take the bigger one between H_bias and J*Ms(exchange field)
  real GlobalMScale; // length in nm of displayed M vector / |M|
  real gamma; //gyromagnetic ratio;
  real D;
  real dt; //time step
  real tstop; //time to stop
 //  real tstart; //time to start
  real Temp; //temperature
  real gdt; // is gamma*dt, for convenience
  PulsedField *Ext_Field1, *Ext_Field2;   //pulsed field (PBV 7/20/04)
  Hysteresis *Hyst;

  //**************constructor and member functions******************
  System();
  ~System();
  void ReadScript();
  void writeFrameBinary(ofstream &); //Store necessary information for restart
  int readFrameBinary(ifstream &); // Read in the data written by writeFrameBinary 
  void prepareExit (); //write final information and close files Shuxia
  void finalCheck(); //added 06/08/00, checks if we get the same result for
                     //the same cell when we changed the code
  void CalcAllFromM();
  int evolve(); //added 05/30/00, for purpose of seperating Graphics part
  void CreatePartners (); //added by DMAk, create Partners and NearPartners lists
  int almost_same(real, real);   //added by DMApf for comparison of real numbers
  MVector Displace(MVector, const int*);
  #ifdef GRAPHICS
    //needed for movie: (added by DMAm)
  real movieVersion;
     //movie version that this program can read, depends on readFrame()
  int Repeat; //=0 if the movie reading pauses after reading numberSkipFrames frames
  enum Direction
  {FORWARD, BACKWARD, STOP, EXIT, HOME, COMPUTE, DONE, 
     WAITFORUSERTOCLOSE} direction; //added by DMAm
   int numberSkipFrames; //number of frames to skip for fast-forward, -backward
   int this_frame; //current frame, DMAm
   streampos pos_frame[MAX_FRAMES]; //position of a frame, DMAm
   SoMaterial *surfaceMaterial;  //DMA
   SoTimerSensor* mySensor;
   int HandleCallback();  //used to handle the callback action, DMAm
   void updateOIScene (cell*);//to update the scene, added by DMAm
   void setupGraphics();
   void ListCells(ostream& fstrm);
  #endif // GRAPHICS
};
extern System *My;

 //#endif // this is VERY odd, suddenly "unexpected" so I removed it
