// this is micromagnetic cell.C function 
#include "units.h"
#include "mpole.h" // so cell can have moments
#include "cell.h"
#include <iomanip> // needed for setw
#include "System.h"
#include "FMM.h" // so NearPartner is defined, to make lists of it
#include "Average.h" // to average cell properties
#ifdef GRAPHICS
  #include "display.h" // for Open Inventor classes
#endif

extern System *My;

#define SUCC(x) (x+1)%3
void GenTwoNorVar(real *FirstVariate, real *SecondVariate); // in Random.cpp

const int MAXLEN = 100;
//struct mallinfo memory = mallinfo();

Rectang::Rectang()
{
  PreviousLayer=NULL;
  alpha=0.;
  gadt=0.;
  Ms=0.;
  H_rand_scale=0.;
  MinCS = MVector_zero;
  avEasyAxis = MVector_zero;
  M0 = MVector(.0, .0, .0); 
  Hk = 0.;
  AnisPower=NULL;
  ExchangeA = 0.;
  SpinAx=SpinAy=SpinPh=0.; //spin wave amplitude=0
  kVector.SetToZero(); // spin wave vector
  Mp= MVector(.0, .0, .0); // Shuxia_1120
  aJ=0;

}

void Rectang::setup()
{  // reads from script file My->paramFile; just finished R line with size.
  char junk[MAXLEN+1], form[MAXLEN+1];
  char ch;
  MVector Origin;

  My->paramFile>>ws; // eatwhite, get rid of tabs before reading line
  while ((ch = My->paramFile.peek()) != '!')
    // '!' returns to case R of ParseBuild
  {My->outlog<<"In rect->setup, read char="<<ch<<endl;
   switch(ch)
   {
    case '#': // comment, just ignore
      break;

	case 'D': // divide rectangle into cells
      My->paramFile>>junk>>MinCS[0]>>MinCS[1]>>MinCS[2];
      My->outlog<<"After D, read"<<junk<<" & min cell size = "<<MinCS<<endl;
    // Note that at this point MinCS is a TARGET size. We may not end up 
    // with cells of precisely this size.  Redefine in InitMRecursively as actual size.

      Origin = WholeRect->center - WholeRect->size*0.5;
      My->outlog<<"Origin="<<Origin<<endl;
      WholeRect->buildRect(Origin);
      My->outlog<<"rect->setup() built rect, size="<<WholeRect->size<<
		  ", center="<<WholeRect->center<<endl;
	  break;

	case 'M': // M_saturation
	  My->paramFile>>junk>>form>>Ms;
  assert ( (junk[0]=='M') && (junk[3]=='a') );  //should read "M_sat.."
  assert ( form[0]=='c');  //should read "constant", others not supported yet
      My->outlog<<"Ms = "<<Ms<<" kA/m"<<endl;
	     M0 = MVector(.0, .0, Ms); // wrong place?????????but someday might depend on rectangle?

  break;

	case 'K': // Anisotropy parameters
	  My->paramFile>>junk>>form;
  assert ( (junk[0]=='K') && (junk[3]=='n') );  //should read "K_aniso.."
      switch(form[0]) // may have several forms for anisotropy, now powerlaw
	  {
		case 'p': real HK_soft,HK_hard;
		AnisPower = new PowerLaw;
		My->paramFile>>AnisPower->exponent>>HK_soft>>HK_hard>>
				avEasyAxis[0]>>avEasyAxis[1]>>avEasyAxis[2];
		AnisPower->constant=HK_soft/Ms;
        AnisPower->SetCoef((HK_hard-HK_soft)/Ms,
			WholeRect->size[0]); // diff,length
 		 // WholeRect is cell corresponding to this rectangle
		My->outlog<<"Read anisotropy, hard="<<HK_hard<<"PowerLaw="<<
			AnisPower->constant<<"+"<<AnisPower->coef<<"*x^"<<
			AnisPower->exponent<<endl;
		break;

		default: cout<<"rect->setup() found error in script file, reading anisotropy, read "<<form<<endl; 
      MYEXIT; 
	  }
  break;
 
  	case 'L': // LL damping 
	  My->paramFile>>junk>>form>>alpha;
 
	My->outlog<<"in Rectang::setup, read junk="<<junk<<", form="<<form<<", alpha="<<alpha<<endl;
	assert (( junk[0]=='L') && (form[0]=='c'));  //should read "LL" & constant

  break;
 
case 'E': // Exchange constant
	  My->paramFile>>junk>>form>>ExchangeA;

  My->outlog<<"the following parameters were read in Rectang::setup:\n"
    <<" A = "<<ExchangeA<<" J/m"<<endl;
  assert ( (junk[0]=='E') && (form[0]=='c') );  //should read "Exchange_A", "constant"

  ExchangeA*=1e18;   //J/m to J'/nm (see description of the units)
  
  My->outlog<<"In our units, A = "<<ExchangeA<<" J'/m"<<endl;

  break;
  case 'S': //Shuxia_1120
     My->paramFile>>junk>>aJ>>Mp[0]>>Mp[1]>>Mp[2];
     My->outlog<<"a(I) is  "<<aJ<<" Mp="<<Mp<<endl;
  break;
    case 'W': // Spin wave vector k, 0.5 gives HALF wavelength in system
      real nx, ny, nz;
      My->paramFile>>junk>>nx>>ny>>nz;
      cout<<"found 1st char in inner loop(W), (nx,ny,nz)=("<<nx<<","
                     <<ny<<","<<nz<<")"<<endl;
	  assert(WholeRect->rect == this); // sanity check, WholeRect is this cell
      assert(WholeRect->size[0]); 
      kVector=MVector(nx/WholeRect->size[0],ny/WholeRect->size[1],
                              nz/WholeRect->size[2]);
      kVector *= (2*M_PI);
      My->outlog<<" kVector is = ("<<kVector[0]<<","<<kVector[1]<<","
              <<kVector[2]<<")"<<endl;
      break;
    case 'A': //amplitude angle and phase angle
      My->paramFile>>junk>>SpinAx>>SpinAy>>SpinPh;
      My->outlog<<"found 1st char in inner loop(A), SpinAx="<<SpinAx
                     <<", SpinAy="<<SpinAy<<", SpinPh="<<SpinPh<<endl;
      break;

    case 'Q': //arbitrary rotation q_init (can also set M via 'I')
      real q0, q1, q2, q3;
      My->paramFile>>junk>>q0>>q1>>q2>>q3;
      q_init = Quaternion(q0,q1,q2,q3);
      My->outlog<<"found 1st char in inner loop(Q), q_init = ("<<q0
          <<","<<q1<<","<<q2<<","<<q3<<")"<<endl;
      break;

      case 'X': // exchange coupling to this layer (from below)
#ifdef COUPLING
      real Tx,Ty,Tz; // thickness of AF coupling layer
      My->paramFile>>junk>>Tx>>Ty>>Tz>>tJ;
         // the variable tJ = energy/area / (mu0*M1*M2)
      assert(junk[1]=='c'); // should be "Xchange..."
      My->outlog<<"Read line starting "<<junk<<
         ", and spacer thickness="<<Tx<<","<<Ty<<","<<Tz<<
         ", interlayer coupling, tJ(in nm)="<<tJ<<endl;
      My->outlog<<"Upper layer (this) = "<<*LastLayer<<endl;
      spacing = MVector(Tx,Ty,Tz);
      // Now couple layers by looping cells in "this" rectangular cell.
      // We haven't initialized the linear index CellFromID, 
      // so we have to do it recursively.
      LastLayer->SetInterPartners();
#else
      My->outlog<<"Found Xchange layer coupling line, need compiler flag on"<<endl; 
      MYEXIT; exit(2);
#endif
      break;

	  case 'I': // initial magnetization of rectangle (sets q_init)
 		My->paramFile>>junk>>WholeRect->M; // M dummy, to set q_init
		 // since we know how to make a spin wave, just turn this into a spin wave:
		
	    q_init.rotateTo(WholeRect->M); // now q_init rotates to M
		// unless explicitly set, spin wave amplitude defaults to zero
        // initMRecursively() will be called later, in ParseBuild
		My->outlog<<"Read I & "<<junk<<"&M= "<<WholeRect->M<<"\n &q="<<q_init<<endl;
        break;

    default:
      cout<<"rect->setup() found error in script file, 1st char is:"<<ch<<endl; 
      MYEXIT; 
    } // end of switch, ready for new line {should be at '!' now????}
   My->paramFile.getline(junk, MAXLEN, '\n');
   My->paramFile>>ws; // eatwhite, get rid of tabs
   } // while loop terminates when peeks at !end 

   assert(Ms>0.0); // check that you really read material parameters
   // My->paramFile.getline(junk, MAXLEN, '\n'); 
    // so you leave this function just before '!'
   My->outlog<<"about to return from rect->setup(),just read "<<junk<<endl;

}


void Rectang::setAvAndMomentCoef()
{
  //calculate some derived parameters:

//create AvCoef and MomentCoef arrays
  for (int i=0; i<3; i++)
    AvCoef[i] = new mpole(-1); //these coefficients are used in cell::Average()

  for (int i=0; i<6; i++)
    MomentCoef[i] = new mpole(-1);

//initialize them:

  for (int axis=0; axis<3; axis++)
  {
    AvCoef[axis]->SetToPowers(MinCS*0.5);
    MomentCoef[2*axis]->SetToPowers(MinCS*0.5); //negative directions for x, y, z
    MomentCoef[2*axis+1]->SetToPowers(MinCS*0.5); //positive directions

    for (int iVal=0; iVal < My->MyFMM->NumComps; iVal++)
    {
      //for convenience:
      int n0=My->MyFMM->IndArr[iVal][axis],
          n1=My->MyFMM->IndArr[iVal][(axis+1)%3],
          n2=My->MyFMM->IndArr[iVal][(axis+2)%3];

      if (n0%2 && !(n1%2) && !(n2%2))  //non-zero if n0 is odd, n1, n2 are even
        AvCoef[axis]->values[iVal]/=-MinCS[axis]*0.5*(n1 + 1)*(n2 + 1);
      else
        AvCoef[axis]->values[iVal]=0.0;  //zero otherwise

      if (!(n1%2) &&!(n2%2))  //non-zero if n1 and n2 are even
      {
        MomentCoef[2*axis]->values[iVal]*=MinCS[(axis+1)%3]*MinCS[(axis+2)%3]
          *((n0 % 2) ? -1. : 1.)/((n1+1)*(n2+1));  //negate if n0 is odd
        MomentCoef[2*axis+1]->values[iVal]*=MinCS[(axis+1)%3]*MinCS[(axis+2)%3]
          /((n1+1)*(n2+1));
      }
      else
      {
        MomentCoef[2*axis]->values[iVal]=0.0;
        MomentCoef[2*axis+1]->values[iVal]=0.0;
      }
    } //end for loop over iVal
  } //end for loop over axis

  if (My->Psetup)
  {
    My->outlog<<"\nAvCoef mpole:"<<endl;
    for (int i=0; i<My->MyFMM->NumComps; i++)
    {
      My->outlog<<setw(2)<<i;
      My->MyFMM->print("   ", My->MyFMM->IndArr[i]);
      My->outlog<<"->  "<<setw(10)<<AvCoef[0]->values[i]<<"   "<<setw(10)
        <<AvCoef[1]->values[i]<<"   "<<setw(10)<<AvCoef[2]->values[i]<<endl;
    }

    My->outlog<<"\n\nMomentCoef mpole:"<<endl;
    for (int i=0; i<My->MyFMM->NumComps; i++)
    {
      My->outlog<<setw(2)<<i;
      My->MyFMM->print("   ", My->MyFMM->IndArr[i]);
      My->outlog<<"->  ";
      for (int j=0; j<6; j++)
        My->outlog<<setw(10)<<MomentCoef[j]->values[i]<<"   ";
      My->outlog<<endl;
    }
  }  //end of if (My->Psetup)
}

int cell::NextcellID = 0; //initialize static data member
  // if you don't you get a linking error
#ifdef COUPLING
cell* Rectang::LastLayer=NULL; 
#endif

cell::cell(cell *parent1)
{
  child0=NULL; child1=NULL; septum=NULL; rect=NULL;
  InterPartner=NULL; moment=NULL; SmoothField=NULL;
  parent=parent1;
  cellID=cell::NextcellID;
  cell::NextcellID++;
  #ifdef GRAPHICS
  display=0;
  ShowM=0; ShowH=0;
  #endif
  //added by DMAf:
  if(My->MyFMM) {moment = new mpole(-1); SmoothField = new mpole(-1);}
}

cell::~cell()
{ }
 
void cell::check_faces()
{ 
  if(My->Psetup) My->outlog<<"entering check_faces() for cell: "<<this<<endl;
  // debug:
  int NumFaces=0;
  for(int faceN=0; faceN<6; faceN++)
   {  
    NumFaces++;
     assert(cellfacelist[faceN].myFace); // should not be null
     if(My->Psetup) 
       {My->outlog<<"current face is: "<<cellfacelist[faceN].myFace<<", normal is: "
                  <<cellfacelist[faceN].myFace->normal<<", out is: "<<cellfacelist[faceN].out
                  <<", disp is: ("<<cellfacelist[faceN].disp[0]<<","
                  <<cellfacelist[faceN].disp[1]<<","
                  <<cellfacelist[faceN].disp[2]<<")"<<endl;
        My->outlog<<"cellList[0].cel is: "<<cellfacelist[faceN].myFace->cellList[0].cel<<endl;
        My->outlog<<" disp is ("<<cellfacelist[faceN].myFace->cellList[0].disp[0]<<","
                  <<cellfacelist[faceN].myFace->cellList[0].disp[1]<<","
                  <<cellfacelist[faceN].myFace->cellList[0].disp[2]<<")"<<endl;
        My->outlog<<"cellList[1].cel is:"<<cellfacelist[faceN].myFace->cellList[1].cel<<endl;
        My->outlog<<" disp is ("<<cellfacelist[faceN].myFace->cellList[1].disp[0]<<","
                  <<cellfacelist[faceN].myFace->cellList[1].disp[1]<<","
                  <<cellfacelist[faceN].myFace->cellList[1].disp[2]<<")"<<endl;
        }
     cellfacelist[faceN].myFace->sane();
     //cellList[0] is in back of face, check that back of face is "this" 
     assert(cellfacelist[faceN].myFace->cellList[1-cellfacelist[faceN].out].cel == this); 
     //check the cell in front
     if(cellfacelist[faceN].myFace->cellList[cellfacelist[faceN].out].cel)
     assert(cellfacelist[faceN].myFace->cellList[cellfacelist[faceN].out].cel->sane()); 
    }
  assert(NumFaces==6);
}

int cell::sane()
{ // note that we can't call check_faces from this, it would be circular call.
  if(My->Psetup) My->outlog<<"entering cell::sane()"<<endl;
  if (parent)
     assert( (parent->child0==this) || (parent->child1==this) );
   else // no parent
     assert(this==(My->root));
  if (child0) assert(child0->parent==this);
  if (child1) assert(child1->parent==this);
  return 1; 
}

int cell::whichChild()
{  //return 0 if this is child0 of parent
  // if(My->Psetup) My->outlog<<"entering cell::whichChild()"<<endl;
  if(parent)
   {
     //if(My->Psetup) My->outlog<<"this cell has parent"<<endl;
     if(parent->child0 == this) return 0;
     else { assert(parent->child1 == this); return 1; }
   }
  else
   { 
     //if(My->Psetup) My->outlog<<"this cell has no parent, return 0"<<endl;
     return 0;  //in case of root cell
    }
}
 
int cell::whichSide(MVector V) // ASSUMES rectangle or smaller
{  //returns 0 if V is within child0,1 if child1, 2 if neither
 //if(My->Psetup) My->outlog<<"entering cell::whichSide() function"<<endl;
 switch(CellType)
 {
 case 102: // MultiRectangle; not used
   cout<<"error in whichSide"<<endl;
   exit(0); // this  aborts
 case 101: // rectangle, ret 2 if any coord outside box, else check longaxis
   for (int i=0;i<3;i++) if(2.*fabs(V[i]-center[i])>size[i]) return 2;
 default:   
   return V[longaxis]>=
     (septum->edgeList[0].ed->vertexList[0]->its_val(longaxis));
   //could check ANY vertex of ANY edge, same result
 } // end switch(CellType)
}

void cell::GetSizeCenter(int dir) // from children
	  {real dmin,dmax; // used only for non-rectangular cells
	  dmin=min(child0->LowerLeft()[dir],child1->LowerLeft()[dir]);
	  dmax=max(child0->UpperRight()[dir],child1->UpperRight()[dir]);
	  size[dir]=dmax-dmin; center[dir]=0.5*(dmax+dmin);}

cell* cell::SubCellContaining(MVector &v)
   // returns smallest subcell of "this" cell that v is in,NULL if v not in this
   {
   // cout<<"Find SubCellContaining "<<v<<", ptr c="<<this<<", child0="<<child0<<endl;
 //if(My->Psetup) My->outlog<<"entering cell::SubCellContaining() function"<<endl;
 switch(CellType)
 {
 case 102: // MultiRectangle
   cell *result;
   result = child0->SubCellContaining(v); // test if in child0
   if(result) return result;
   result = child1->SubCellContaining(v);
   return result; // even if null
 case 101: // rectangle treated same as smaller cell
 default: // smaller cell
   if (child0)
     switch(whichSide(v))
     { case 0: return child0->SubCellContaining(v); 
       case 1: return child1->SubCellContaining(v);
       default: return NULL;
     }
   else // no child 
     return this; 
 } // end switch(CellType)
} // end SubCellContaining(v)

ostream& operator<<(ostream &fout, cell c){ // overload <<
  fout<<"Cell @ "<<c.center; return fout; }

//added by DMAf
//add magnetostatic field from NearPartner to H_av
void cell::AddNearField(const NearPartner *source)
{
  H_av[0] += source->K->Kx*source->f->magneticCharge;
  H_av[1] += source->K->Ky*source->f->magneticCharge;
  H_av[2] += source->K->Kz*source->f->magneticCharge;
}

//added by DMAf
//recursively update all multipole moments in all cells, to be called before updateStat
void cell::updateMoment()
{  // Assumes M is known for smallest cells.
  SmoothField->SetToZero();
  moment->SetToZero();
  Energy_stat = 0.0;
  H_av.SetToZero(); //althoug H_av is used only for smallest cell,  (DMA)
                      //it is better to initialize it to zero for all cells
  if (child0) //not the smallest cell
  {
    child0->updateMoment(); child1->updateMoment();
    moment->AddShift(child0->moment, child0->center - center);
    moment->AddShift(child1->moment, child1->center - center);
  }
  else //smallest cell, see notes for definition of MomentCoefs
  {
    //initialize the magnetic charges and calculate the moment:
    for (int faceN=0; faceN<6; faceN++)
      if (cellfacelist[faceN].owned)   //this cell owns the face
      {
        if (!My->MyFMM->calculateKernels)
        {  //we need to find the magnetic charge if we do not calculate Kernels
          cellfacelist[faceN].myFace->magneticCharge = 0.0;
          if (cellfacelist[faceN].myFace->cellList[0].cel)  //cel on the left is present
            cellfacelist[faceN].myFace->magneticCharge+=
              cellfacelist[faceN].myFace->cellList[0].cel->M[faceN>>1];
          if (cellfacelist[faceN].myFace->cellList[1].cel)  //cel on the right is present
            cellfacelist[faceN].myFace->magneticCharge-=
              cellfacelist[faceN].myFace->cellList[1].cel->M[faceN>>1];
        }
//if we calculate Kernels the magnetic charges are set in setChargeAtCenterFace,
//which sets charge=1 in a given cell and = 0 in all other cells
          *moment+=*(rect->MomentCoef[faceN])*cellfacelist[faceN].myFace->magneticCharge;
      }   //end of if (owned)
  }  //end of else for smallest cell
}


void cell::updateExch()
{
  if(child0)
  { child0->updateExch(); child1->updateExch(); }
  //We are computing H in lab frame here -- someday we may want to
  //consider using local frame
  //if(My->Pcpt) My->outlog<<" looping over faces to get Exch field "<<endl;
  //we compute the exchange field only for the smallest cells
  //if we want to calculate coarse-grained quantities, we need to remove else
  else // smallest cell
  {
    H_exch.SetToZero();
#ifdef COUPLING
    if (InterPartner) // if interlayer exchange
      {H_exch+=InterPartner->M*InterCoupling;
     // cout<<"updateExch+intLayer "<<H_exch<<", M="<<InterPartner->M<<endl;
      }
#endif
    for (int faceN=0; faceN<6; faceN++) 
      { //loop faces to get exchange field
       //need to find neighbor cell on the other side of this face
       //recall cellfacelist[faceN].myFace is an undirected face
         cell *nbr;
         nbr = cellfacelist[faceN].myFace->cellList[cellfacelist[faceN].out].cel;
         if(nbr)
           H_exch += (nbr->M)*(cellfacelist[faceN].myFace->Jex);
           //contribute to exchange field, Jex is exchange integral
      } //end for
   } // end else smallest cell
} // end updateExch

MVector cell::RandomField()
{
  MVector HR;
  real hx=0,hy=0; //get random h, in local frame where M along z
  GenTwoNorVar(&hx, &hy); //have variance 1
  HR[0] = hx*(rect->H_rand_scale);
  HR[1] = hy*(rect->H_rand_scale); 
  HR[2] = 0;
  //not really kT, but proportional to sqrt(dt*temperature)
  return HR;
}

void cell::update()
//recursively updates cell and all children
//(rotates M, etc).
//This version puts everything (exchange, random) into H field,
//then q rotates around H.
{
  //First calculate M and q. Differently at lowest level.
  if(child0)  // *** NOT SMALLEST CELL ***
   {
    if(My->Pcpt)  My->outlog<<"\ninside update(), not smalllest cell"<<endl;

    child0->update();  child1->update(); // recursive calls

  } //end if not smallest

  else // ***SMALLEST CELL****
       //need calculate M(really q) from LLG eqn.
  {
     switch (My->use_q)
     {
       case 0:  //using dM/dt algorithm
         update_m();
         break;
       case 1:  //using dq/dt algorithm
         update_q();
         break;
       default:
         cout << "Incorrect value of My->use_q" <<endl;
         exit(0);
     }// end of switch
     assert(~My->LastUpdate||My->almost_same(M.Magnitude(),rect->Ms)); //Shuxia_Pc 
      //assert(My->almost_same(M.Magnitude(),rect->Ms)); // M has right length
       assert(My->almost_same(q.norm(),1.0));
	 if(My->LastUpdate) // check this only occasionally
	 {	MVector MfromQ = ((q * (rect->M0)) * (~q)).getMVector();
		if ((MfromQ-M).Magnitude()>0.001*rect->Ms)  My->outlog<<"**** UpdateDiscrep, qM0="<<
		  (q * (rect->M0)) * (~q)<<", M="<<M<<endl; // M,q fit
	 }

  } //end smallest cell

  // Now do things that are the same for small and large cells:

  if(My->CompAvg)
  {
   if((My->nt%My->IncInt)==0 && My->Avgs[CellType])
   {
     real *v = Average::val; //rename for conciseness
     v[0] = (((-M).azimuthal() - M_PI)*180)/M_PI;
     v[1] = M.Magnitude();// /My->Ms;
     MVector MxH = M^H_av;
     MVector MxMxH = M^MxH;
     v[2] = H_av[0];
     v[3] = H_av[1];
     v[4] = H_av[2];
     v[5] = M[0];
     v[6] = M[1];
     v[7] = M[2];
     // to print out single points(normally use), use the following one line of code.
     My->Avgs[CellType]->increment();
    }
   }
} // end function update()


//added by DMA
MVector cell::RandomFieldForM()
{ // find vectors N_1, N_2 both perp. to M
  MVector N_1, N_2, HR;
  if (fabs(M[1]) < 0.9*(rect->Ms)) { // M not along y, cross prod. not zero
    MVector temp (0,1,0);
    N_1=(M^temp).normalize();
    N_2=(M^N_1).normalize();
  }
  else {
    MVector temp (0,0,1);
    N_1=(M^temp).normalize();
    N_2=(M^N_1).normalize();
  }

  assert (N_1*N_2 < 0.05);
  real hx, hy;
  GenTwoNorVar (&hx, &hy);
  HR = N_1*hx + N_2*hy;
  HR *= (rect->H_rand_scale);
  return HR;
}

//added by DMA
void cell::update_m() // assumes M, H_exch, H_av (if FMM) are set
{
  if (My->Pcpt)
  {                   //for debug
    My->outlog<<"\ninside update_m(), smallest cell, cell is: "<<this<<endl;
    q.print(My->outlog, "in update_m, original q ");
    My->outlog<<"CellType is:"<<CellType<<endl;
    M.print(My->outlog, "in update_m(), original M ");
  }
  if (My->FirstUpdate) H_rand = RandomFieldForM(); // don't recalculate 2nd time through
// note that this is random field from t-dt to t

  MVector H,  //total field for LL equation 
          H_anis;  //anisotropy field
  // H_av was calculated in UpdateStat unless FMM not used:
  if(!My->MyFMM) H_av = My->H_ext;
  H_anis = MVector(rect->avEasyAxis*(anisCoef*(M*rect->avEasyAxis))); //anisotropy field
 //cout<<"+++++++++++++++++++++++++++AnisotropyField="<<H_anis<<endl;
  //H = H_av + H_anis + RandomFieldForM() + H_exch;
  H = H_av + H_anis + H_rand + H_exch;
  MVector HxM = H^M;
 // delM = HxM*My->gdt -(HxM^M)*rect->gadt/rect->Ms;
 //Shuxia 
  delM = HxM*My->gdt -(HxM^M)*rect->gadt/rect->Ms -(M^(M^rect->Mp))*My->gamma*rect->aJ*My->dt/rect->Ms;
   
  if(My->FirstUpdate)
   {M = M + delM;
   //M.normalize(); M*=rect->Ms; q.rotateTo(M); //For debugging Shuxia_pc
   if(My->Heun) delMpre=delM; // save delM
   // q.rotateTo(M); // don't fix q on first update too? fixed 7/15/07
   }
  else // must be 2nd pass of Heun algorithm
  M = M + (delM-delMpre)/2;
if (My->LastUpdate) // normalize to keep M from increasing:
{M.normalize(); M*=rect->Ms; q.rotateTo(M);}
//cout<<"First="<<My->FirstUpdate<<" "<<"Last="<<My->LastUpdate<<endl;
// it would be more efficient to have a function that normalizes to a specific value!
  /*M += HxM*My->gdt; //precession part
  M -= (HxM^M)*rect->gadt/rect->Ms; //damping part
  M.normalize();     //to eliminate increasing of its amplitude
  M *= rect->Ms;
  q.rotateTo(M);  //to save running times
 */// Shuxia_pc
  if (My->Pcpt)
  {
    M.print(My->outlog, "in update_m(), M ");
    q.print(My->outlog, "in update_m(), q ");
  }
#ifdef GRAPHICS
  if (this==My->WatchedCell)
  { // logically should be outside fn., BUT, H_anis known only here!
	    // could make them static members of cell...
	 if (My->ShowWatchedHk) My->AnisField->updateVector(H_anis);
	 if (My->ShowWatchedMStatic) My->MStaticField->updateVector(H_av-My->H_ext);
	 if (My->ShowWatchedExch) My->ExchField->updateVector(H_exch);
	 if (My->ShowWatchedTot) My->TotField->updateVector(H);
  }
#endif
} //end of update_m()

//added by DMA
void cell::update_q() // evolves q through dt.
{  // Also calculates M afterwards
    //cout<<"\ninside update_q(), smallest cell, cell is: "<<this<<endl;
    //q.print(cout, "in update_q(), original q ");
    //cout<<", original M = "<<M<<", H_av="<<H_av<<endl;
    //cout<<"CellType is:"<<CellType<<endl;
  MVector H_anis, //anisotropy field
          H_rand; //random field

//fields slowly changing in the lab frame:
//H_av includes the magnetostatic and external fields
  H_anis = MVector(rect->avEasyAxis*(anisCoef*(M*rect->avEasyAxis))); //anisotropy field
  MVector Hs_lab_loc = (((~q)*(H_av + H_anis)) * q).getMVector();

//fields slowly varying in the local frame:
  H_rand = RandomField();
  MVector H_exch_loc = (((~q) * H_exch) * q).getMVector();
  if( My->Proj) H_exch_loc[2] = 0; //project onto xy plane; see paper

//Calculate total field in local frame, for damping
  MVector Hs_loc_loc = H_exch_loc + H_rand;

  MVector Hloc = Hs_lab_loc + Hs_loc_loc;
  Quaternion qprec;
  qprec.qRot((H_av+H_anis)*(My->gdt));

  MVector LocalAxis = -(Hloc.Rot90())*(rect->gadt) + (Hs_loc_loc)*(My->gdt);
  if (My->Proj) LocalAxis[2] = 0.0;
  Quaternion qlocal;
  qlocal.qRot(LocalAxis);
  qprec *= (q*qlocal);
  q = qprec;
  q.normalize1();
  M = ((q * (rect->M0)) * (~q)).getMVector();

  if (My->Pcpt)
  {
    My->outlog<<" at the end M = "<<M<<endl;
    My->outlog<<"\nH_anis = "<<H_anis
      <<", Hexch = "<<H_exch<<", H_rand = "
      <<H_rand<<", H_av = "<<H_av<<endl;
    q.print(My->outlog, "in update_q(), q ");
  }
} //end of update_q()

void cell::getlongaxis(int& diameterInt, int& longaxis)
{ 
  int axis=0; int len=0;  //len is in terms of MinCS
  for (int faceN=0; faceN<6; faceN++)
   {
      cellfacelist[faceN].myFace->getlongedge(len, axis);
      if((len>diameterInt) || ((len==diameterInt) && (axis<longaxis))) 
      {
        diameterInt=len;
        longaxis=axis;
       }
   } 
}

void cell::setCenter(){
  center.SetToZero(); // initialize, then add up centers of 2 end faces
  for (int faceN=0; faceN<6; faceN++)
  {
    if(longaxis==cellfacelist[faceN].myFace->normal)
     center+=cellfacelist[faceN].absCenter(); // this is to common origin
   }
  center*=0.5;
  if(My->Psetup) My->outlog<<"\n center for cell "<<this<<" is, "<<center<<endl;
 }

void cell::split(int lgaxis, int splitPoint1)  //splitPoint was added by DMAg
{
   	if(My->Psetup)
   	My->outlog<<"entering cell::split(), cell being split is: "<<this<<", diameter2="<<diameter2<<endl;
   check_faces();

   //create two child cells
   child0=new cell(this);
   assert(child0);
   //if(My->Psetup) My->outlog<<"child0 pointer is: "<<child0<<endl;
   child1=new cell(this);
   assert(child1);
   //if(My->Psetup) My->outlog<<"child1 pointer is: "<<child1<<endl;

 //we need to pass the pointer to Rectang class down the hierarchy
   child0->rect = rect;
   child1->rect = rect;

 //added by DMA for FMM. We need to know the size of all cells for cell::Near
   child0->size = size;
   child1->size = size;
   child0->size[lgaxis] = splitPoint1*rect->MinCS[lgaxis];
   child1->size[lgaxis] -= splitPoint1*rect->MinCS[lgaxis];
   //if (My->Psetup) My->outlog<<"child0->size = "<<child0->size
   //  <<"child1->size = "<<child1->size<<endl;

   child0->diameter2 = (child0->size).Square();
   child1->diameter2 = (child1->size).Square();

   septum=new face(lgaxis,NULL); //normal should be longaxis of the cell
   assert(septum);

   //loop faces, creating child faces where needed
   for (int faceN=0; faceN<6; faceN++)
   {
    if(lgaxis == cellfacelist[faceN].myFace->normal)
   	    // normal of faces is parallel to longaxis, end faces
            //add the end face to the face list of child0 or child1 
     {
       if((cellfacelist[faceN].out) ==0 ) // left end face, 
            // add it (in the form of a directed face) to the facelist of child0
       {
         //normal*2+out gives the index for this cell, 0 for -x, 1 for x etc.
          child0->cellfacelist[cellfacelist[faceN].myFace->normal*2]
            =DispFace(cellfacelist[faceN]);
          // fix cellList: presently points to parent, out and disp are OK:
          cellfacelist[faceN].myFace->cellList[1].cel = child0;
            // face's cell list now points to child0
          if(My->Psetup)
            My->outlog<<"left end face, push into facelist of child0, face is: "
               <<cellfacelist[faceN].myFace<<", normal is: "<<cellfacelist[faceN].myFace->normal
               <<", out is: "<<cellfacelist[faceN].out<<endl;
       } //end left end face
       else //right end face, add it to facelist of child1
         {
          child1->cellfacelist[cellfacelist[faceN].myFace->normal*2+1]
            =DispFace(cellfacelist[faceN]);
            // put child1 in cellList as cell in back:
          cellfacelist[faceN].myFace->cellList[0].cel = child1;
            // relative displacement (cell-face) = -face, but actually
            // that shouldn't change, so we comment it out:
          if(My->Psetup)
           {
            My->outlog<<"right end face,push into facelist of child1, face is: "
               <<cellfacelist[faceN].myFace<<", normal is: "<<cellfacelist[faceN].myFace->normal
               <<", out is: "<<cellfacelist[faceN].out<<endl;
            }
          } //end right end face
       }    //end if(end faces)
     else   // this is a side face
      {
       if(!cellfacelist[faceN].myFace->face0) // if the face has not been split
            // split the face now
         {
          if(My->Psetup)
          {My->outlog<<"side faces has not been splited,split it now. face is: "
             <<cellfacelist[faceN].myFace<<", normal is: "<<cellfacelist[faceN].myFace->normal
              <<" out is: "<<cellfacelist[faceN].out<<endl;}
           //changed by DMAg: added splitPoint1
          cellfacelist[faceN].myFace->facesplit((lgaxis-(cellfacelist[faceN].myFace->normal)+2)%3, splitPoint1);
            //we are doing a cyclic permutation, transferring from the cell
            //coordinate system(3D) to the face coordinate system(2D).
          } //end if(face not split)
        else // if the face has been splited 
             // It must have been split in a call to cell::split() that split
             // the cell on the other ("out") side of the face.  HOWEVER, that
             // might be the same cell (in a periodic system) in which case the 
             // face was split earlier in this same loop over i.
             // In this "same cell" case the cell was split and the child faces'
             // cellList[] were correctly set, but it does no harm to set it again.
             // In the more common "different cell" case, cellList[] was not
             // set and we have to set it.
         {
          if(My->Psetup)
          {My->outlog<<"side faces has been split,face is: "<<cellfacelist[faceN].myFace
                    <<", normal is: "<<cellfacelist[faceN].myFace->normal
                    <<" out is: "<<cellfacelist[faceN].out<<endl; }
          assert(cellfacelist[faceN].myFace->cellList[1-cellfacelist[faceN].out].cel==this);
                      // sanity check
           // here "1-out" means "in"; cellList[in] should be this cell

          cellfacelist[faceN].myFace->face0->cellList[1-cellfacelist[faceN].out].cel
             = child0; // disp OK
          cellfacelist[faceN].myFace->face1->cellList[1-cellfacelist[faceN].out].cel
             = child1;
          // Note that the CHILDREN of face0 and face1 may still have incorrect
          // cellList[in] (ancestors of the correct cell).         
          // This will be fixed when we split child0, etc.
          // The only better solution would be to set them recursively here.
          }  //end else (face split)
       DispFace *newDispFace;
       newDispFace = new DispFace(cellfacelist[faceN]); // copy constructor
       assert(newDispFace);
          // for children's facelists:
       newDispFace->myFace = cellfacelist[faceN].myFace->face0; 
         // change to child face, out and disp stay the same
       child0->cellfacelist[newDispFace->myFace->normal*2+newDispFace->out]
            =*newDispFace;
       newDispFace = new DispFace(cellfacelist[faceN]);
       assert(newDispFace);
       newDispFace->myFace = cellfacelist[faceN].myFace->face1;
       child1->cellfacelist[newDispFace->myFace->normal*2+newDispFace->out]
            =*newDispFace;
          //centerEdge of this face is an edge of the septum; need its index i
          // convert iter's normal to face frame of septum
       int norm = (cellfacelist[faceN].myFace->normal - septum->normal + 2)%3;
          //edge is perpendicular to norm(low bit of i)
       int ilow = 1- norm; assert(ilow>=0); 
          //ilow is the direction of the edge
          //high bit is determined by out of face
       int i = (cellfacelist[faceN].out << 1) + ilow ;
       assert (i>=0); assert (i<=3); //sanity checks
          //Finally, insert centerEdge into edgeList of septum;
       septum->edgeList[i] = DispEdge(cellfacelist[faceN].myFace->centerEdge,
         cellfacelist[faceN].disp); 
          // displacement of edge borrowed from parent face
      }//end else(side faces)
    } //end for(loop over faces)
    septum->setFaceCenter();
    septum->cellList[0].cel = child0; // no displacement possible here
    septum->cellList[1].cel = child1; // (disp=0 in constructor)
    septum->sane();
    if(My->Psetup)
    My->outlog<<"septum is: "<<septum<<", normal is: "<<septum->normal<<endl;

    //left & right cells
    // create 2 directed faces, one for each child cell:
    child0->cellfacelist[septum->normal*2 + 1] = DispFace(septum,1);
    if(My->Psetup)
      My->outlog<<"push septum into facelist of child0 (out 1)"<<endl;
    child1->cellfacelist[septum->normal*2] = DispFace(septum,0);
    if(My->Psetup)
      My->outlog<<"push septum into facelist of child1 (out 0)"<<endl;
    child0->check_faces();
    child1->check_faces();
    int lengthInt0 = 0; int dir0 = 0;
    child0->getlongaxis(lengthInt0, dir0);
    child0->longaxis = dir0;
    	if(My->Psetup)
    	My->outlog<<"child0->longaxis is: "<<child0->longaxis<<endl;
    int lengthInt1 = 0; int dir1 = 0;
    child1->getlongaxis(lengthInt1,dir1);
    child1->longaxis = dir1;
    	if(My->Psetup)
    	My->outlog<<"child1->longaxis is: "<<child1->longaxis<<endl;
    //set center for displaying the M, H vectors in a cell 
    child0->setCenter();
    child1->setCenter();
}

void cell::splitRecursively()
{ 
  if(My->Psetup)
  My->outlog<<"\n entering splitRecursively for cell: "<<this<<endl;
  int lengthInt = 0; int dir = 0;
  getlongaxis(lengthInt, dir);

  if (lengthInt>1)   //this is not the smallest cell, need to split:
  {
    int splitPoint = 1;

    while (2*splitPoint < lengthInt)
      splitPoint*=2;
    split(longaxis, splitPoint);
    child0->splitRecursively();
    child1->splitRecursively();
  }
  else  //smallest cell
    {
     #ifdef GRAPHICS
     display=1;
     ShowM=1; // ShowH=1;
     #endif
     }
}




void cell::initIDRecursively()
{
  cout<<"entering initIDRecursively for cell: "<<this<<endl;
   // for all cells:
   My->CellFromID[cellID]=this; // store pointer in array
  if(child0 || child1) //not smallest cell, call itself recursively
   {
     child0->initIDRecursively();
     child1->initIDRecursively();
     // cout<<"in initIDRecursively, not smallest cell, called itself recursively"<<endl;
   }
  // else {}//smallest cell, may be things to init but don't know any
  
} // end of initIDRecursively

void cell::initMRecursively()
{  // called only within each rectangle.  Init M, also fix MinCS=smallest
	My->outlog<<"entering initMRecursively for cell: "<<this<<"->"<<*this<<endl;
  if(child0 || child1) //not smallest cell, call itself recursively
   {
     child0->initMRecursively();
     child1->initMRecursively();
	 My->outlog<<"in initMRecursively, not smallest cell, called itself recursively"<<endl;
    }
  else //smallest cell, init M and q
  {
     if(My->Psetup)
      {
        My->outlog<<"center is: "<<center<<endl;
        My->outlog<<"rect->kVector is: "<<rect->kVector<<endl;
        My->outlog<<"rect->kVector*center is: "<<rect->kVector*center<<endl;
        My->outlog<<"rect->SpinPh is: "<<rect->SpinPh<<endl;
        My->outlog<<"rect->kVector*center + rect->SpinPh is: "
            <<rect->kVector*center + rect->SpinPh<<endl;
        My->outlog<<"smallest cell, init q, cellID is: "<<cellID<<endl;
        }

      Quaternion qSpin;
	  real phase = rect->kVector*center + rect->SpinPh;
      qSpin.qRot(MVector(-(rect->SpinAy)*sin(phase),rect->SpinAx*cos(phase), 0));
      q = (rect->q_init)*qSpin;
      q.print(My->outlog,"q is"); // or does this need to be real file, My->outlog?
            //this rotates magnetization away from standard position 001
            //by angle SpinA toward x direction,
            //then precess by k*r(dot product) + phase
      M = ((q * (rect->M0)) * (~q)).getMVector();
      My->outlog<<"rect->M0="<<rect->M0<<", new M is:"<<M<<endl;
	  // Set cell::anisCoef
	  real x=center[0]-rect->WholeRect->LowerLeft()[0]; // assume gradient along x
	  if (rect->AnisPower) anisCoef= rect->AnisPower->value(x);
	  else MYEXIT; // no other possibilities, now
	  My->outlog<<"anisCoef of this cell = "<<anisCoef<<endl;
	  rect->MinCS = size; // redundant of course, all sizes same, use last one.
   } // end else smallest
} // end of initMRecursively

void cell::initJexRecursively()
{
  if(My->Psetup)
  My->outlog<<"entering initJexRecursively for cell: "<<this<<"**************"<<endl;
  if(child0) //not smallest cell, call itself recursively
   {
     child0->initJexRecursively();
     child1->initJexRecursively();
     if(CellType != 101) CellType = child0->CellType + 1; 
	 // so CellType measures your distance up the tree from smallest cell
     // except 101 is "Rectangular" cell, CellType was set earlier, don't overwrite it
     assert(CellType <= Max_Type);
     if(My->Psetup)  My->outlog<<"CellType is: "<<CellType<<endl;
     }
   else // smallest cell
   {
     if (CellType != 101) CellType = 0; 
     // set CellType to indicate smallest cell, but don't overwrite rectangular CellType (101)
     if(My->Psetup) My->outlog<<"CellType is(smallest): "<<CellType<<endl;
  
   for(int faceN=0; faceN<6; faceN++)
   {
     if(My->Psetup)
		 My->outlog<<"setting Jex for face "<<cellfacelist[faceN].myFace->faceCenter<<endl;
     if(cellfacelist[faceN].myFace->cellList[0].cel && cellfacelist[faceN].myFace->cellList[1].cel)
     { // if cells on both sides of face exist
       if(My->Psetup)
       {
        cellfacelist[faceN].myFace->cellList[0].cel->center.print(My->outlog, 
                                    "cellfacelist[faceN].myFace->cellList[0].cel->center");
        My->outlog<<" disp is: ["<<cellfacelist[faceN].myFace->cellList[0].disp[0]<<","
                  <<cellfacelist[faceN].myFace->cellList[0].disp[1]<<","
                  <<cellfacelist[faceN].myFace->cellList[0].disp[2]<<"]"<<endl;
        cellfacelist[faceN].myFace->cellList[1].cel->center.print(My->outlog, 
                                    "cellfacelist[faceN].myFace->cellList[1].cel->center");
        My->outlog<<" disp is: ["<<cellfacelist[faceN].myFace->cellList[1].disp[0]<<","
                  <<cellfacelist[faceN].myFace->cellList[1].disp[1]<<","
                  <<cellfacelist[faceN].myFace->cellList[1].disp[2]<<"]"<<endl; 
        }
       MVector center0 = My->Displace(cellfacelist[faceN].myFace->cellList[0].cel->center, 
                                      cellfacelist[faceN].myFace->cellList[0].disp);
       MVector center1 = My->Displace(cellfacelist[faceN].myFace->cellList[1].cel->center, 
                                      cellfacelist[faceN].myFace->cellList[1].disp);
       real a = (center0 - center1).Magnitude();
       assert(a);
       cellfacelist[faceN].myFace->Jex = rect->ExchangeA/(mu_0*rect->Ms*rect->Ms*a*a);
       if(My->Psetup) My->outlog<<" a = "<<a<<", Jex = "<<cellfacelist[faceN].myFace->Jex<<endl;
      } // end if both cells exist
     else // one cell missing, no exchange possible
     { 
       cellfacelist[faceN].myFace->Jex = 0;
       if(My->Psetup) My->outlog<<" no neighbor, Jex = "<<cellfacelist[faceN].myFace->Jex<<endl;
      }
      //set the ownership:  //DMA
      if (this==cellfacelist[faceN].myFace->cellList[0].cel)
           //the cell is to the left of the face; in this case it is owned only if the other
           //cell is missing
        cellfacelist[faceN].owned = (!cellfacelist[faceN].myFace->cellList[1].cel);
      else //the cell is to the right of the face; it always owns the face
        cellfacelist[faceN].owned = 1;
   } // end for(face) loop
   } // end if(smallest cell)
   if(My->Psetup)
     My->outlog<<"returning from initJexRecursively() for cell"<<this<<"***********************"<<endl;
} // end function initJexRecursively


//added by DMAf, returns 1 if source1 is close to this
int cell::Near(const DispCell &source1)
{
  real Diam2=(My->MyFMM->MaxAngle2)*((source1.absCenter()) - center ).Square();
// max allowed squared diameter
  return ( (diameter2>Diam2) || (source1.cel->diameter2>Diam2) );
}

//added by DMAf
int operator<(const int3 &left, const int3 &right) //needed for map operations
{
  for (int i=0; i<3; i++)
    if (left.values[i] < right.values[i])//if wrong then left>=right
      return 1;
    else
      if (left.values[i] > right.values[i]) //if left > right return 0
        return 0;
    return 0;//if left==right
}


//used to average SmoothField
MVector cell::averageMpole (const mpole *mpoleInput)  const
{
  MVector averageField;
  averageField[0] = *(rect->AvCoef[0])*(*mpoleInput);
  averageField[1] = *(rect->AvCoef[1])*(*mpoleInput);
  averageField[2] = *(rect->AvCoef[2])*(*mpoleInput);
  return averageField;
}



//added by DMAf
void cell::CullPartners() // called for root, before first updateStat call.
{  // on entry, assume Partners list contains all cells near parent
  // i.e. sources whose field is not included in parents SmoothField.
  // On exit, Partners list should contain only sources in this cell's
  // SmoothField.
  std::list<DispCell>::iterator j;
  std::map<int3, Kernel>::iterator iterMap;//iterator for Kernels
//we need to clear the children's lists: (this is used for M-based FMM)
  cout<<"Entering CullPartners for cell "<<cellID<<" at "<<center<<endl;
  if (child0)
  {
    child0->Partners.clear();
    child1->Partners.clear();
    child0->NearPartners.clear();
    child1->NearPartners.clear();
  }
  if (My->MyFMM->draftFMM)  //added by DMAfs
  {      //if we use draft FMM the partners list must have only this cell for
    Partners.clear();                     //smallest cells
    if (CellType == 0) 
      Partners.push_back(DispCell(this, ZeroDisp));
  }

  j = Partners.begin();

  while (j!=Partners.end()) // loop partners
  {
    //cout<<"Partner = cell #"<<j->cel->cellID<<" at "<<j->cel->center<<endl;
    if (Near(*j)) //very close, not in the crescent region
    {// don't give up yet, children might be far
      if (j->cel->diameter2 > diameter2 && j->cel->child0) // split it if big
      {
        //cout<<"Partner big, split it & go on to new partners (including children)"<<endl;
        Partners.push_back(*(j->GetChild0()));// add to list;
        Partners.push_back(*(j->GetChild1()));
        j = Partners.erase(j);//returns iterator that points to the next node
      }
      else // if already small, must pass to children
      {
        if (child0)
        {
          child0->Partners.push_back(*j);
          child1->Partners.push_back(*j);
          j = Partners.erase(j);//returns iterator for next node
        }
        else //smallest cell, need to put this cell to NearPartners
        {
          cout<<"this=smallest; rect="<<rect<<", split partner rect="<<
			  j->cel->rect<<" to faces"<<endl;

          for (int faceN=0; faceN<6; faceN++)  //loop over faces
            if (j->cel->cellfacelist[faceN].owned)   //need to insert to NearPartners
            {
			  // Calculate distance between face and cell, to get kernel
		      // Later, key may include rect->MinCS and j->cel->rect->MinCS, both /quantum
              // key for Kernel in double space to ensure integral values
              MVector dispFaceCenter = j->absCenter()-j->cel->center
                 +j->cel->cellfacelist[faceN].absCenter();
              assert (My->almost_same ((dispFaceCenter-j->absCenter()).Magnitude(),
                  0.5*rect->MinCS[faceN/2]));
              int3 doubleKey = (dispFaceCenter - center).ToInt (My->quantum*0.5);
              //cout<<"CullP found face of partner at "<<dispFaceCenter<<
              //   ", cell="<<center<<endl;
              //cout<<"& sep="<<doubleKey<<", look for kernel"<<endl;
              iterMap = My->MyFMM->Kernels.find (doubleKey);
              if (iterMap==My->MyFMM->Kernels.end())  //key was not found
                if(My->MyFMM->calculateKernels) //we calculate Kernels
                {
                  Kernel Ktemp;
                  Ktemp.SetToMonopoleField(doubleKey);
                  My->MyFMM->Kernels[doubleKey] = Ktemp;
                  My->MyFMM->newKernels[doubleKey] = Ktemp;  //initiliaze newKernels
                  iterMap = My->MyFMM->Kernels.find (doubleKey);
                  assert (iterMap!=My->MyFMM->Kernels.end());
                }
                else
                {
                  cout<<"ERROR, see log"<<endl; // crashes so this isn't read
				  My->outlog<<"ERROR: Kernel for separation "<<doubleKey<<
					  " was not read in. Have you specified the correct *.ker"<<
					  " file in script file?"<<endl;
                  exit(12);
                } // now, if kernel found:
              NearPartners.push_back(NearPartner(j->cel->cellfacelist[faceN].myFace,
                 &iterMap->second));
              // cout<<"Found kernel:"<<iterMap->second<<endl;
           }  //end of if owned
         j=Partners.erase(j);  //erases the object and sets j to the next one
          
        } //end of else for smallest cell
      }  //end of else if already small
    } // end if Near(j)
    else //it is far
      j++;// if far, leave source j in Partners list
  } // end for while
  //cout<<"End loop over partners of cell #"<<cellID<<", call CullPartners for children"<<endl;

  if (child0)
  {
    child0->CullPartners();
    child1->CullPartners(); // recurse
  }
} // end CullPartners()


//added by DMAf
real cell::updateStat()
{
  if (My->Pcpt)
     My->outlog<<"\nBEGIN updateStat for center = "<<center<<", size = "<<size<<endl;
  if (parent) {
            //parent is present, need to initialize SmoothField to that of parent
            // with proper shift
    My->MyFMM->scratchMpole2->SetToPowers(center-parent->center);
      //to translate the origin of SmoothField from parent, need to be mpole
    SmoothField->AddDownConv(parent->SmoothField, My->MyFMM->scratchMpole2);
      //translated SmoothField from parent
  }

  else   //no parent, root cell, we need to take into account exterior potential
  if (My->periodic[0] || My->periodic[1] || My->periodic[2])
  { 
    SmoothField->invert();  //DMAh: we need to invert only DownConv(Vext, moment)
    SmoothField->AddDownConv (My->MyFMM->Vext, moment);
    SmoothField->invert();
  }

  std::list<DispCell>::iterator iterP;
  std::list<NearPartner>::iterator iterN;
//need to add the magnetostatic field from Partners to this. This is smooth field since
//Partners are far
  if (My->Pcpt)
    My->outlog<<"\n\nupdateStat, center = "<<center<<" H_av = "<<H_av
     <<" before adding the field from Partners"<<endl;

  for (iterP=Partners.begin(); iterP!=Partners.end(); iterP++)
  {
    if (My->Pcpt)
    My->outlog<<"\n\nupdateStat, loop over Partners, iterP->absCenter is: "<<iterP->absCenter()<<endl;
    if (My->Pcpt)
      My->outlog<<"\tPartner of "<<iterP->cel->CellType
        <<" type "<<iterP->cel->size<<" size, at "<<iterP->absCenter()
        <<" with moment = "<<iterP->cel->moment->values[0]
        <<" "<<iterP->cel->moment->values[1]
        <<" "<<iterP->cel->moment->values[2]
        <<" "<<iterP->cel->moment->values[3]
        <<"  SF = "<<SmoothField->values[0]
        <<" "<<SmoothField->values[1]<<" "<<SmoothField->values[2]
        <<" "<<SmoothField->values[3]<<endl;
    SmoothField->AddTaylor(iterP->cel->moment, center-iterP->absCenter());//DMApf
  }

  if (!child0)
  //smallest cell, base case for recursion, need to add field from NearPartners
  {
    H_av = averageMpole(SmoothField); 
       //we do not need Taylor expansion for smallest cells
   if (My->Pcpt)
     My->outlog<<"H_av = "<<H_av<<", adding the field from NearPartners"<<endl;

    for (iterN=NearPartners.begin(); iterN!=NearPartners.end(); iterN++)
	{AddNearField(&*iterN);  
	// cout<<iterN->f->faceCenter<<endl;
	}
    Energy_stat-=0.5*(moment->values[1]*H_av[0] + moment->values[2]*H_av[1]
      + moment->values[3]*H_av[2]);  //demagnetizing energy: 1/2 factor
  } //end for if(!child0)
  else {//not smallest cell, need to recurse
    Energy_stat+= child0->updateStat() + child1->updateStat();
    H_av = (child0->H_av + child1->H_av)*0.5;  //average field of children
  }
  return Energy_stat;
}

void cell::parseBuild()
{  	//parses Build Command, puts subtree under "this",
	//assumes input stream pointer at beginning of line (before tabs)
        // leaves input stream pointer at the beginning of the next line
  char ch;
  if(My->Psetup) My->outlog<<"enter parseBuild() function"<<endl;
  cout<<"In parseBuild:";
  char junk[MAXLEN+1];
  My->paramFile>>ws; // eatwhite, get rid of tabs
  // cout<<"After ws";  
  //  for (int i=0; i<10; i++) {My->paramFile>>ch; cout<<ch<<"(#"<<(short)ch<<")";}
	  //cout<<"| was next"<<endl; // dumps 10 characters to cout

  while ((ch = My->paramFile.peek()) != '!')   // '!' makes it go back one level
    // in parsing recursion, i.e. returns to the calling ParseBuild
  {cout<<"In parseBuild, read char #"<<(int)ch<<":"<<ch<<endl;
   switch(ch)
   {
    case '#': 
      My->paramFile.getline(junk, MAXLEN, '\n');
      break;
    case 'N': // NonRectBuild, construct children & build them
      if(My->Psetup) My->outlog<<"in parseBuild(), in case N"<<endl;
        //parseBuild assumes we're at the beginning of a line.
      // readSize(); // end of line used to has size info, even for non-rect cell
	  My->paramFile>>junk>>ShowM; // 1 if we show M of this composite cell
	  assert(junk[1]='o'); // as in Non...
      My->paramFile.getline(junk, MAXLEN, '\n'); //EOL,ready to call parseBuild

      CellType = 102;
      child0 = new cell(this); // parent is "this" cell
      child0->parseBuild(); 
	// for 0th child, parseBuild leaves us at the beginning of a line.
      child1 = new cell(this);
      child1->parseBuild(); // for first child
	  // Now have both children, construct parent size
	  GetSizeCenter(0); GetSizeCenter(1); GetSizeCenter(2); // x,y,z coords
      My->outlog<<"parseBuild got non-rect size="<<size<<"center="<<center<<endl;

      break;

    case 'R': // RectBuild, construct entire rectangle
      readSize(); // size info at end of line
          //create an instance of Rectang to store all necessary parameters
      rect = new Rectang;
	  rect->WholeRect=this; // pointer to corresponding cell
	  CellType = 101; // i.e., "rectangle"
      cout<<"CellType is : "<<CellType<<", read in cell size: "<<size<<endl;

	  // can't do this until CellfromID is initialized!: My->ListCells(cout);
	  My->outlog<<"Now list cells recursively:"<<endl;
	  My->root->ListCellsRecursively(); // to debug
      assert (rect);
#ifdef COUPLING
      rect->PreviousLayer = rect->LastLayer; // remember order of deposition
      rect->LastLayer = this; // is pointer to cell, not rectangle
	  // this is not in setup() so we can access "this" cell
#endif


      rect->setup(); // reads more lines; separate function to refer to rect
        // reads the rest of the lines relating to this rect, until '!'

      // gone: rect->readParameters (My->paramFile);    //read the parameters;
	    // this is separate function so it can refer to rect easily

//changed by DMAk; NOT debugged recently (8/05)
//root cell size has been set in case 'P', if we calculate Kernels we leave it unchanged
      if (My->MyFMM) 
	  if (My->MyFMM->calculateKernels)   //we calculate Kernels, added by DMAk, DMAfs
      {  //we need to find the smallest required size of the system
        assert(this==My->root); 
        real dMin = sqrt (rect->MinCS*rect->MinCS/My->MyFMM->MaxAngle2);
                                    //MinCS*MinCS is diameter2 for smallest cell
        int id[3];                  //size of the system in units of smallest cell
        id[0]=id[1]=id[2]=4;        //initialization
        for (int iSizeDet=0; iSizeDet<3; iSizeDet++)
        { 
          while (rect->MinCS[iSizeDet]*0.5*id[iSizeDet] < dMin)
            id[iSizeDet]*=2;
       //we use coarse grain cells for Kernel calculation (must be 2 times larger)
          My->PeriodVec[iSizeDet] = 2.*id[iSizeDet]*rect->MinCS[iSizeDet];
          size = My->PeriodVec;
          center = size/2.;
        }
        cout<<"the size of the system is found: "<<My->PeriodVec[0]<<" x "
          <<My->PeriodVec[1]<<" x "<<My->PeriodVec[2]<<endl;
      }
      //find the largest size of the cell (used to scale the M vector in Arko's experiment)
      
        assert(rect); 
		real lengthScale;
	    lengthScale = rect->MinCS[0]; // set to largest side (will stick out on others)
        for (int iSide=1; iSide <3; iSide++)
          if (lengthScale < rect->MinCS[iSide]) lengthScale = rect->MinCS[iSide]; 
        lengthScale*= 0.45; //take <half of that
        My->GlobalMScale = lengthScale/rect->Ms; // determined by LAST rectangle
  	      //so you can multiply this by M and get the right length 
      
      if(My->Psetup) 
      {
        My->outlog<<"in parseBuild(), in case R, CellType is:  "<<CellType<<endl;
        size.print(My->outlog, "this->size");
      }

      initMRecursively(); // to constant or spin wave, as read in rect->init.
	    //  May be overwritten later by initial condition file, or option 'c'.

/* NO longer test aspect ratio; might if quantum is MVector.
if (!My->MyFMM->calculateKernels) //if we read the kernels in
  { // test that aspect ratio agrees.  Here MinCS must be ACTUAL small cell size.
    real eqCube = (real) pow((double)(rect->MinCS[0]*rect->MinCS[1]*rect->MinCS[2]), (double)1/3.);
	  // for some reason VC++ won't take (real)
     for (int i=0; i<3; i++)
    assert (My->almost_same(My->MyFMM->aspectRatio[i], rect->MinCS[i]/eqCube));
  }
*/
	  
	  break;

    default:
      cout<<"Error in parseBuild() input file, 1st char is:"<<ch<<endl; 
      MYEXIT; // exit(2);
    } // end of switch, should be at '!' now
   My->paramFile>>ws; // eatwhite, get rid of tabs
   } // while loop terminates when peeks at !end 
   My->paramFile.getline(junk, MAXLEN, '\n'); 
    // so you leave this function at beginning of next line
   if(My->Psetup) My->outlog<<"just read "<<junk<<", shd be '!', about to return from parseBuild-ing cell "<<*this<<endl;
} // end parseBuild()

#ifdef COUPLING
void cell::SetInterPartners() 
   // sets inter-layer interaction recursively   
   // "this" is initially rectangular upper layer,
   // in recursion becomes small piece of it.
   {
   cout<<"Enter SetInterPartners for "<<*this<<endl;
   if(child0) {child0->SetInterPartners();
      child1->SetInterPartners();}
   else // smallest cell, need actual partner
     {
     assert(rect->spacing[0]==0.0); 
     assert(rect->spacing[1]==0.0);
       // need rewrite this if spacing isn't in the z direction 
     MVector PartnerLoc=center-MVector(0,0,0.51*size[2]+rect->spacing[2]);
         // this should be slightly (0.01*size) inside the previous layer 
     assert(rect->PreviousLayer);
     cout<<"Seek point "<<PartnerLoc<<" in Prev.Layer "<<*(rect->PreviousLayer)<<endl;
     InterPartner = rect->PreviousLayer->SubCellContaining(PartnerLoc);
     // cout<<"Sub.. found point "<<PartnerLoc<<" in cell InterPartner="<<*InterPartner<<endl;
     assert(InterPartner);
     InterCoupling=rect->tJ/size[2]; // assume z direction again

     InterPartner->InterPartner=this; // must point back to this
     InterPartner->InterCoupling=rect->tJ/InterPartner->size[2];
     cout<<"Set up coupling, InterPartner = "<<*InterPartner
             <<", InterCoupling = "<<InterCoupling<<endl; 
     // cout<<"Partner's partner="<<*(InterPartner->InterPartner)<<", coupling = "<<InterPartner->InterCoupling<<endl;
     assert(InterPartner->InterPartner==this);
     } 
   } // end SetInterPartners
#endif
     
void cell::readSize() // reads the end of the line with size
   // and center information
{
      real L1x, L1y, L1z, R1x, R1y, R1z;
      char junk[MAXLEN+1];
      My->paramFile>>junk>>L1x>>L1y>>L1z>>R1x>>R1y>>R1z;
      size[0] = R1x - L1x;
      size[1] = R1y - L1y;
      size[2] = R1z - L1z;
      center[0] = (R1x + L1x)*0.5; // this may be overwritten for rectangles
      center[1] = (R1y + L1y)*0.5; 
      center[2] = (R1z + L1z)*0.5;
      diameter2 = size.Square(); 
}

void cell::buildRect(MVector Ori)
{
   if(My->Psetup) My->outlog<<" enter buildRect() "<<endl;
	   // create vertices, edges and faces of current cell
   MVector* CellVertices[8]; 
       	   // this is LOCAL to buildRect(),permanent storage is edge::vertexList
           // note that in a periodic system, all vertices are equivalent, 
           // tho stored separately
   for (int i=0; i<8; i++) //create vertices in loop
   { //extract 3 bits from i, these are x,y,z coordinates
     //in binary number 011, these are zyx, not xyz, since rightmost bit is #0.
     CellVertices[i]=new MVector((i&1)*size[0], ((i >> 1)&1)*size[1],((i >> 2)&1)*size[2]);
     assert(CellVertices[i]);
     *CellVertices[i] += Ori;
    }
  
  //create RectPeriodic[3], it is set only when the system is periodic and the 
  //System contains only this Rectang
  int RectPeriodic[3]; 
  for (int i=0; i<3; i++)
   {
    RectPeriodic[i]=((size[i] == My->PeriodVec[i]) && My->periodic[i]);
    // cout<<"RectPeriodic["<<i<<"] is: "<<RectPeriodic[i]<<endl;
    }

  //Now create edges
  DispEdge CellEdges[12]; // creates 12 DispEdge structures, not just pointers
  for (int dir=0; dir<3; dir++) //create 4 edges in direction "dir"
  {
     // In a coordinate system with dir along the 0 axis, the "left" ends of the
     // 4 edges are at zyx = 000,010,100,110, the right ends at 001,011,101,111.
     // edge index has bits (high to low) (0..2=dir),l1,l0

     // Below is modified for periodic bc's:
     int d0=(dir+1)%3; int d1=(dir+2)%3; // define axes d0,d1 normal to edge
     
     for (int l0=0;l0<2;l0++) // loop displacement along dir+1
     {
       for (int l1=0;l1<2;l1++)
       {
         int e = (dir<<2)+(l1<<1)+l0; // edge index
         if(My->Psetup) My->outlog<<"\n Constructing edge #"<<e;
         int eEquiv=-1; // means there is no equivalent edge
         if (RectPeriodic[d0] && l0)  // had an equivalent edge before
         {
           eEquiv=e-1; CellEdges[e].disp[d0]=1; // and this one is displaced
           if(My->Psetup){
               My->outlog<<", CellEdges["<<e<<"] is displaced along "<<d0
               <<", displacement is: "<<CellEdges[e].disp[d0]<<endl; }
          }
         if (RectPeriodic[d1] && l1) 
		//if both are true, e-1 and e-2 are equivalent!
         {
           eEquiv=e-2; CellEdges[e].disp[d1]=1; // displaced in d1 direction
           if(My->Psetup){
               My->outlog<<", CellEdges["<<e<<"] is displaced along "<<d1
               <<", displacement is: "<<CellEdges[e].disp[d1]<<endl; }
          }
         if (eEquiv<0)  // didn't find equivalent one, must construct edge
         {
           int i=l0*(1<<d0) + l1*(1<<d1); // index of low vertex
           CellEdges[e].ed = new edge(CellVertices[i],CellVertices[i+(1<<dir)]);
           assert(CellEdges[e].ed);
           CellEdges[e].ed->orien = dir;
           if(My->Psetup){
                My->outlog<<", Couldnt find equivalent edge, making new one,"<<endl;
                    CellEdges[e].print(); }
          }
         else // found equivalent edge
         {
           CellEdges[e].ed = CellEdges[eEquiv].ed;
           if(My->Psetup){
                    My->outlog<<" Found equivalent edge, #"<<eEquiv<<": "; 
                    CellEdges[e].print(); }
          }
        } // end l1 loop
      } //endl l0 loop

   } // end dir loop

  //Now create faces
  face *newf; //dummy
  for (int norm=0; norm<3; norm++) //normal to face (x is 0, etc)
  {
    // In "face frame", normal is direction 2.
    // The directions that are called 0 and 1 in the face frame are:
    int d0=(norm+1)%3;
    int d1=(norm+2)%3;
    for (int ni=0; ni<2; ni++) 
    //loop over 2 faces with this out direction, ni=0,1
    // so ni is the coordinate along the normal direction
    {
      DispFace *newDispFace;
      newDispFace = new DispFace(); 
      assert(newDispFace);
       // this constructor just sets disp=0
       // Set properites of newDispFace before pushing it onto cellfacelist:

      if (RectPeriodic[norm] && ni)// this is not really a new face
      {
         // newf still refers to the ni=0 face
         newDispFace->myFace = newf; newDispFace->out = ni;
         newDispFace->disp[norm]=1; // other 2 components remain 0
         if(My->Psetup) {
                   My->outlog<<"Found equivalent face, DispFace norm is: "
                   <<newDispFace->myFace->normal<<", out is: "<<newDispFace->out
                   <<", disp is: ["<<newDispFace->disp[0]<<","
                   <<newDispFace->disp[1]<<","<<newDispFace->disp[2]<<"]"<<endl;
                   }
       }
      else // it is a new face
      {
         newf = new face(norm,NULL); // NULL means no parent
         assert(newf);
           // The edges are labeled 0,1,2,3 in a standard way in the face frame;
           // the labels (e, above) in the cell frame are different.
           // We must calculate the e labels e0, e1, e2, e3 of these 4 edges.
           // e0 is along 0 direction in face frame, but d0 in cell frame.
           // Coords to put in e index are d1 (least sig.) and norm (most sig.)
           // Coordinate along norm is ni, coordinate along d1 is zero:
         int e0 = (d0 << 2) + 2*ni; // e2 same,but coord[d1, least sig.]=1
           // Now e1 is along d1,
           // e index has norm (least sig., value ni) and d0 (most, value 0)
         int e1 = (d1 << 2) +ni; 
           // e3 is same, but coord[d0,most sig.]=1, so add 2.
         newf->edgeList[0] = CellEdges[e0]; // a list object, with edge and disp
         newf->edgeList[1] = CellEdges[e1]; // disp defaults to 0
         newf->edgeList[2] = CellEdges[e0+1];//1 displaces along d1 axis
         newf->edgeList[3] = CellEdges[e1+2];//2 displaces along d0 axis
           //set front and back cells for this new face, first check if this is
           //a periodic system, if so, set both cellList to be the current cell
         if(RectPeriodic[norm])
          {
           newf->cellList[ni].cel = this;
             //now set displacement
           newf->cellList[ni].disp[norm] = 2*ni-1;
           }
         newf->cellList[1-ni].cel=this;
         if(My->Psetup){
                   My->outlog<<"newf is created,norm is:"<<norm<<",edgeList is:"
		             <<endl;
                   newf->edgeList[0].print(); newf->edgeList[1].print();
                   newf->edgeList[2].print(); newf->edgeList[3].print(); }
                   
         newf->sane();
         newf->setFaceCenter();
         newDispFace->myFace = newf; newDispFace->out = ni;
         //My->outlog<<"newDispFace is created,out is:"<<newDispFace->out<<endl;
       } //end else
      this->cellfacelist[newDispFace->out+newDispFace->myFace->normal*2] = *newDispFace; 
         // insert new face in Cell's face list
     }   //end loop over ni
   } //end loop over norm
  int lengthInt = 0; int orien =0; 
  getlongaxis(lengthInt, orien);
  longaxis = orien;

  if(My->Psetup) My->outlog<<"this->longaxis is: "<<this->longaxis<<endl;
  setCenter();
  center.print(My->outlog, "center");

  splitRecursively(); // wait to initJexRec. until read A, Ms!
 
  if(My->Psetup) My->outlog<<"about to return from buildRect()"<<endl;
}


//added by DMA to set the magnetic charges to zero except
//the center 2x2 face, in which it is set to 1.
//KernelType gives the normal of this face
void cell::setChargeAtCenterFace (int KernelType)
{
  if (child0)
  {
    child0->setChargeAtCenterFace (KernelType);
    child1->setChargeAtCenterFace (KernelType);
  }
  else //smallest cell
    for (int faceN=0; faceN<6; faceN++)
      cellfacelist[faceN].myFace->magneticCharge=0.;

  if (this==My->root) //we need to do this only once
    for (int cellIndex=0; cellIndex<4; cellIndex++)
      My->MyFMM->centerFaces[KernelType][cellIndex]->magneticCharge=1.;

}


//added by DMAk
//function to get the Kernels. Hav is known at every cell
void cell::getKernelsRecursively (const int KernelType)
{
  std::map<int3, Kernel>::iterator iterMap;  //iterator for Kernels
  MVector coarseFaceCenter; //the center of the 2x2 face
  //centerFaces - pointer to small faces, that constitute coarse grain faces

  if (My->Psetup)
    My->outlog<<"\n\nin getKernelsRecursively for center = "<<center
      <<", KernelType = "<<KernelType<<", size = "<<size<<endl;
  if (CellType == 3)   //coarse grain cell, need to find Kernel
  {
    for (int i=0; i<4; i++)
      coarseFaceCenter+=My->MyFMM->centerFaces[KernelType][i]->faceCenter*0.25;
    int3 doubleKey;  //key for the Kernel for this cell
    doubleKey = (coarseFaceCenter - center).ToInt(size*0.5);  //double space
    iterMap = My->MyFMM->newKernels.find (doubleKey);

    if (iterMap!=My->MyFMM->newKernels.end()) //key was found
    {
      iterMap->second.Kx = H_av[0];
      iterMap->second.Ky = H_av[1];
      iterMap->second.Kz = H_av[2];
    }
    else //the key was not found
      if (My->Psetup)//not an error, not all keys are used
        My->outlog<<"the doubleKey = "<<doubleKey
          <<" was not found, Kernel was not calculated"<<endl;
  }  //end of if (CellType==3)
  else   //cell is larger than coarse grain size (=2)
  {
    child0->getKernelsRecursively (KernelType);
    child1->getKernelsRecursively (KernelType);
  }
}


//added by DMA to set parameters for every rectangle
//needs to be called after dt is read (at the end of System::init)
void cell::setRectangleParamRecurs()
{
  if (!rect) //this is not a rectangular cell
  {
    child0->setRectangleParamRecurs();
    child1->setRectangleParamRecurs();
  }
  else       //this is a rectangular cell
  {
    rect->gadt = My->gamma*My->dt*rect->alpha;
    rect->H_rand_scale = sqrt(2*rect->alpha*kB*My->Temp/
      (mu_0*My->gamma*rect->Ms*My->dt*rect->MinCS[0]*rect->MinCS[1]
       *rect->MinCS[2]));
    if(My->MyFMM) rect->setAvAndMomentCoef();   //set AvCoef and MomentCoef
  }
}
  

MVector DispCell::absCenter() const // absolute center, adds disp
{
  return My->Displace(cel->center,disp);
}

void DispCell::SetDispToMinus(DispFace *DF)
{
  disp[0]=-DF->disp[0]; disp[1]=-DF->disp[1]; disp[2]=-DF->disp[2];
  // rel. displ = cell-face = -face usually
}

void DispCell::SetDispFrom(DispCell DC)
{
  disp[0]=DC.disp[0]; disp[1]=DC.disp[1]; disp[2]=DC.disp[2];
  // set disp from another DispCell
}

//fails: // void cell::ListCellsRecursively(ostream &ost)
void cell::ListCellsRecursively()
{  My->outlog<<"@ "<<this<<" is "<<*this<<", has child0="<<child0<<", child1="<<child1<<endl;
if (child0) {My->outlog<<"  child0 is "<<endl; 
  child0->ListCellsRecursively();}
if (child1) {My->outlog<<"  child1 is "<<endl;
  child1->ListCellsRecursively();}
}

//Added by Shuxia to calculate the energy
real cell::energy(MVector Min)
{
  real f = anisCoef*0.5*(Min[1]*Min[1] + Min[2]*Min[2]) - My->H_bias*Min+ 0.5*Min[2]*Min[2] ;
  //   + 0.5*Min[0]*Min[0];// This term with shape anistropy energy, namely, with ex=1/2 Mx* Mx
// cout<<"anisCoef "<<anisCoef<<endl;
//cout<<"Hbias  "<<My->H_bias << " Min "<<Min<<endl;
//real K=2*rect->Hk*rect->Ms*1e6*M_PI*1e-7; Shuxia_07/05
real K=.5*rect->Hk*rect->Ms; //07/05
//real K=.5*(anisCoef+1)*rect->Ms*rect->Ms;
//real K=2*(anisCoef+1)*rect->Ms*rect->Ms*1e6*M_PI*1e-7;
//real normTotEnergy=2*rect->Hk*rect->Ms*1e6*M_PI*1e-7*(1-rect->MinCS[2]*rect->MinCS[2]/(rect->MinCS[2]*rect->MinCS[2]+rect->MinCS[1]*rect->MinCS[1]+rect->MinCS[0]*rect->MinCS[0]));
//no exchange energy?
//exchange energy would give -J*Min*Sum (M_nbrs)
//this gives the energy in kJoules/m^3:
 // real normTotEnergy = f*25./M_PI;
//real normTotEnergy = f/(0.14*Min.Magnitude()*Min.Magnitude());  //This gives the energy normalized by .2*Ms*Ms
real normTotEnergy = f/K;
// real normTotEnergy = f/kT;
//real normTotEnergy = f; //normalized by 25./PI
//cout <<" Norm TotEnergy "<<normTotEnergy<<endl;
//if(Min[2]<0)
//  return (0.- normTotEnergy);
//else
return ( normTotEnergy);
}

