#ifndef CELL_H
#define CELL_H

// #include <ctype.h>
// #include <unistd.h> // for exit?
#include <list>

#include "types.h"
#include "edge.h" // need edge object (lower level) in cell class
#include "MVector.h" // need in cell class
#include "Quaternion.h"
#include "utility.h"

extern int ZeroDisp[3]; // ugly, but works? Defined in main.C?
class mpole;
class FMM;
class int3;
class NearPartner;

#ifdef GRAPHICS
class SoTransform; // so these can be members of cell class
class SoSeparator;
class SoMaterial;
class VectorDisplay;
#endif

class face;
class DispCell;
class Rectang;

class DispFace{
  public:
  int out; //0 or 1, 1 if increasing x points out of cell
                  // 0 if increasing x points into cell
  int owned; //0 if this DispFace is not owned by the cell
             //1 if it is owned by the cell (used to computer moment)
  face* myFace;
  int disp[3]; // displacement in periodic system
  DispFace(){ myFace=NULL; out=0; disp[0]=0; disp[1]=0; disp[2]=0; owned=0;};
  DispFace(face*, int, int* disp1=ZeroDisp);
  DispFace(const DispFace& DF);
  ~DispFace() { myFace = NULL; };
  MVector absCenter();
};


//this operator is needed for map operations
int operator<(const int3&, const int3&); //added by DMAf

class cell{
public:

  /**********data members **********/
  int cellID; // cell ID to identify cells when write into or read from system
  static int NextcellID; 
    //static data member, used to count the total number of cells in the system
  face *septum; //the face that separates the two children for the cell
                    //we add this face to compute the stress on this face
  int longaxis; //the long axis of a cell
  MVector M; //magnetization vector
  Quaternion q; //someday we may want to get rid of M and just store q 
  MVector H_av; //magnetostatic and external fields (DMAh)
  MVector H_exch; //exchange field
  MVector delMpre, delM, H_rand;
  // Material Properties
  real anisCoef;          //=Hk/Ms

#ifdef COUPLING
  // This compiler option allows us to have ferro or anti-ferromagnetic
  // exchange between layers. Can be turned off to save memory.
  cell *InterPartner; // interlayer exchange partner
  real InterCoupling; // coupling to that partner (for small cells)
#endif
  MVector size;  // the size of the cell 
  real diameter2; //used for FMM, added by DMAf
  cell *child0, *child1, *parent;


  Rectang *rect;   //the pointer to a class that contains material and other param.
  mpole *moment; //multipole moment of this cell, added by DMAf
  mpole *SmoothField; //taylor expansion of the smooth field, added by DMAf
  real Energy_stat;  //magnetostatic energy, added by DMAf
  std::list<DispCell> Partners;  //added by DMAf - list of Partners
  std::list<NearPartner> NearPartners;  //added by DMAf - list of NearPartners
          //it will be used for smallest cells

  DispFace cellfacelist[6]; //facelist for a specific cell (DMA)
     	//we use array. Index for direction: -x=0, +x=1, -y=2, +y=3, 
      //-z=4, +z=5 so last bit is sign, next 2 are axis label
  MVector center;//center of the cell in System window, used in display function
  int CellType; //used for averaging, 0 is smallest, 1 is next, ......
       // 101 is (largest) rectangle, 102 is non-rectangle

  int inputWhichChild; // input value to determine which child it is

  /********** constructor and destructor **********/

  cell(cell *parent1);
  cell();
  ~cell();

  /********** member functions **********/
  void check_faces(); //sanity for all the faces in a cell
  MVector RandomField();
  MVector RandomFieldForM();

  void update();  //will be called in evolve()
  void update_q();   //quaternion-based algorithm is used
  void update_m();   //usual algorithm is used
  void updateExch();
              //functions used for FMM:
  void updateMoment(); //update recursively multipole moments, added by DMA
//calculate H_stat at this cell due to Partners and NearPartners,
//return magnetostatic energy, which is stored in cell::Energy_stat  (DMA)
  real updateStat();
  MVector averageMpole(const mpole*) const;   //returns averaged SmoothField
  void CullPartners(); //setup Partners and NearPartners lists, added by DMAf
  void setRectangleParamRecurs();  //to set gadt for this cell
  int Near(const DispCell &);//is this DispCell near to this, added by DMAf
               //adds the field from NearPartner, added by DMAf
  void AddNearField(const NearPartner *);
  //added by DMAk (used for recursive Kernel calculation):
  void getKernelsRecursively (const int);  //to get Kernels from H_av
  void setChargeAtCenterFace (int);
   //to initialize charges of all faces to zero except the faces of
   //coarse 2x2x2 center cell (closest to the center of the root cell)

  void getlongaxis(int&, int&); 
  //return maximum length of all the edges of the cell, the longaxis of the cell
  void parseBuild(); //parse build cells according to the script input file
  void buildRect(MVector Ori); //build a rectangle cell, called within parseBuild()
  void print();
  friend ostream& operator<< (ostream&, cell) ;
  int sane(); //sanity check
  void setCenter();
  void readSize();
  void SetInterPartners();
  //splitPoint was added by DMAg for modification of geometry
  void split(int lgaxis, int splitPoint);
  //split a large into child cells along the longaxis
  void splitRecursively();//split the cell recursively untill the bottom of tree
  void initMRecursively();//set the initial magnetization and quaternion
  void initIDRecursively();//set the array CellFromID
  void initJexRecursively();//set the initial exchange constant  
  // void initGraphics();//initialize the static UpVector
  //the following member functions are for debugging problem at a specific place
  int whichChild();
  int whichSide(MVector);
  inline MVector LowerLeft() {return center-size*0.5;}
  inline MVector UpperRight() {return center+size*0.5;}
  void GetSizeCenter(int dir); // get from children
//   int contains(MVector);
  cell* SubCellContaining(MVector&);
  void ListCellsRecursively();
  //added by Shuxia
  real energy(MVector Min);
#ifdef GRAPHICS
    int display; 
   	//Boolean; tells whether to display in real space and System; 
    	//may be true only for some cells
    int ShowM;
    int ShowH; 
        //Boolean, tells whether to display the M or H vector in certain cells
	VectorDisplay *HDisp;
    SoTransform *myMTransform; //translates to the cell center and rescales 
    SoMaterial *color, *color1;  //not used
    SoSeparator *RealPtr;
   	//pointers to corresponding Open Inventor nodes in the System window 
    // static SoSeparator *UpVector; delete
    	//constant Vector, can be transformed, scaled, and rotated according to
        // specific needs for a cell.
    void change_color(real *material, real *material1, int flag);  //not used
    void MakeSceneTree(SoSeparator*);
#endif //GRAPHICS

};

class DispCell {
public:   
  cell *cel;
  int disp[3];
  
  //constructor
  DispCell(){ cel=NULL; disp[0]=0; disp[1]=0; disp[2]=0; };
  DispCell(cell *c, int3 i){cel=c; disp[0]=i[0]; disp[1]=i[1]; disp[2]=i[2];};
  DispCell(cell *c, int *d){cel=c; disp[0]=d[0]; disp[1]=d[1]; disp[2]=d[2];};
  ~DispCell() {cel = NULL; };

  //member function
  void SetDispToMinus(DispFace *DF);
  void SetDispFrom(DispCell DC);
  MVector absCenter() const;
  DispCell *GetChild0() // creates a new DispCell corresponding to child0
    {return new DispCell(cel->child0,disp);}
  DispCell *GetChild1()
    {return new DispCell(cel->child1,disp);}
  int operator==(const DispCell right)
    {return (cel==right.cel && disp[0]==right.disp[0]
       && disp[0]==right.disp[0] && disp[0]==right.disp[0]);}
};

extern int ZeroDisp[3];

class face{
  public:
  real Jex; //exchange integral between two cells
  real magneticCharge;  //magnetic charge density of the face (DMA)
  face *face0, *face1, *parent; //two child faces for septum and their parent
  DispCell cellList[2]; // cellList[0] is to left (low coordinate).
  DispEdge edgeList[4];
  edge* centerEdge; //the edge which separates two child faces
  int normal;  //0,1,2 for x,y,z
  MVector faceCenter;
  face(int normal1, face* parent1);
  ~face() {face0 = NULL; face1 = NULL; parent = NULL; centerEdge = NULL; };
  //splitPoint was added by DMAg for modified geometry code
  void facesplit(int longa, int splitPoint); 
  //split the face along it's own longaxis
  void getlongedge(int&, int&);
  //return maximum length of the four edges and the direction of the edge
  void sane(); //sanity check function
  void setFaceCenter();
};

class Rectang // was called Rectangle, but conflicts w/Windows function
{
public:
  Rectang ();
  ~Rectang ();
  void setup();   //read lines with parameters
  void setAvAndMomentCoef();   //initialize AvCoef and MomentCoef

     /**************** data members **************/
  real alpha;    //damping constang
  real gadt;     //gamma*alpha*dt
  real Ms;     //saturation magnetization of this rectangle
  real H_rand_scale;   //used to calculate the effective thermal field
  MVector MinCS;       //the size of the smallest cell in this rectangle
  MVector avEasyAxis;       //average easy axis of the uniaxial anisotropy
  MVector M0;             //MVector(0,0,Ms)
  real Hk;                  //average effective anisotropy field
  PowerLaw *AnisPower;      //coefficients in powerlaw anisotropy
  real ExchangeA;                  //exchange constant
  cell *WholeRect; // cell that corresponds to this
#ifdef COUPLING
  cell *PreviousLayer; // previous (below) layer, needed for interlayer exchange 
  real tJ; // interlayer exchange constant with previous layer 
  MVector spacing; // thickness of exchange-inducing spacer layer below this
  static cell *LastLayer; // used to set PartnerLayer; ***STATIC***
     //  refers to last-created rectangle
#endif
  real SpinAx; //spin wave amplitude
  real SpinAy; 
  real SpinPh; //spin wave phase
  MVector kVector; // spin wave vector

  Quaternion q_init;      //used to set initial quaternion for a cell
  mpole *AvCoef[3];      // used to calculate the average magnetostatic field
  mpole *MomentCoef[6];   //used to calculate the magnetic multipole moment
  //Shuxia_1120
  MVector Mp; // Magnetizaition of the pinned layer //Shuxia_Spintronics
  real aJ;   //coefficient of the spintronics term

};

#endif					   
  
