#ifdef GRAPHICS
#include "Lowest.h" // has iostream,math,assert
#include "types.h"
#include "cell.h"
#include "display.h"
#include "System.h"

extern System *My;

//added by DMA (SPIN) for capturing key pressed by the user

void myKeyPressCB(void *userData, SoEventCallback *eventCB){
  // Mourad's was is adapted from Ex. 10-2 of Mentor, PBV changed it
  // based on 10.3.4.Gate.c++

  const SoEvent *event = eventCB->getEvent();
  if (!event->isOfType(SoKeyboardEvent::getClassTypeId())) cout<<"wrong type event:"<<
    endl<<flush; // not needed, Viewer intercepts mouse events
  const SoKeyboardEvent *e = (SoKeyboardEvent *) event; // type cast to keyboard event
  if (!e->isKeyPressEvent(e,SoKeyboardEvent::ANY)) return;
  //(don't respond to key releases!)

  switch (e->getKey()){
    case SoKeyboardEvent::RIGHT_ARROW:
     if (My->Movname[0] != NULL)  //we are reading movie file
       My->direction=My->FORWARD;
     else
     {
       My->direction=My->COMPUTE;
       // cout<<"right_arrow is pressed, continue"<<endl;
     }
     break;
   case SoKeyboardEvent::LEFT_ARROW:
     if (My->Movname[0] != NULL) My->direction=My->BACKWARD; // only if movie read
     break;
   case SoKeyboardEvent::DOWN_ARROW:
     cout<<"program is paused, press right_arrow to continue"<<endl;
     My->direction=My->STOP;
     break;
   case SoKeyboardEvent::PERIOD: // or >
     My->numberSkipFrames*=2;
     cout<<"number of frames to skip:"<<My->numberSkipFrames<<endl;
     break;
   case SoKeyboardEvent::COMMA:
     if (My->numberSkipFrames>=2)
       My->numberSkipFrames/=2;
     cout<<"number of frames to skip:"<<My->numberSkipFrames<<endl;
     break;
   case SoKeyboardEvent::PAD_2: // decrease selected vector(y)
     My->H_bias[1]-=My->Increment;
     break;
   case SoKeyboardEvent::PAD_4: // decrease x
     My->H_bias[0]-=My->Increment;
     break;
    case SoKeyboardEvent::PAD_6:
     My->H_bias[0]+=My->Increment;
     break;
   case SoKeyboardEvent::PAD_8:
     My->H_bias[1]+=My->Increment;
     break;
     case SoKeyboardEvent::PAD_ADD: // increase selected vector(z)
     My->H_bias[2]+=My->Increment;
     break;
   case SoKeyboardEvent::PAD_SUBTRACT:
     My->H_bias[2]-=My->Increment;
     break;
// Repeat pad keys for laptop, to avoid having to hit FN key
   case SoKeyboardEvent::K: // pad2, decrease selected vector(y)
     My->H_bias[1]-=My->Increment;
     break;
   case SoKeyboardEvent::U: // pad4, decrease x
     My->H_bias[0]-=My->Increment;
     break;
    case SoKeyboardEvent::O: // pad6
     My->H_bias[0]+=My->Increment;
     break;
   case SoKeyboardEvent::NUMBER_8:
     My->H_bias[1]+=My->Increment;
     break;
     case SoKeyboardEvent::SLASH: // PAD_ADD increase selected vector(z)
     My->H_bias[2]+=My->Increment;
     break;
	 case SoKeyboardEvent::SEMICOLON: // was PAD_SUBTRACT:
     My->H_bias[2]-=My->Increment;
     break;

   case SoKeyboardEvent::Q:
     My->direction=My->EXIT;
     break;
   case SoKeyboardEvent::P:
     My->Repeat = !My->Repeat;
     break;
   case SoKeyboardEvent::MINUS:
     if (My->surfaceMaterial->transparency[0]>0.02)
       My->surfaceMaterial->transparency=My->surfaceMaterial->transparency[0]-0.02;
     cout<<"decreasing the transparency, it is now "
       <<My->surfaceMaterial->transparency[0]<<endl;
     break;
   case SoKeyboardEvent::EQUAL: // oddly, this is plus sign
     if (My->surfaceMaterial->transparency[0]<0.98)
     My->surfaceMaterial->transparency=My->surfaceMaterial->transparency[0]+0.02;
     cout<<"increasing the transparency, it is now "
       <<My->surfaceMaterial->transparency[0]<<endl;
     break;
   case SoKeyboardEvent::HOME:
     My->direction=My->HOME;
     break;
  default:
    cout<<"incorrect key was pressed"<<endl;
    break;
  }
}
  
/*-----------------------------------------------------------------
Constructor VectorDisplay::VectorDisplay takes 6 arguments:
  1. char *label - label of the Vector;
  2. MVector VectorTranslation - translation of the origin of vector visualization;
  3. real multFactor - maximum value of this vector (used to scale it down);
  4. int colorR, colorG, colorB - color of the vector in RGB format, default
value of the color is white (RGB = [1,1,1]); 

Constructor - creates all the tree. After constructor you can use getNode() member
function that returns SoSeparator that contains all the tree for this vector 
to add it to another tree

           EXAMPLE: to add H_total to realTree:
    VectorDisplay *H_total_display = new VectorDisplay("H_tot",
           MVector_zero, My->fieldScale); //create it
    realTree->addChild(H_total_display->getNode());  //add it to realTree

..in update():
    H_total_display->update(H_total);  //update it

  NOTE: each time H_total changes you need to call H_total_display->update(H_total);

 multFactor (data member of VectorDisplay class) is used to scale
 down the magnitude of the vector in OI.
-------------------------------------------------------------------*/


VectorDisplay::VectorDisplay(const char *labelIn, int WhichCompsIn, int WhereLabel1,
   const MVector &VectorTranslation, real multFactorIn, 
  float colorR, float colorG, float colorB)
{  // Sets up scene tree as follows: VectorNode
	// Matl  OriginTr  TextSep             VectTrans  UpVector
	//                 TextTr VectorText
	// so VectorTextTransform and VectorTranform are relative to origin.
	
  //deleted long ago: use VectorFieldScale to scale the visualization picture
  multFactor = multFactorIn;
  WhichComps=WhichCompsIn;
  WhereLabel=WhereLabel1;
  strcpy(label,labelIn); // copy to class member so it will be available later
  VectorNode = new SoSeparator; //node that contains all the tree structure
  assert (VectorNode);
  SoMaterial *VectorMaterial = new SoMaterial; //material for the vector
  assert (VectorMaterial);
  assert ((colorR<=1.) && (colorG<=1.) && (colorB<=1.0)); //sanity check
  VectorMaterial->diffuseColor.setValue(colorR, colorG, colorB);
  VectorNode->addChild(VectorMaterial);

//to translate the origin of the Vector and the label
  originTransform = new SoTransform;
  assert (originTransform);
  originTransform->translation.setValue(VectorTranslation.get_val());
  VectorNode->addChild(originTransform);

// we assume SoFont node was already added in the tree. If not, add here.
//put the label
  SoSeparator *VectorTextSep = new SoSeparator;
  assert (VectorTextSep);
  VectorTextTransform = new SoTransform;
  assert (VectorTextTransform);
  VectorText = new SoText2;
  assert (VectorText);
  VectorTextSep->addChild(VectorTextTransform);
  VectorTextSep->addChild(VectorText);
  VectorNode->addChild(VectorTextSep);

//transformation of the UpVector to get the Vect
  VectorTransform = new SoTransform;
  assert (VectorTransform);
  VectorNode->addChild(VectorTransform);
  VectorNode->addChild(My->UpVector);
}

//this function updates the value of the vector and the origin of the vector
void VectorDisplay::updateVector(const MVector &newOrigin, const MVector &newVector)
{ // FUNCTION IS NOT USED! & not debugged.
  originTransform->translation.setValue(newOrigin.get_val());
  updateVector(newVector);
} // end function w/ 2 arguments

//this function updates the value of the vector but does not change the origin
//it should be called anytime the vector changes (example in cell::update function)
void VectorDisplay::updateVector(const MVector &newVector) //newVector has units kA/m
{
  Quaternion qRotation;
  real a, x, y, z;
  qRotation.rotateTo(newVector);
  qRotation.CopyTo(a,x,y,z);
  VectorTransform->rotation.setValue(x,y,z,a);
  real scaleFact = newVector.Magnitude()*multFactor;
  VectorTransform->scaleFactor.setValue(scaleFact, scaleFact, scaleFact);
  char VecText[30]; // plenty long
  switch(WhichComps) { // which components to write
	  case -1: sprintf(VecText,"%s",label);  // just label
		break;
	  case 0: case 1: case 2:// only one comp, write label+newVector[WhichComps]
        sprintf(VecText,"%s=%4.2e",label,newVector[WhichComps]); 
		break;
	  case 3: // all 3 comps
        sprintf(VecText,"%s=(%4.2,e%4.2e,%4.2e)",label,
			newVector[0],newVector[1],newVector[2]); 
		break;
	  default: My->outlog<<"error in WhichComps"<<endl; MYEXIT;
  } // end switch
  VectorText->string=VecText;
  MVector TextTrans;
  if (WhereLabel) // 1 means tip
	  TextTrans = newVector*multFactor+MVector (0.05, 0.05, 0.05);
  else // tail
	  TextTrans = MVector(0.05, 0.05, 0.05); // same extra disp 0.05, for now!
  VectorTextTransform->translation.setValue(TextTrans.get_val());
} // end fn. updateVector

//this function returns the node to insert it into a tree
SoSeparator *VectorDisplay::getNode()
{
  return VectorNode;
}
  
/*-------------------------------------------------------------------------
Constructor QuaternionDisplay::QuaternionDisplay takes 7 arguments:
  1. char *label - label of the Quaternion;
  2. MVector QuatTranslation - translation of the origin of the visualization;
  3. real radiusOfArc - radius of the arc used in visualization (default value is 0.5); 
  4. int numArcPoints - number of control points used to create the arc
     (default value is 10);
  5. int colorR, colorG, colorB - color of the visualization in RGB format, default
value of the color is white (RGB = [1,1,1]);

Constructor - creates all the tree. After constructor you can use getNode() member
function that returns SoSeparator that contains all the tree for this Quaternion 
to add it to another tree

           EXAMPLE: to add q_rot to realTree:
    QuaternionDisplay *q_rot_display = new QuaternionDisplay("q_rot", 
           MVector_zero); //create it
    realTree->addChild(q_rot_display->getNode());  //add it to realTree

..in update():
    q_rot_display->update(q_rot);  //update it

  NOTE: each time q_rot changes you need to call q_rot_display->update(q_rot);
-------------------------------------------------------------------------*/


QuaternionDisplay::QuaternionDisplay(const char *label,
  const MVector &qTranslation, real radiusOfArc, int numberArcPoints,
  int colorR, int colorG, int colorB)
{
//initialization of member data
  Radius = radiusOfArc;  //setting the radius of the arc
  numPoints = numberArcPoints;  //setting the number of control points

//memory allocation for all pointers
  QuaternionNode = new SoSeparator; //node that contains all the tree structure
  assert (QuaternionNode);
  arrowCoords = new SoCoordinate3;
  assert (arrowCoords);
  controlPts = new SoCoordinate3;
  assert (controlPts);
  sectorCoords = new SoCoordinate3;
  assert (sectorCoords);
  QuaternionTransform = new SoTransform;
  assert (QuaternionTransform);
  QuaternionTextTransform = new SoTransform;
  assert (QuaternionTextTransform);

  SoMaterial *QuaternionMaterial = new SoMaterial; //material of the vector
  assert (QuaternionMaterial);
  assert ((colorR<=1.) && (colorG<=1.) && (colorB<=1.0)); //sanity check
  QuaternionMaterial->diffuseColor.setValue(colorR, colorG, colorB);
  QuaternionNode->addChild(QuaternionMaterial);

  QuaternionTransform->translation.setValue(qTranslation.get_val());
  QuaternionNode->addChild(QuaternionTransform);

  QuaternionNode->addChild(makeArc());

//I assume that SoFont node was already added in the tree. If not we need to
//uncomment the following lines:
/*
  SoFont *QuaternionFont = new SoFont;
  assert(QuaternionFont);
  QuaternionFont->name.setValue("Times-Roman");
  QuaternionFont->size.setValue(10.0);
  QuaternionNode->addChild(QuaternionFont);
*/
  QuaternionNode->addChild(QuaternionTextTransform);

  SoText2 *QuaternionText = new SoText2;
  assert (QuaternionText);
  QuaternionText->string = label;
  QuaternionNode->addChild(QuaternionText);
}

SoSeparator *QuaternionDisplay::makeArc (real angle)
{
  SoSeparator *curveSep = new SoSeparator;
  curveSep->ref();

  SoComplexity *complexity = new SoComplexity; //needed for NurbsCurve
  SoNurbsCurve *curve = new SoNurbsCurve;
  complexity->value=0.8;
  curveSep->addChild(complexity);

  curve->numControlPoints = numPoints;

  curve->knotVector.set1Value(0,0);
  curve->knotVector.set1Value(1,0);
  curve->knotVector.set1Value(2,0);
  for (int i = 0; i<=numPoints - 3; i++)
    curve->knotVector.set1Value(3+i, i);
  curve->knotVector.set1Value(numPoints+1, numPoints-3);
  curve->knotVector.set1Value(numPoints+2, numPoints-3);
  curve->knotVector.set1Value(numPoints+3, numPoints-3);

  curveSep->addChild (controlPts);//controlPts is set in setArcCoords
  curveSep->addChild (curve);

//setup arrow lines:
  SoLineSet *arrowLineSet = new SoLineSet;
  assert (arrowLineSet);

  setArcCoords(angle); //sets all controlPts and arrowCoords

  arrowLineSet->numVertices.set1Value(0,2);
  arrowLineSet->numVertices.set1Value(1,2);
  arrowLineSet->numVertices.set1Value(2,2);
  arrowLineSet->numVertices.set1Value(3,2);
  curveSep->addChild(arrowCoords);
  curveSep->addChild(arrowLineSet);

//add sector lines:
  SoLineSet *sectorLineSet = new SoLineSet;
  assert (sectorLineSet);
  sectorLineSet->numVertices.set1Value(0, 2);
  sectorLineSet->numVertices.set1Value(1, 2);
  curveSep->addChild(sectorCoords);
  curveSep->addChild(sectorLineSet);

  return curveSep;
}


//this utility function sets the coordinates of the arc (control points
//for the curve and coordinates of the arrow) (it is used in two places)

void QuaternionDisplay::setArcCoords(real arcAngle)
{
  real dAlpha, xCoord, yCoord; //dummies

  dAlpha = arcAngle/(numPoints-1); //angle increment for creating controlPoints

  for (int i = 0; i<numPoints; i++)
    controlPts->point.set1Value(i, Radius*cos(dAlpha*i), Radius*sin(dAlpha*i), 0.0);

//now set the coordinates of the arrow
  real Alpha = arcAngle - dAlpha;
  xCoord = Radius*cos(arcAngle);
  yCoord = Radius*sin(arcAngle);

  arrowCoords->point.set1Value(0, 0.92*Radius*cos(Alpha),
                               0.92*Radius*sin(Alpha), 0.0);
  arrowCoords->point.set1Value(1, xCoord, yCoord, 0.0);
  arrowCoords->point.set1Value(2, xCoord, yCoord, 0.0);
  arrowCoords->point.set1Value(3, 1.08*Radius*cos(Alpha),
                               1.08*Radius*sin(Alpha), 0.0);
  arrowCoords->point.set1Value(4, xCoord, yCoord, 0.0);
  arrowCoords->point.set1Value(5, Radius*cos(Alpha),
                               Radius*sin(Alpha), 0.08*Radius);
  arrowCoords->point.set1Value(6, xCoord, yCoord, 0.0);
  arrowCoords->point.set1Value(7, Radius*cos(Alpha),
                               Radius*sin(Alpha), -0.08*Radius);

//set the coordinates of the sector lines
  sectorCoords->point.set1Value(0, 0.0, 0.0, 0.0);
  sectorCoords->point.set1Value(1, Radius, 0.0, 0.0);
  sectorCoords->point.set1Value(2, 0.0, 0.0, 0.0);
  sectorCoords->point.set1Value(3, Radius*cos(arcAngle),
    Radius*sin(arcAngle), 0.0);
}


//this function updates the visualization of the quaternion 
//is should be called anytime the quaternion changes (example in cell::update function)
void QuaternionDisplay::updateQuaternion(const Quaternion &newq)
{
  real qAngle, qX, qY, qZ, x0, x1, x2, x3;
  Quaternion qAxis, qRotation;

  qAxis = newq.GetAngleAndAxis_rad();
  qAxis.CopyTo(qAngle, qX, qY, qZ);  //qAngle is used to update the arc
  
//qRotation is used to update the transformation of the arc
  qRotation.rotateTo (MVector(qX, qY, qZ));
  qRotation.CopyTo (x0, x1, x2, x3);
  QuaternionTransform->rotation.setValue(x1, x2, x3, x0);

  setArcCoords(qAngle);  //set all control points and arrow coordinates

  QuaternionTextTransform->translation.setValue(1.1*Radius*cos(qAngle),
      1.1*Radius*sin(qAngle),0.);
}

//this function returns the node to insert it into a tree
SoSeparator *QuaternionDisplay::getNode()
{
  return QuaternionNode;
}


//not used in the code
/*
void cell::change_color(real *material, real *material1, int flag){
    color1->diffuseColor.setValue(material1); 
    if(flag){
	if( child0) {
	    color->diffuseColor.setValue(material1);
	} else { // leaf node, color differently 
	    color->diffuseColor.setValue(material);    
	}
    } else{
	color->diffuseColor.setValue(material);
    }
    if(child0!=NULL) child0->change_color(material, material1, flag);
    if(child1!=NULL) child1->change_color(material, material1, flag);
}
*/

void cell::MakeSceneTree(SoSeparator *RealParent) 
   //display this cell, and its children recursively, in the real-space window.
{   //cout<<"Starting MakeSceneTree"<<endl;
    static real norms[6][3] = { {-1,0,0},{0,-1,0},{0,0,-1},{1,0,0},{0,1,0},
                                 {0,0,1} }; 
    RealPtr=new SoSeparator; //corresponding node in real space 
    assert(RealPtr);
    RealParent->addChild(RealPtr);
    if(ShowM) //added so display and ShowM work independentlyma
	{
	//the following part deals with the magnization vector magVector
      SoSeparator *magVector;
      magVector = new SoSeparator;
      assert(magVector);

      //set the color for magVector
      SoMaterial *mymagMaterial;
      mymagMaterial = new SoMaterial;
      assert(mymagMaterial);
      mymagMaterial->diffuseColor.setValue(1.0,0.0,0.0);
   
      //set linewidth of magVector
      SoDrawStyle *mymagDrawStyle=new SoDrawStyle;
      assert(mymagDrawStyle);
      mymagDrawStyle->lineWidth.setValue(1.0);

      //set magVector to the center of the cell
      myMTransform = new SoTransform;
      assert (myMTransform);
      myMTransform->translation.setValue(this->center.get_val());

      magVector->addChild(mymagMaterial);
      magVector->addChild(mymagDrawStyle);
      magVector->addChild(myMTransform);
      magVector->addChild(My->UpVector);

      //now actually put the magVector into the real-space graph
     // cout<<"About to add M vector"<<endl;
      RealPtr->addChild(magVector);	

	} //end of ShowM added by CM 6/04

    if(ShowH)
	{
	   HDisp = new VectorDisplay("H",-1,1,center, My->fieldScale, 0, 0.5, 0);
        assert (HDisp); // 5 is fudge
        RealPtr->addChild(HDisp->getNode()); 
	}


    if (display) //actually display cell, as "surface", a set of square faces.
    {   //ShowM and magnetization components now in if (ShowM)
     SoSeparator *surface;
     surface = new SoSeparator;
     assert(surface);

     //now set the draw style of the face to be line
     SoDrawStyle *myfaceDrawStyle;
     myfaceDrawStyle=new SoDrawStyle;
     assert(myfaceDrawStyle);
     myfaceDrawStyle->style=SoDrawStyle::FILLED;

     //for now, try using the SAME face set in the real space window
      SoCoordinate3 *myCoords;
      myCoords = new SoCoordinate3; 
      assert(myCoords);
     // this will hold coords for all faces of cell
      SoTriangleStripSet *myStripSet; 
     // each strip is a face, strip set is all faces
      myStripSet = new SoTriangleStripSet;
      assert (myStripSet);
      SoNormal *myNormals;
      myNormals = new SoNormal;
      assert(myNormals);
      int numVert=0; //the number of vertices inserted in myCoords, so far
      int numFace=0; //number of faces so far
      //loop faces, put in face set (changed by DMA)
      for (int i=0; i<6; i++)
      {
        myNormals->vector.set1Value(numFace,
                                   norms[cellfacelist[i].myFace->normal+3*(cellfacelist[i].out)]);
        //sets normal based on face:://low bit=axis, high bit=orientation
        //numVert is the number of vertices inserted in myCoords, so far
        myCoords->point.set1Value(numVert++,
                  (My->Displace(cellfacelist[i].myFace->edgeList[0].vertexListAbsolute(0),
                                    cellfacelist[i].disp)).get_val());
        myCoords->point.set1Value(numVert++,
                  (My->Displace(cellfacelist[i].myFace->edgeList[0].vertexListAbsolute(1),
                                    cellfacelist[i].disp)).get_val());
        myCoords->point.set1Value(numVert++,
                  (My->Displace(cellfacelist[i].myFace->edgeList[2].vertexListAbsolute(0),
                                    cellfacelist[i].disp)).get_val());
        myCoords->point.set1Value(numVert++,
                  (My->Displace(cellfacelist[i].myFace->edgeList[2].vertexListAbsolute(1),
                                    cellfacelist[i].disp)).get_val());
        myStripSet->numVertices.set1Value(numFace,4);
        numFace++;
        //set 4 vertices
       } //end of loop over faces

      surface->addChild(myfaceDrawStyle);
      surface->addChild(myNormals); // actually add stripset
      surface->addChild(myCoords);
      surface->addChild(myStripSet);

      //now actually put the surface into the real-space graph
      RealPtr->addChild(surface); 

    } //end of if(display)
     
  //now call recursively for children--BEFORE displaying this cell, because we may want to USE children in display of this cell
   else{ 
      assert(child0); //check if it is a bottom cell
      child0->MakeSceneTree(RealPtr);
      child1->MakeSceneTree(RealPtr);
    } 
 
}//end of MakeSceneTree()

void System::setupGraphics()
{
        cout<<"starting setupGraphics"<<endl;
  #ifdef WIN32 // SoWin uses different class and function names from SoXt:
	HWND myWindow=SoWin::init("main"); // cannot create any "So" objects until now?
  #else
	Widget myWindow=SoXt::init("main");
  #endif
        if(myWindow==NULL) 
		  {cout<<"Couldn't create window"<<endl; exit(1);}
        rootReal = new SoSeparator;		
		cout<<"created rootReal at "<<rootReal<<endl;
        assert(rootReal);
        rootReal->ref();

  SoEventCallback *myEventCB = new SoEventCallback; //added by DMA for key press
  assert(myEventCB); // 
  cout<<"Constructed myEventCB at "<<myEventCB<<endl;
  myEventCB->addEventCallback(SoKeyboardEvent::getClassTypeId(), myKeyPressCB, rootReal);
  rootReal->addChild(myEventCB);
//inserting camera
  SoOrthographicCamera *myCamera = new SoOrthographicCamera;
  assert (myCamera);
  //the following line is to change orientation of the camera
//  myCamera->orientation.setValue(SbRotation(SbVec3f(0.0,1.0,0.0), 1.57));
  rootReal->addChild(myCamera);

  #ifdef WIN32
        SoWinExaminerViewer *myViewer=new SoWinExaminerViewer(myWindow);
  #else
        SoXtExaminerViewer *myViewer=new SoXtExaminerViewer(myWindow);
  #endif
        SoOrthographicCamera *orthoView = new SoOrthographicCamera;
        myViewer->setCameraType(orthoView->getClassTypeId());
        myViewer->setSceneGraph(rootReal);
		myViewer->setTitle((runID+": U of Alabama Micromagnetic Simulator").c_str());

        surfaceMaterial = new SoMaterial;
        assert(surfaceMaterial);
        surfaceMaterial->diffuseColor.setValue(0.05,0.05, 0.05);
        surfaceMaterial->transparency = .82;
        rootReal->addChild(surfaceMaterial);

        SoNormalBinding *myNormalBinding;
        myNormalBinding=new SoNormalBinding;
        assert(myNormalBinding);
        myNormalBinding->value=SoNormalBinding::PER_PART;
        // means per triangle, with TriangleStripSets
        rootReal->addChild(myNormalBinding);
        // we hope this setting applies to whole tree

        rootReal->addChild(new SoDirectionalLight);

        // construct some stuff for MakeSceneTree recursion:
	SoFont *myFont;
        myFont = new SoFont;
        assert(myFont);
        myFont->name.setValue("Times-Roman");
        myFont->size.setValue(20.0);
        rootReal->addChild(myFont);

  UpVector=new SoSeparator; //initialize data member
  assert(UpVector); // will be unit vector in z direction, with cone point.
  SoCoordinate3 *myUpCoords;
  myUpCoords = new SoCoordinate3;
  assert(myUpCoords);
  SoLineSet *myLineSet;
  myLineSet = new SoLineSet;
  assert(myLineSet);
  myUpCoords->point.set1Value(0,0,0,0);
  myUpCoords->point.set1Value(1,0,0,1.0);
  myLineSet->numVertices.set1Value(0,2);
  //set up arrow line coords and LineSet 
  SoCone *myCone = new SoCone;
  myCone->height = 0.3;
  myCone->bottomRadius = 0.1;
  SoTransform *myConeTransform = new SoTransform;
  assert (myConeTransform);
  myConeTransform->translation.setValue(0,0,0.9); 
  myConeTransform->rotation.setValue(1/sqrt(2.0),0,0,1/sqrt(2.0));
  UpVector->addChild(myUpCoords);
  UpVector->addChild(myLineSet);
  UpVector->addChild(myConeTransform);
  UpVector->addChild(myCone);

/* previous code: arrows are used to display magnetization vectors
  SoCoordinate3 *myArrowCoords[4];
  SoLineSet *myArrowLineSet[4];
  for (int i = 0; i<4; i++)
  {
    myArrowCoords[i] = new SoCoordinate3;
    assert(myArrowCoords[i]);
    myArrowLineSet[i] = new SoLineSet;
    assert(myArrowLineSet[i]);
   }
  //set up arrow line 0
  myArrowCoords[0]->point.set1Value(0,0.06,0,0.44);
  myArrowCoords[0]->point.set1Value(1,0,0,.5);
  myArrowLineSet[0]->numVertices.set1Value(0,2);
  //set up arrow line 1
  myArrowCoords[1]->point.set1Value(0,-0.06,0,0.44);
  myArrowCoords[1]->point.set1Value(1,0,0,.5);
  myArrowLineSet[1]->numVertices.set1Value(0,2);
  //set up arrow line 2
  myArrowCoords[2]->point.set1Value(0,0,0.06,0.44);
  myArrowCoords[2]->point.set1Value(1,0,0,.5);
  myArrowLineSet[2]->numVertices.set1Value(0,2);
  //set up arrow line 3
  myArrowCoords[3]->point.set1Value(0,0,-0.06,0.44);
  myArrowCoords[3]->point.set1Value(1,0,0,.5);
  myArrowLineSet[3]->numVertices.set1Value(0,2);
  //add M coordinates and LineSet to UpVector
  UpVector->addChild(myUpCoords);
  UpVector->addChild(myLineSet);
  //add 4 arrow lines to UpVector
  for (i = 0; i<4; i++)
  {
    UpVector->addChild(myArrowCoords[i]);
    UpVector->addChild(myArrowLineSet[i]);
   }
*/
        //cout<<"About to call MakeSceneTree"<<endl;
        root->MakeSceneTree(rootReal);
        SoDrawStyle *fieldDrawStyle = new SoDrawStyle;
        assert (fieldDrawStyle);
        fieldDrawStyle->lineWidth.setValue(2);
        rootReal->addChild(fieldDrawStyle);
     

		if(ShowWatchedMStatic) 
		{
			MStaticField = new VectorDisplay("Hmag",-1,1,
          WatchedCell->center, My->fieldScale, 1, 1, 0);
        assert (MStaticField);
        rootReal->addChild(MStaticField->getNode()); 
		}

        if(ShowWatchedHk) 
		{
		  AnisField = new VectorDisplay("Hanis",-1,1,
          WatchedCell->center, My->fieldScale, 0, 0.5, 0.5);
          assert (AnisField);
          rootReal->addChild(AnisField->getNode()); 
		}

		if(ShowWatchedExch) 
		{
			ExchField = new VectorDisplay("Hex",-1,1,
            WatchedCell->center, My->fieldScale, 0.5, 0.0, 0.5);
            assert (ExchField);
            rootReal->addChild(ExchField->getNode()); 
		}
		if(ShowWatchedTot) 
		{
			TotField = new VectorDisplay("Htot",-1,1,
              WatchedCell->center, My->fieldScale, 0, 0, 0);
            assert (TotField);
            rootReal->addChild(TotField->getNode()); 
		}

        myViewer->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_BLEND);
        myViewer->setBackgroundColor(SbColor(1.0,1.0,1.0));
        myViewer->show();
        myViewer->viewAll();

  #ifdef WIN32
        SoWin::show(myWindow);
  #else
        SoXt::show(myWindow);
  #endif
        //cout<<"exiting setupGraphics"<<endl;

}

#endif //GRAPHICS