
#ifdef HAVE_CONFIG_H // from COIN example
#include "config.h"
#endif // HAVE_CONFIG_H

#ifdef WIN32 // these are COIN versions of names
  #include <Inventor/Win/SoWin.h>
  #include <Inventor/Win/SoWinRenderArea.h>
  #include <Inventor/Win/viewers/SoWinPlaneViewer.h>
  #include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#else
  #include <Inventor/Xt/SoXt.h>
  #include <Inventor/Xt/SoXtRenderArea.h>
  #include <Inventor/Xt/viewers/SoXtPlaneViewer.h>
  #include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#endif
#include <Inventor/nodes/SoCylinder.h>
// #include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodekits/SoShapeKit.h>
#include <Inventor/actions/SoLineHighlightRenderAction.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText2.h>
// #include <Xm/Form.h>
#include <Inventor/SbColor.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoOrthographicCamera.h>

#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoNurbsCurve.h> // for quaternionDisplay

#include "types.h"

class MVector;
class Quaternion;

#ifndef VECTORDISPLAY
#define VECTORDISPLAY

class VectorDisplay
{
public:
  VectorDisplay (const char *labelIn, int WhichCompsIn, int WhereLabel1,
	     const MVector &VectorTranslation, real multFactorIn,
         float colorR=1., float colorG=1., float colorB=1.);
  static void initVectorSep();  //initialize static VectorSep
  void updateVector(const MVector &newOrigin, const MVector &newVector);
  void updateVector(const MVector &newVector);//no change in the origin
  SoSeparator *getNode(); //returns VectorNode

private:
  char label[10]; // text label, like "H", generous length limit
  real multFactor;  //unit for displaying this vector
  // The following variable has been replaced by System::UpVector.
  // It worked on SGI, but had to be constructed outside any function,
  // before COIN is initialized, which caused errors in COIN.
  // static SoSeparator *VectorSep;  //line with arrows at the end
  SoTransform *VectorTransform; //rotate and scale VectorSep
  SoTransform *VectorTextTransform; //transformation of the label
  SoText2 *VectorText;
  SoTransform *originTransform; //origin of the vector
  SoSeparator *VectorNode; //node that contains all the tree structure
  int WhichComps; // -1=none, 012=xy or z, 3=write all 3
  int WhereLabel; // 0 for tail, 1 for tip
};

class QuaternionDisplay
{
public:
  QuaternionDisplay (const char *label, const MVector &QuatTranslation,
         real radiusOfArc=0.5, int numArcPoints=10,
         int colorR=1., int colorG=1., int colorB=1.);
  void updateQuaternion(const Quaternion &newQuaternion);
  SoSeparator *getNode();  //returns QuaternionNode 

private:
  SoSeparator *makeArc(real angle=0); //utility function to create an arc
  void setArcCoords(real angle); //utility function to set the coordinates of the arc

  real Radius;  //radius of the arc (default value is 0.5)
  SoCoordinate3 *arrowCoords; //coordinates of the arrow
  SoCoordinate3 *controlPts; //control points to create the arc
  SoCoordinate3 *sectorCoords; //coordinates for sector lines
  int numPoints;  //number of controlPoints in the arc
  SoTransform *QuaternionTransform; //transformation of the visualization
  SoTransform *QuaternionTextTransform; //tranformation of the label
  SoSeparator *QuaternionNode; //node that contains all the tree structure
};

#endif
