#ifndef MPOLE_H
#define MPOLE_H

class MVector;
class System;

extern System *My;

#include "types.h"
#include <fstream> // for ofstream,ifstream

class mpole
{
  friend std::ostream& operator<<(std::ostream&, const mpole&);
  friend std::istream& operator>>(std::istream&, mpole&);
  friend mpole operator*(const mpole&, const real&);//multiplication by a real number
public:
  real *values; // actual numbers Q_n_i as in paper
  int order; // order of the mpole

       //member functions:
  mpole(int);  //constructor, allocate memory for values
  mpole(const mpole&);  //copy-constructor
  ~mpole();  //destructor, deallocate memory occupied by values
  real compare (const mpole *);//returns the difference between two mpoles
  real& GetValue(int, int, int); //to get value[int][int][int]
  const real& GetValue(int, int, int) const; //constant version of the previous function
  void SetToZero(); //set values to be zero
  void print(char *, std::ofstream &); //to output values
  void SetToSimplePowers(const MVector &);
  //this[n] = cx^nx*cy^ny*cz^nz
  void SetToPowers(const MVector &); //this[n] = cx^nx*cy^ny*cz^nz/n!
  void SetToVector(const MVector &); 
  //set values = {0, c[0], c[1], c[2], 0, 0, 0, ..}
  void SetToDeriv(const MVector &);//set values to D[r] see paper
    //set values_p = Sum_t{W(t)*(-.5*t)^p/p!}  (DMApf)
  void SetToPowerSym (const int*);
  void rescale (real factor); //divide values by factor^(n+1)
  void AddTaylor(const mpole *, const MVector &);//add Taylor expansion
  void AddUpConv(const mpole *, const mpole *);//up-convolution
  void AddDownConv(const mpole *, const mpole *);//down-convolution
  void AddShift(const mpole*, const MVector &);//shift the origin of multipole moment
  void invert(); //values = (-1)^order*values

       //overloaded operations
  real operator*(const mpole& right) const;  //dot-product
  mpole operator/(const real&) const;  //division by a real number
  mpole operator+(const mpole&) const; //mpole addition
  const mpole & operator+= (const mpole&);
  const mpole & operator*= (const real &);
  const mpole & operator= (const mpole&);  //assignment
};

#endif

