#include "pulse.h"
#include "Lowest.h" // has iostream, math,..
#include "units.h"

#include "System.h" // for My
extern System *My;

real PulsedField::RiseFunction(real t)
{
   real x;
   switch (PulseType)
   {
      case 1: x=(t-StartTime)/RiseTime;
		break;
      case 2: x=(exp(t-StartTime)-1)/(exp(RiseTime)-1);
		break;
      default: cout<<"error in the choice of pulse type at t="<<t<<"\n";
	       exit(1);
   }
   return x;
}

real PulsedField::FallFunction(real t)
{
   real x;
   switch (PulseType)
   {
      case 1: x=(StartTime+TotalTime-t)/FallTime;
		break;
      case 2: x=(exp(-t+StartTime+RiseTime+PeakTime)-exp(-FallTime))/(1-exp(-FallTime));
		break;
      default: cout<<"error in the choice of pulse type%%%\n";
	       exit(1);
   }
   return x;
}

void PulsedField::Read(ifstream *File)
{
   real Hx,Hy,Hz;
   *File>>PulseType>>Hx>>Hy>>Hz>>StartTime>>RiseTime>>FallTime>>PeakTime;
   //Hx *= KOe_to_kA_per_meter;
   //Hy *= KOe_to_kA_per_meter;
   //Hz *= KOe_to_kA_per_meter;
   TotalTime = RiseTime+PeakTime+FallTime;
   Hmax = MVector(Hx, Hy, Hz);
}

MVector PulsedField::GetValue(real time)
{
   real ampl; // normalized so max is 1
   if ((time > StartTime) && (time < StartTime+TotalTime))
   {
      if (time < StartTime+RiseTime)
         ampl = RiseFunction(time);
      else if (time < StartTime+RiseTime+PeakTime)
         ampl = 1.0;
      else
         ampl = FallFunction(time);
   }
   else
      ampl = 0;
   // My->outlog<<"about to return from PulsedField:GetValue, ampl="
        // <<ampl<<", Hmax="<<Hmax<<endl;
   return (Hmax*ampl);
}
